require "klnk_forward"
local _, features = pcall(require, "features")
local mp = require('msgpack')
local selfInfo = {}
local pl = require "pl.pretty"
local thread = require "coaf.thread"
local md5 = require "md5".sumhexa
local klnkservice = KLNK_FORWARD.createNew()
local klnkSerial, api = ""
local klnkOnline = false
local _M = {
    hashProxy = "vVYra34TX91jCT4Hknh1ttGuXvuDI25G",
}

local function GetDeviceMode(DeviceMode)
    if DeviceMode == "" then
        local _done, features = pcall(require, "oem")
        if _done then
            if type(features) ~= "table" then
                DeviceMode = _G.OEM_TYPE
            else
                DeviceMode = features.OEM_TYPE
            end
        end
        if DeviceMode == "" then
            _done, features = pcall(require, "features")
            if _done then
                if type(features) ~= "table" then
                    DeviceMode = _G.PRODUCT_ID
                else
                    DeviceMode = features.PRODUCT_ID
                end
            end
        end
    end
    return DeviceMode
end
local DeviceMode = GetDeviceMode("")

local network = {}
local ifname_list
local function GetNetWorkInfo()
    if not ifname_list then
        ifname_list = _M.NetworkInterface("eth")
        local add = _M.NetworkInterface("wwan")
        for _, v in pairs(add or {}) do
            ifname_list = ifname_list or {}
            table.insert(ifname_list, v)
        end
        if not next(ifname_list) then ifname_list = nil end
    end
    for k, v in pairs(ifname_list or {}) do
		local net = { }
        local cmd = "/sbin/ifconfig "..v
        local f = io.popen(cmd)
        if f then
            while true do
                local line = f:read("l")
                if not line then break end

                local ip = line:match("inet addr:(.-)%s")
                if ip then
                    if ip:match("%d+%.%d+%.%d+%.%d+") then
                        net.ip = ip
                    end
                else
                    local rx = line:match("RX bytes:(.-)%s")
                    local tx = line:match("TX bytes:(.-)%s")
                    if rx and tx then
						net.time = os.time()
						net.rx_bytes = rx
						net.tx_bytes = tx
						if network[v] then
							net.diff_rx = (tonumber(rx) - tonumber(network[v].rx_bytes or 0)) * 8
							net.diff_tx = (tonumber(tx) - tonumber(network[v].tx_bytes or 0)) * 8
						else
							net.diff_rx = 0
							net.diff_tx = 0
						end
                    end
                end
            end
            f:close()
            if next(net) then
                network[v] = net
            end
        end
    end
    local tbl = {}
    for k, v in pairs(network) do
        table.insert(tbl, {
            k = k,
            ip = v.ip or "unknow",
            receive = v.diff_rx,
            transmit = v.diff_tx,
            name = k,
        })
    end
    table.sort(tbl, function(a, b)
        return a.k < b.k
    end)
	return tbl
end

local function UpdatePicture(serial)
    if api.updatePicture then
        local jpg = api.updatePicture()
        if jpg then
            local encode = mp.encode("UpdatePicture", {
                jpg = jpg
            })
            klnkservice:SendMsgpack(serial, encode, #encode)
        end
    end
end

local function update()
    local is_ok, err = xpcall(function()
        local flag
        --NOTE: 特殊的型号名字
        if DeviceMode == "E_DOCTOR_S" then
            api = api or require("ext_D")
            api.Init( DeviceMode )
            local decoder = {}
            flag, decoder  = api.GetDecoderInfo( decoder )
            if flag then selfInfo.decoder = decoder end
            flag, selfInfo = api.GetBasicInformation( selfInfo )
            klnkSerial = selfInfo.serial_number
            return
        end
        if DeviceMode:match("^E") or DeviceMode:match("^G") or DeviceMode:match("^P") or DeviceMode:match("^RE")
            or DeviceMode:match("^N1") or DeviceMode:match("^N2") or DeviceMode:match("^M2") then
            api = api or require("ext_E")
            selfInfo.stream = nil
            selfInfo, klnkSerial = api.getEncoderInfo(selfInfo)
        elseif DeviceMode:match("^S2") then
            api = api or require("ext_S")
            selfInfo, klnkSerial = api.getEncoderInfo(selfInfo, features or _G)
        elseif DeviceMode:match("^N") or DeviceMode:match("^U") or DeviceMode:match("^RN") then
            api = api or require("ext_N")
            api.Init(DeviceMode)
            flag, selfInfo = api.GetBasicInformation( selfInfo )
            selfInfo.encoder = {}
            selfInfo.decoder = {}
            selfInfo.show_streamer_button = true
            if selfInfo.driving_mode == "encoder" then
                _, selfInfo.encoder = api.GetEncoderInfo({})
            else
                _, selfInfo.decoder = api.GetDecoderInfo({})
            end

            local network_traffic = GetNetWorkInfo()
            if network_traffic then selfInfo.network_traffic = network_traffic end

            if selfInfo.driving_mode == "decoder" then
                local decoder = {}
                flag, decoder  = api.GetDecoderInfo( decoder )
                if flag then selfInfo.decoder = decoder end
            else
                local capture = {}
                flag, capture  = api.GetCaptureInfo( capture )
                if flag then selfInfo.capture = capture end
            end
            selfInfo.device_status = api.GetDeviceStatus()
            klnkSerial = selfInfo.serial_number
        elseif DeviceMode:match("^D") or DeviceMode:match("^MG") or DeviceMode:match("^RD") or DeviceMode:match("^RMG") or DeviceMode:match("^MS") then
            --NOTE: 解码器
            api = api or require("ext_D")
            api.Init( DeviceMode )
            local decoder = {}
            flag, decoder  = api.GetDecoderInfo( decoder )
            if flag then selfInfo.decoder = decoder end
            flag, selfInfo = api.GetBasicInformation( selfInfo )
            klnkSerial = selfInfo.serial_number
        else
            --TODO: 
        end
    end, debug.traceback)
    if not is_ok then
        print("Update function get ERROR:", err)
    end
end

local function sendGetFirmware(serial, path, name, off)
    local encode = mp.encode("GetFirmware", {
        FilePath = path,
        Offset = off,
        ReadSize = 4 * 1024,
        name = name
    });
    -- print("send Msgpack ---->> GetFirmware ")
    klnkservice:SendMsgpack(serial, encode, #encode);
end

local function sendUpgradeStatus(serial, status)
    local encode = mp.encode("BasicInformation", {
        upgrade_status = status
    })
    klnkservice:SendMsgpack(serial, encode, #encode)
end

local function UpdateFirmware(map)
    local file = "/tmp/" .. api.FileName
    if api.writeSize == 0 then
        api.writeCount = api.writeCount + 1
    end

    local f = io.open(file, "a+")
    if f then
        f:write(map)
        f:close()
    end
    api.writeSize = api.writeSize + #map
    if api.writeSize == api.FileSize then
        collectgarbage()
        api.ReceivingUpgradeFile = false
        sendUpgradeStatus(klnkSerial, "done")
        --TODO: 做真正的升级
        if not api.UpgradeDevice(file) then
            sendUpgradeStatus(klnkSerial, "failed")
            os.execute("/bin/rm -rf " .. file)
        end
    else
        sendGetFirmware(klnkSerial, api.FilePath, api.FileName, api.writeSize)
    end
end

local function CloseUpgrade(delete)
    api.ReceivingUpgradeFile = false
    if delete and api.FileName then
        os.execute("/bin/rm -rf /tmp/" .. api.FileName )
    end
end

math.randomseed(string.reverse(os.time()))
local function sendBasicInfo(serial)
    update()
    selfInfo.random_num = selfInfo.random_num or math.random(1000, 9999)
    -- print("Dump the BasicInformation : ")
    -- pl.dump(selfInfo)
    local encode = mp.encode("BasicInformation", selfInfo)
    klnkservice:SendMsgpack(serial, encode, #encode);
end

local function update_func()
    if klnkOnline and klnkSerial~="" then
        update()
        selfInfo.software_version = nil
        selfInfo.web_port = nil
        local encode = mp.encode("BasicInformation", selfInfo)
        klnkservice:SendMsgpack(klnkSerial, encode, #encode);
        UpdatePicture(klnkSerial)
    end
end

_M.client_cfg = require "client_config".Load()
function _M.ReInit()
    print("####### in function ReInit !!!!!!!!!")
    if _M.receive_task then _M.receive_task:stop() end
    if _M.send_task then _M.send_task:stop() end
    if _M.update_task then _M.update_task:stop() end

    klnkOnline = false
    klnkservice:Stop()

    print("dump the cfg:")
    pl.dump(_M.client_cfg)
    if not _M.client_cfg.enable or not _M.client_cfg.ifname[1]
        or _M.client_cfg.ip == "" or not tonumber(_M.client_cfg.port) then
        return
    end

    update()
    pl.dump({
        port = _M.client_cfg.port, 
        ip = _M.client_cfg.ip, 
        ifname = _M.client_cfg.ifname[1], 
        key = _M.client_cfg.key,
        serial = selfInfo.serial_number, 
        encrypt = _M.client_cfg.crypto and 1 or 0,
    })
    local a, b = klnkservice:StartClient({
        port = _M.client_cfg.port, 
        ip = _M.client_cfg.ip, 
        ifname = _M.client_cfg.ifname[1], 
        key = _M.client_cfg.key,
        serial = selfInfo.serial_number, 
        encrypt = _M.client_cfg.crypto and 1 or 0,
    })
    for i = 2, #_M.client_cfg.ifname do
        klnkservice:PathAdd(_M.client_cfg.ifname[i])
    end


    klnkservice:ExternalCommandAdd("Reboot")
    klnkservice:ExternalCommandAdd("Reset")
    klnkservice:ExternalCommandAdd("Upgrade")
    klnkservice:ExternalCommandAdd("UpdateFirmware")
    klnkservice:ExternalCommandAdd("CloseUpgrade")
    klnkservice:ExternalCommandAdd("GetFirmware")
    klnkservice:ExternalCommandAdd("BasicInformation")
    klnkservice:ExternalCommandAdd("NetWork")
    klnkservice:ExternalCommandAdd("Capture")
    klnkservice:ExternalCommandAdd("Encoder")
    klnkservice:ExternalCommandAdd("Decoder")
    klnkservice:ExternalCommandAdd("MediaStream")
    klnkservice:ExternalCommandAdd("ControlDeviceStream")
    klnkservice:ExternalCommandAdd("ControlAllStream")
    klnkservice:ExternalCommandAdd("VideoSet")
    klnkservice:ExternalCommandAdd("NDISet")
    klnkservice:ExternalCommandAdd("AudioSet")
    klnkservice:ExternalCommandAdd("OutputSet")
    klnkservice:ExternalCommandAdd("AddStreamToService")
    klnkservice:ExternalCommandAdd("DeviceDelStreamService")

    _M.receive_task = thread.NewTimer(function()
        local serial, msgpack = klnkservice:RecvMsgpack({timeout = 100})
        if serial and msgpack then
            local decode, map = mp.decode(msgpack)
            print("##### serial = ", serial, "decode = ", decode)
            local is_ok, err = xpcall(function()
                if decode == "online" then
                    sendBasicInfo(serial) 
                    klnkOnline = true
                elseif decode == "offline" then
                    --TODO
                    CloseUpgrade()
                    klnkOnline = false
                elseif decode == "Reboot" then -- S -> C
                    api.Reboot()
                elseif decode == "Reset" then -- S -> C
                    api.Reset()
                elseif decode == "ControlDeviceStream" then
                    api.ControlStream(map)
                elseif decode == "ControlAllStream" then
                    api.ControlAllStream(map)
                elseif decode == "Upgrade" then -- S -> C
                    -- if map then pl.dump(map) end
                    if not api.ReceivingUpgradeFile then
                        os.execute("/bin/rm -rf /tmp/" .. map.FileName )
                        api.FilePath = map.FilePath
                        api.FileSize = map.FileSize
                        api.FileName = map.FileName
                        api.writeSize = 0
                        api.writeCount = 0
                        sendGetFirmware(klnkSerial, api.FilePath, api.FileName, 0)
                        api.ReceivingUpgradeFile = true
                    end
                elseif decode == "UpdateFirmware" then -- S -> C
                    UpdateFirmware(map)
                elseif decode == "CloseUpgrade" then
                    CloseUpgrade(true)
                elseif decode == "VideoSet" then
                    if api.VideoSet then
                        api.VideoSet(map)
                    end
                elseif decode == "AudioSet" then
                    if api.AudioSet then
                        api.AudioSet(map)
                    end
                elseif decode == "OutputSet" then
                    if api.OutputSet then
                        api.OutputSet(map)
                    end
                elseif decode == "NDISet" then
                    if api.NDISet then
                        api.NDISet(map)
                    end
                elseif decode == "AddStreamToService" then
                    api.AddStreamToService(map, _M.client_cfg.ifname)
                elseif decode == "DeviceDelStreamService" then
                    api.DeviceDelStreamService()
                end
            end, debug.traceback)
            if not is_ok then
                print("Receve task, but do failed. Error : ", err)
            end
        end
    end, 1, 0.03)
    if _M.receive_task then
        _M.receive_task:Start()
    end
    
    _M.send_task = thread.NewTimer(function()
        local is_ok, err = xpcall(function()
            local network_traffic = GetNetWorkInfo()
            -- pl.dump(network_traffic)
            if network_traffic then
                if klnkOnline then
                    local encode = mp.encode("BasicInformation", {network_traffic = network_traffic})
                    -- print("send network traffic... ", klnkSerial)
                    klnkservice:SendMsgpack(klnkSerial, encode, #encode)
                end
            end
            local bonding_status = api.GetBondingStatus and api.GetBondingStatus() or nil
            if bonding_status and klnkOnline then
                local encode = mp.encode("BasicInformation", {bonding_status = bonding_status})
                    -- print("send network traffic... ", klnkSerial)
                    klnkservice:SendMsgpack(klnkSerial, encode, #encode)
            end
        end, debug.traceback)
        if not is_ok then
            print("Send task get ERROR:", err)
        end
    end, 1, 1)
    if _M.send_task then
        _M.send_task:Start()
    end

    _M.update_task = thread.NewTimer(update_func, 8, 8)
    if _M.update_task then
        _M.update_task:Start()
    end
end

function _M.GetConfig()
    -- 1 不在线， 2 在线， 3 连接中， 4 鉴权失败
    local status
    if not _M.client_cfg.enable then
        status = 1
    elseif klnkOnline then
        status = 2
    else
        status = 3
    end

    return _M.client_cfg, status
end

function _M.UpdateConfig(new)
    if type(new) ~= "table" then
        return false
    end
    for k, v in pairs(_M.client_cfg) do
        if new[k] ~= nil then
            _M.client_cfg[k] = new[k]
        end
    end
    _M.client_cfg:__TRIGGER(true)
    _M.ReInit()
    return true
end

function _M.NetworkInterface(skip)
    skip = skip or "^[a-z]"
    local cmd = "/sbin/ifconfig | /bin/grep " .. skip .. " | /usr/bin/awk -F: '{print $1}'"
    local f = io.popen(cmd)
    local list = {}
    if f then
        local line = f:read("l")
        while line do
            line = line .. ""
            if not line:match("^lo") and not line:match("%:") then
                table.insert(list, line:match("(.-)%s"))
            end
            line = f:read("l")
        end
        f:close()
    end
    if not next(list) then
        return
    end
    return list
end

function _M.CheckProxyToken(token)
    local skip = token == md5(tostring(selfInfo.serial_number) .. _M.hashProxy .. tostring(selfInfo.random_num))
    return skip
end

_M.klnkOnline = false

return _M