
local pl = require "pl.pretty"
require "version"
local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
client.InitSocket(nil, adapter)
local api = {
	device_type = "",
	driving_mode = "",
	network = {}
}
local __flg, FEATURES = pcall( require, "features" )
if not __flg or type(FEATURES) ~= "table" then
	FEATURES = {
	}
end

local function GetProxy(objPath, service, loc, timeout, socket_addr)
	local proxy
	local cli

	cli = client.New( service, loc, socket_addr, timeout or 5 ) --5 seconds timeout.
	if not cli then
		print( "CO", "Fail to create connection to service '" .. (loc or "local") .. "/" .. (service or "nil") )
		return nil
	end
	proxy = cli:CreateProxy(objPath, true) --Exclusive
	if not proxy then
		print( "CO", "Fail to create object proxy for '" .. (objPath or "unknown") .. "'" )
		return nil
	end
	return proxy
end


local function SerialNumber()
	local proxy = GetProxy( "/", "systemctrl" )
	if  proxy then
		local r, sn = proxy.GetSerialNo()
		proxy:Destroy()
		if r and sn then
			return sn
		end
	end
	return "000000000"
end

local function GetDeviceName()
	local filename = "/etc/hostname"
    local f = io.open( filename, "r" )
	if f then
		local r = f:read("*l")
		f:close()
        if r then
		    return r
        end
    end
    return ""
end

local function GetOutput( info, output )
	local layout_id = nil
	local name = "/output/hd" .. output
	local proxy = GetProxy( name, "codec" )
	local vis
	if proxy then
		local a_r, analog_dis, analog_gain, analog_local_en = proxy.GetAudioGain("analog")
		if not a_r then
			analog_dis = 1
			analog_gain = 0
			analog_local_en = 0
		end
		local e_r, emb_dis, emb_gain, emb_local_en = proxy.GetAudioGain("embedded")
		if not e_r then
			emb_dis = 1
			emb_gain = 0
			emb_local_en = 0
		end

		local r, status,err = proxy.GetStatus(nil)
		local p_r, port_details = proxy.GetInterfacesBinding()
		local v_r
		v_r, vis = proxy.GetShowLayouts()
		if not r or not vis then
			vis = {"1"}
		end
		proxy:Destroy()
		if r and status then
			local result = {}
			result.audio_analog_gain = {
				value = analog_gain,
				disable = analog_dis,
				localOutput = analog_local_en
			}
			result.audio_digital_gain = {
				value = emb_gain,
				disable = emb_dis,
				localOutput = emb_local_en
			}
			if p_r and port_details then
				result.output_port = port_details
			end
			result.layout_id     = status.layout or 1
			result.layout_number = status.layout_number or 1
			result.output     = status.started and "on" or "off"
			result.resolution = status.resolution or "1080P30"
			result.resolution_name = status.resolution_name or status.resolution or "1080P30"
			result.sampling   = status.sampling or 48000
			result.hdmi_mode  = status.hdmi_mode or "AUTO"
			result.colorspace = status.colorspace or "AUTO"
			if not status.border and status.border~="" then
				result.bordercolor_enabled = status.border.enabled
				result.bordercolor  = status.border.bordercolor
			else
				result.bordercolor_enabled = false
				result.bordercolor  = 0
			end
			layout_id = result.layout_id
			if status.channels then
				result.layout = {}
				proxy = GetProxy( "/streams", "codec" )
				if proxy then
					local i, item
					for i=1, #status.channels do
						item = {
							id = status.channels[i].id,
							stream_id = status.channels[i].stream,
							status = status.channels[i].bind and "on" or "off",
							stream_name = "",
							volume = "",
							src = "",
							manual_url = "",
							bitrate = 0,
							resolution = "",
							buffer = "live:200:200:500:0:0"
						}
		
						if proxy then
							local detail
							r, detail = proxy.GetStreamDetail( status.channels[i].stream )
		
							if r and detail then
								item.buffer      = detail.buffer_option or item.buffer_option
								item.stream_name = detail.name
								item.source      = detail.source or 1
								item.type        = detail.type 
								item.origin      = detail.origin 
								item.src         = detail.address or "UNKNOWN"
								item.manual_url  = detail.manual_url or "UNKNOWN"
								local originIp,imgUrl
								if detail.origin =="remote" then
									local num = string.find(detail.imgUrl,"%/")
									originIp  = string.sub(detail.imgUrl,1,num-1)
									imgUrl    = string.sub(detail.imgUrl,num)
								else
									imgUrl = detail.imgUrl
								end
		
								item.imgUrl   = imgUrl 
								item.originIp = originIp 
		
								item.volume = detail.audio_enables and detail.audio_enables[ "hd" .. output ] and "on" or "off"
								if detail.stream_info then
									item.bitrate = detail.stream_info.bitrate or 0
									item.resolution = detail.stream_info.resolution or "分辨率未知"
		
									if detail.stream_info.live == "connected" then
										if detail.stream_info.stream_start and detail.stream_info.output_start then
											item.status = "view_pull"
										elseif detail.stream_info.stream_start then
											item.status = "pull"
										elseif detail.stream_info.output_start then
											item.status = "view"
										end
									else
										if detail.stream_info.live ~= "closed" and detail.stream_info.live ~= "" then
											item.status = detail.stream_info.live
										else
											item.status = "off"
										end
									end
								end
		
								if detail.srt_status then
									item.srtStatus = detail.srt_status
								end
		
								if detail.rtmp_server_status then
									item.rtmp_server_status = detail.rtmp_server_status
								end
							end
						end
		
						table.insert( result.layout, item )
					end

					proxy:Destroy()
				end
			end
			info.output = result
		end
	end

	proxy = GetProxy( name, "codec" )
	if proxy then
		local r, details = proxy.GetInterfacesBinding()
		proxy:Destroy()
		if r and details then
			info.bind = details
		end
	end

	if layout_id then
		local proxy = GetProxy( "/layout", "codec" )
		if proxy then
			local r, icons, err = proxy.GetAllLayouts()
			proxy:Destroy()
			for kk, vv in pairs(vis) do
				if icons[vv] then
					icons[vv].enable = 1
				end
			end
			for k,v in pairs(icons) do
				if v.id == tonumber(layout_id) then
					info.position = v.position
					v.output_used = 1
					break
				end
			end
			info.icons = {}
			for _, v in pairs(icons) do
				table.insert(info.icons, v)
			end
			table.sort(info.icons, function(a, b)
				return a.id < b.id
			end)
		end
	end

	if info.position then
		for _, v in ipairs(info.output.layout) do
			if v.id and info.position[v.id] then
				for kk, vv in pairs(info.position[v.id]) do
					v[kk] = vv
				end
			end
		end
	end

	info.position = nil

	return true, info
end

function api.Init( device_type )
	api.device_type  = device_type
	api.driving_mode = "decoder"
end

function api.GetNetWorkInfo( info )
    local ifname_list = {"eth0", "lo"}
    for k, v in pairs(ifname_list) do
		local net = { }
        local cmd = "/sbin/ifconfig "..v
        local f = io.popen(cmd)
        if f then
            while true do
                local line = f:read("l")
                if not line then break end

                local ip = line:match("inet addr:(.-)%s")
                if ip then
                    if ip:match("%d+%.%d+%.%d+%.%d+") then
                        net.ip = ip
                    end
                else
                    local rx = line:match("RX bytes:(.-)%s")
                    local tx = line:match("TX bytes:(.-)%s")
                    if rx and tx then
						net.time = os.time()
						net.rx_bytes = rx
						net.tx_bytes = tx
						if api.network[v] then
							net.diff_rx = (tonumber(rx) - tonumber(api.network[v].rx_bytes)) * 8
							net.diff_tx = (tonumber(tx) - tonumber(api.network[v].tx_bytes)) * 8
						else
							net.diff_rx = 0
							net.diff_tx = 0
						end
                    end
                end
            end
            f:close()
			api.network[v] = net
        end
    end
	info = api.network
	return true, info
end

function api.SerialNumber()
    return SerialNumber()
end

function api.DeviceName()
    return GetDeviceName()
end

function api.Reboot()
	if FEATURES.FEATURE_LOCAL_STREAM then
		local proxy = GetProxy( "/local", "CoManager" )
		if proxy then
			local r, data,err = proxy.GetLocalEnInfo()
			proxy:Destroy()
			if r and data then
				for k,v in pairs(data) do
					if v.id then
						local amba_proxy = GetProxy( "/", "systemctrl",v.id )
						if amba_proxy then
							amba_proxy.Reboot(1)
							amba_proxy:Destroy()
						end
					end
				end
			end
		end
	end

	local proxy = GetProxy( "/", "systemctrl" )
	if proxy then
		proxy.Reboot(3)
		proxy:Destroy()
	end
end

function api.Reset()
	local proxy = GetProxy( "/", "systemctrl" )
	if  proxy then
		proxy.Reset()
		proxy:Destroy()
	end
end

local cpu_saved = {}
local function GetCpuLoad()
	local file = io.open( "/proc/stat", "r" )
	if not file then
		return 0, 99	--load: 0, idle: 99
	end

	local line = file:read("l")
	file:close()

	local cpu,u,n,s,i,w,x,y = line:match( "(.-)%s+(%d+)%s+(%d+)%s+(%d+)%s+(%d+)%s+(%d+)%s+(%d+)%s+(%d+)" )
	u = u and tonumber(u) or 0
	n = n and tonumber(n) or 0
	s = s and tonumber(s) or 0
	i = i and tonumber(i) or 0
	w = w and tonumber(w) or 0
	x = x and tonumber(x) or 0
	y = y and tonumber(y) or 0

	local du = u - (cpu_saved.u or 0)
	local dn = n - (cpu_saved.n or 0)
	local ds = s - (cpu_saved.s or 0)
	local di = i - (cpu_saved.i or 0)
	local dw = w - (cpu_saved.w or 0)
	local dx = x - (cpu_saved.x or 0)
	local dy = y - (cpu_saved.y or 0)

	cpu_saved.u = u
	cpu_saved.n = n
	cpu_saved.s = s
	cpu_saved.i = i
	cpu_saved.w = w
	cpu_saved.x = x
	cpu_saved.y = y

	local total = du + dn + ds + di + dw + dx + dy
	if total < 1 then
		total = 1
	end

	local scale = 100/total
	local idle = di*scale
	return 100-idle, idle
end

local function GetMemUse()
	local file = io.open( "/proc/meminfo", "r" )
	if not file then
		return nil, nil
	end

	local free = nil
	local total = nil
	local v
	local line = file:read("l")
	while line and ( not free or not total ) do
		v = line:match( "MemTotal:%s+(%d+)" )
		if v then
			total = tonumber(v)
		else
			v = line:match( "MemFree:%s+(%d+)" )
			if v then
				free = tonumber(v)
			end
		end
		line = file:read("l")
	end

	file:close()
	return free, total
end

local function GetPersisTime()
	local file=io.open("/proc/uptime", "r")
	if not file then
		return 0, {year=1970,month=1,day=1,hour=8,min=0,sec=0}
	end

	local line = file:read("l")
	file:close()
	local pers = line:match("(%d+)")
	if not pers then
		return 0, {year=1970,month=1,day=1,hour=8,min=0,sec=0}
	else
		return tonumber(pers), os.date("*t",os.time() - tonumber(pers))
	end
end

local function GetDiskSpaces()
	local result = {
		sys = { total=0, used=0, free=0, percent=100 },
		data = { total=0, used=0, free=0, percent=100 }
	}

	local file=io.popen("/bin/df -m","r")
	if not file then
		return result
	end

	local line = file:read("l")
	while line and (result.sys.total <= 0 or result.data.total <=0 ) do
		local flag,t,u,f,p = line:match( "(.-)%s+(%d+)%s+(%d+)%s+(%d+)%s+(%d+)%%%s+/$" )
		if flag then
			result.sys.total = tonumber(t)
			result.sys.used = tonumber(u)
			result.sys.free = tonumber(f)
			result.sys.percent = tonumber(p)
		else
			flag,t,u,f,p = line:match( "(.-)%s+(%d+)%s+(%d+)%s+(%d+)%s+(%d+)%%%s+/data$" )
			if flag then
				result.data.total = tonumber(t)
				result.data.used = tonumber(u)
				result.data.free = tonumber(f)
				result.data.percent = tonumber(p)
			end
		end

		line=file:read("l")
	end

	file:close()
	return result
end

local function GetDeviceStatus()
	local memFree, memTotal = GetMemUse()
	return {
		cpu = GetCpuLoad(),
		mem = math.floor((1 - (memFree / memTotal)) * 100) or 0, 
		ptime = GetPersisTime(), 
		sysuse = GetDiskSpaces().sys.percent,
	}
end

function api.GetBasicInformation(info)
    info.device_name  = GetDeviceName()
    info.device_mode  = "decoder"
	info.device_type  = api.device_type
    info.driving_mode = "decoder"
	info.serial_number    = SerialNumber()
	info.software_version = _G.SOFTWARE_VERSION
	info.hardware_version = _G.HARDWARE_VERSION
	info.web_index = "home"
	info.device_status = GetDeviceStatus()
    return true, info
end

function api.GetEncoderInfo(info)
	return false, info 
end

function api.GetCaptureInfo(info)
	return false, info
end

function api.GetDecoderInfo(info)
	if info.driving_mode == "" or info.driving_mode == "encoder" then
		return false, info
	end

	local flag, output = false, {}
	flag, output = GetOutput(output, 0)

	if flag then info.output1 = output	end
	local output2 = {}
	flag, output2 = GetOutput(output2, 1)

	if flag then info.output2 = output2	end
	return true, info
end

function api.MediaStream( info )
	return false, info
end

function api.GetSnapshoot()
	return false, ""
end

function api.UpgradeDevice(file)
    local proxy = GetProxy("/", "systemctrl", "local", 30)
    if not proxy then
        print("Can't Get the systemctrl proxy !!!")
        return
    end

    local is_ok, skip = proxy.ProcessFirmware(file)
    proxy:Destroy()
    return is_ok and skip
end

function api.OutputSet(map)
	if map.out and map.layout_id then
		local proxy = GetProxy("/output/hd" .. (map.out - 1), "codec")
		if proxy then
			proxy.SetOutputLayout(map.layout_id)
			proxy:Destroy()
		end
	end
end

return api
