local _M = {}
local MAIN_FREQ = _G.MAIN_FREQ or 28636360

---------------------------
_M.VIRATE_SD = 0
_M.VIRATE_HD = 1
_M.VIRATE_3G = 2
_M.VIRATE_VGA = 3
---------------------------
local VIRATE_SD = _M.VIRATE_SD
local VIRATE_HD = _M.VIRATE_HD
local VIRATE_3G = _M.VIRATE_3G
local VIRATE_VGA = _M.VIRATE_VGA

---------------------------
local RATIO_AUTO = _G.AMBA_VIDEO_RATIO_AUTO or 0
local RATIO_4_3 = _G.AMBA_VIDEO_RATIO_4_3 or 1
local RATIO_16_9 = _G.AMBA_VIDEO_RATIO_16_9 or 2
local RATIO_1_1 = _G.AMBA_VIDEO_RATIO_1_1 or 4

_M.RATIO_AUTO = RATIO_AUTO
_M.RATIO_4_3 = RATIO_4_3
_M.RATIO_16_9 = RATIO_16_9
_M.RATIO_1_1 = RATIO_1_1
---------------------------

local VIDEO_SYSTEM_AUTO = _G.AMBA_VIDEO_SYSTEM_AUTO or 0x0
local VIDEO_SYSTEM_NTSC = _G.AMBA_VIDEO_SYSTEM_NTSC or 0x1
local VIDEO_SYSTEM_PAL = _G.AMBA_VIDEO_SYSTEM_PAL or 0x2
local VIDEO_SYSTEM_SECAM = _G.AMBA_VIDEO_SYSTEM_SECAM or 0x4
local VIDEO_SYSTEM_ALL = _G.AMBA_VIDEO_SYSTEM_ALL or 0xf

_M.VIDEO_SYSTEM_AUTO = VIDEO_SYSTEM_AUTO
_M.VIDEO_SYSTEM_NTSC = VIDEO_SYSTEM_NTSC
_M.VIDEO_SYSTEM_PAL = VIDEO_SYSTEM_PAL
_M.VIDEO_SYSTEM_SECAM = VIDEO_SYSTEM_SECAM
_M.VIDEO_SYSTEM_ALL = VIDEO_SYSTEM_ALL

--------------------------

_M.format_map = {
--480i
	{
		standardId = {6, 7},
		formatId = "480i",
		formatName = "480i 60Hz",
		frameRate = 59.94,
		hTotal = 1716,
		hAct = 1440,
		vTotal0 = 262,
		vAct0 = 240,
		vTotal1 = 263,
		vAct1 = 240,
		interlaced = true,
		rateMode = VIRATE_SD,
		tmdsFreq = 27000000,
		spec_amba_mode = AMBA_VIDEO_MODE_480I,
		spec_adv7611_mode = 0x00,
		pixelRepeat = true,
		aspect_ratio = RATIO_4_3,
		video_system = VIDEO_SYSTEM_NTSC
	},

--576i
	{
		standardId = {21, 22},
		formatId = "576i",
		formatName = "576i 50Hz",
		frameRate = 50,
		hTotal = 1728,
		hAct = 1440,
		vTotal0 = 312,
		vAct0 = 288,
		vTotal1 = 313,
		vAct1 = 288,
		interlaced = true,
		rateMode = VIRATE_SD,
		tmdsFreq = 27000000,
		spec_adv7611_mode = 0x01,
		spec_amba_mode = AMBA_VIDEO_MODE_576I,
		pixelRepeat = true,
		aspect_ratio = RATIO_4_3,
		video_system = VIDEO_SYSTEM_PAL
	},
	
--480i(1:1)
	{
		--standardId = {},
		formatId = "480i(1:1)",
		formatName = "480i 60Hz(1:1)",
		frameRate = 59.94,
		hTotal = 858,
		hAct = 720,
		vTotal0 = 262,
		vAct0 = 240,
		vTotal1 = 263,
		vAct1 = 240,
		interlaced = true,
		rateMode = VIRATE_SD,
		tmdsFreq = 13500000,
		spec_adv7611_mode = 0x00,
		spec_amba_mode = AMBA_VIDEO_MODE_480I,
		pixelRepeat = false,
		aspect_ratio = RATIO_4_3,
		video_system = VIDEO_SYSTEM_NTSC
	},
	
--576i(1:1)
	{
		--standardId = {},
		formatId = "576i(1:1)",
		formatName = "576i 50Hz(1:1)",
		frameRate = 50,
		hTotal = 864,
		hAct = 720,
		vTotal0 = 312,
		vAct0 = 288,
		vTotal1 = 313,
		vAct1 = 288,
		interlaced = true,
		rateMode = VIRATE_SD,
		tmdsFreq = 13500000,
		spec_adv7611_mode = 0x00,
		spec_amba_mode = AMBA_VIDEO_MODE_576I,
		pixelRepeat = false,
		aspect_ratio = RATIO_4_3,
		video_system = VIDEO_SYSTEM_PAL
	},
	
--480p
	{
		standardId = {2,3},
		formatId = "480p",
		formatName = "480p 60Hz",
		frameRate = 59.94,
		hTotal = 858,
		hAct = 720,
		vTotal0 = 525,
		vAct0 = 480,
		vTotal1 = 525,
		vAct1 = 480,
		interlaced = false,
		rateMode = VIRATE_SD,
		tmdsFreq = 27000000,
		spec_adv7611_mode = 0x0a,
		spec_amba_mode = AMBA_VIDEO_MODE_D1_NTSC,
		pixelRepeat = false,
		aspect_ratio = RATIO_4_3,
		video_system = VIDEO_SYSTEM_NTSC
	},
	
--576p
	{
		standardId = {17,18},
		formatId = "576p",
		formatName = "576p 50Hz",
		frameRate = 50,
		hTotal = 864,
		hAct = 720,
		vTotal0 = 625,
		vAct0 = 576,
		vTotal1 = 625,
		vAct1 = 576,
		interlaced = false,
		rateMode = VIRATE_SD,
		tmdsFreq = 27000000,
		spec_adv7611_mode = 0x0b,
		spec_amba_mode = AMBA_VIDEO_MODE_D1_PAL,
		pixelRepeat = false,
		aspect_ratio = RATIO_4_3,
		video_system = VIDEO_SYSTEM_PAL
	},
	
--720p60
	{
		standardId = {4},
		formatId = "720p60",
		formatName = "720p 60Hz",
		frameRate = 60,
		hTotal = 1650,
		hAct = 1280,
		vTotal0 = 750,
		vAct0 = 720,
		vTotal1 = 750,
		vAct1 = 720,
		interlaced = false,
		rateMode = VIRATE_HD,
		tmdsFreq = 74250000,
		spec_adv7611_mode = 0x13,
		spec_amba_mode = AMBA_VIDEO_MODE_720P,
		pixelRepeat = false,
		aspect_ratio = RATIO_16_9,
		video_system = VIDEO_SYSTEM_AUTO
	},		
	
--720p59
	{
		standardId = {4},
		formatId = "720p59",
		formatName = "720p 59.94Hz",
		frameRate = 59.94,
		hTotal = 1650,
		hAct = 1280,
		vTotal0 = 750,
		vAct0 = 720,
		vTotal1 = 750,
		vAct1 = 720,
		interlaced = false,
		rateMode = VIRATE_HD,
		tmdsFreq = 74250000,
		spec_adv7611_mode = 0x13,
		spec_amba_mode = AMBA_VIDEO_MODE_720P,
		pixelRepeat = false,
		aspect_ratio = RATIO_16_9,
		video_system = VIDEO_SYSTEM_NTSC
	},		
	
--720p50
	{
		standardId = {19},
		formatId = "720p50",
		formatName = "720p 50Hz",
		frameRate = 50,
		hTotal = 1980,
		hAct = 1280,
		vTotal0 = 750,
		vAct0 = 720,
		vTotal1 = 750,
		vAct1 = 720,
		interlaced = false,
		rateMode = VIRATE_HD,
		tmdsFreq = 74250000,
		spec_adv7611_mode = 0x13,
		spec_amba_mode = AMBA_VIDEO_MODE_720P,
		pixelRepeat = false,
		aspect_ratio = RATIO_16_9,
		video_system = VIDEO_SYSTEM_AUTO
	},		
	
--720p30
	{
		standardId = {62},
		formatId = "720p30",
		formatName = "720p 30Hz",
		frameRate = 30,
		hTotal = 3300,
		hAct = 1280,
		vTotal0 = 750,
		vAct0 = 720,
		vTotal1 = 750,
		vAct1 = 720,
		interlaced = false,
		rateMode = VIRATE_HD,
		tmdsFreq = 74250000,
		spec_adv7611_mode = 0x13,
		spec_amba_mode = AMBA_VIDEO_MODE_720P,
		pixelRepeat = false,
		aspect_ratio = RATIO_16_9,
		video_system = VIDEO_SYSTEM_AUTO
	},		
	
--720p29
	{
		standardId = {62},
		formatId = "720p29",
		formatName = "720p 29.97Hz",
		frameRate = 29.97,
		hTotal = 3300,
		hAct = 1280,
		vTotal0 = 750,
		vAct0 = 720,
		vTotal1 = 750,
		vAct1 = 720,
		interlaced = false,
		rateMode = VIRATE_HD,
		tmdsFreq = 74250000,
		spec_adv7611_mode = 0x13,
		spec_amba_mode = AMBA_VIDEO_MODE_720P,
		pixelRepeat = false,
		aspect_ratio = RATIO_16_9,
		video_system = VIDEO_SYSTEM_NTSC
	},		
	
--720p25
	{
		standardId = {61},
		formatId = "720p25",
		formatName = "720p 25Hz",
		frameRate = 25,
		hTotal = 3960,
		hAct = 1280,
		vTotal0 = 750,
		vAct0 = 720,
		vTotal1 = 750,
		vAct1 = 720,
		interlaced = false,
		rateMode = VIRATE_HD,
		tmdsFreq = 74250000,
		spec_adv7611_mode = 0x13,
		spec_amba_mode = AMBA_VIDEO_MODE_720P,
		pixelRepeat = false,
		aspect_ratio = RATIO_16_9,
		video_system = VIDEO_SYSTEM_AUTO
	},		
	
--720p24
	{
		standardId = {60},
		formatId = "720p24",
		formatName = "720p 24Hz",
		frameRate = 24,
		hTotal = 4125,
		hAct = 1280,
		vTotal0 = 750,
		vAct0 = 720,
		vTotal1 = 750,
		vAct1 = 720,
		interlaced = false,
		rateMode = VIRATE_HD,
		tmdsFreq = 74250000,
		spec_adv7611_mode = 0x13,
		spec_amba_mode = AMBA_VIDEO_MODE_720P,
		pixelRepeat = false,
		aspect_ratio = RATIO_16_9,
		video_system = VIDEO_SYSTEM_AUTO
	},	
	
--720p23
	{
		standardId = {60},
		formatId = "720p23",
		formatName = "720p 23.976Hz",
		frameRate = 23.976,
		hTotal = 4125,
		hAct = 1280,
		vTotal0 = 750,
		vAct0 = 720,
		vTotal1 = 750,
		vAct1 = 720,
		interlaced = false,
		rateMode = VIRATE_HD,
		tmdsFreq = 74250000,
		spec_adv7611_mode = 0x13,
		spec_amba_mode = AMBA_VIDEO_MODE_720P,
		pixelRepeat = false,
		aspect_ratio = RATIO_16_9,
		video_system = VIDEO_SYSTEM_NTSC
	},

--1080i60
	{
		standardId = {5},
		formatId = "1080i60",
		formatName = "1080i 60Hz",
		frameRate = 60,
		hTotal = 2200,
		hAct = 1920,
		vTotal0 = 562,
		vAct0 = 540,
		vTotal1 = 563,
		vAct1 = 540,
		interlaced = true,
		rateMode = VIRATE_HD,
		tmdsFreq = 74250000,
		spec_adv7611_mode = 0x14,
		spec_amba_mode = AMBA_VIDEO_MODE_1080I,
		pixelRepeat = false,
		aspect_ratio = RATIO_16_9,
		video_system = VIDEO_SYSTEM_AUTO
	},

--1080i59
	{
		standardId = {5},
		formatId = "1080i59",
		formatName = "1080i 59.94Hz",
		frameRate = 59.94,
		hTotal = 2200,
		hAct = 1920,
		vTotal0 = 562,
		vAct0 = 540,
		vTotal1 = 563,
		vAct1 = 540,
		interlaced = true,
		rateMode = VIRATE_HD,
		tmdsFreq = 74250000,
		spec_adv7611_mode = 0x14,
		spec_amba_mode = AMBA_VIDEO_MODE_1080I,
		pixelRepeat = false,
		aspect_ratio = RATIO_16_9,
		video_system = VIDEO_SYSTEM_NTSC
	},
	
--1080i50
	{
		standardId = {20},
		formatId = "1080i50",
		formatName = "1080i 50Hz",
		frameRate = 50,
		hTotal = 2640,
		hAct = 1920,
		vTotal0 = 562,
		vAct0 = 540,
		vTotal1 = 563,
		vAct1 = 540,
		interlaced = true,
		rateMode = VIRATE_HD,
		tmdsFreq = 74250000,
		spec_adv7611_mode = 0x14,
		spec_amba_mode = AMBA_VIDEO_MODE_1080I,
		pixelRepeat = false,
		aspect_ratio = RATIO_16_9,
		video_system = VIDEO_SYSTEM_AUTO
	},

--1080p60
	{
		standardId = {16},
		formatId = "1080p60",
		formatName = "1080p 60Hz",
		frameRate = 60,
		hTotal = 2200,
		hAct = 1920,
		vTotal0 = 1125,
		vAct0 = 1080,
		vTotal1 = 1125,
		vAct1 = 1080,
		interlaced = false,
		rateMode = VIRATE_3G,
		tmdsFreq = 148500000,
		spec_adv7611_mode = 0x1e,
		spec_amba_mode = AMBA_VIDEO_MODE_1080p,
		pixelRepeat = false,
		aspect_ratio = RATIO_16_9,
		video_system = VIDEO_SYSTEM_AUTO
	},		
	
--1080p59
	{
		standardId = {16},
		formatId = "1080p59",
		formatName = "1080p 59.94Hz",
		frameRate = 59.94,
		hTotal = 2200,
		hAct = 1920,
		vTotal0 = 1125,
		vAct0 = 1080,
		vTotal1 = 1125,
		vAct1 = 1080,
		interlaced = false,
		rateMode = VIRATE_3G,
		tmdsFreq = 148500000,
		spec_adv7611_mode = 0x1e,
		spec_amba_mode = AMBA_VIDEO_MODE_1080P,
		pixelRepeat = false,
		aspect_ratio = RATIO_16_9,
		video_system = VIDEO_SYSTEM_NTSC
	},		
	
--1080p50
	{
		standardId = {31},
		formatId = "1080p50",
		formatName = "1080p 50Hz",
		frameRate = 50,
		hTotal = 2640,
		hAct = 1920,
		vTotal0 = 1125,
		vAct0 = 1080,
		vTotal1 = 1125,
		vAct1 = 1080,
		interlaced = false,
		rateMode = VIRATE_3G,
		tmdsFreq = 148500000,
		spec_adv7611_mode = 0x1e,
		spec_amba_mode = AMBA_VIDEO_MODE_1080P,
		pixelRepeat = false,
		aspect_ratio = RATIO_16_9,
		video_system = VIDEO_SYSTEM_AUTO
	},		
	
--1080p30
	{
		standardId = {34},
		formatId = "1080p30",
		formatName = "1080p 30Hz",
		frameRate = 30,
		hTotal = 2200,
		hAct = 1920,
		vTotal0 = 1125,
		vAct0 = 1080,
		vTotal1 = 1125,
		vAct1 = 1080,
		interlaced = false,
		rateMode = VIRATE_HD,
		tmdsFreq = 74250000,
		spec_adv7611_mode = 0x1e,
		spec_amba_mode = AMBA_VIDEO_MODE_1080P,
		pixelRepeat = false,
		aspect_ratio = RATIO_16_9,
		video_system = VIDEO_SYSTEM_AUTO
	},		
	
--1080p29
	{
		standardId = {34},
		formatId = "1080p29",
		formatName = "1080p 29.97Hz",
		frameRate = 29.97,
		hTotal = 2200,
		hAct = 1920,
		vTotal0 = 1125,
		vAct0 = 1080,
		vTotal1 = 1125,
		vAct1 = 1080,
		interlaced = false,
		rateMode = VIRATE_HD,
		tmdsFreq = 74250000,
		spec_adv7611_mode = 0x1e,
		spec_amba_mode = AMBA_VIDEO_MODE_1080P,
		pixelRepeat = false,
		aspect_ratio = RATIO_16_9,
		video_system = VIDEO_SYSTEM_NTSC
	},		
	
--1080p25
	{
		standardId = {33},
		formatId = "1080p25",
		formatName = "1080p 25Hz",
		frameRate = 25,
		hTotal = 2640,
		hAct = 1920,
		vTotal0 = 1125,
		vAct0 = 1080,
		vTotal1 = 1125,
		vAct1 = 1080,
		interlaced = false,
		rateMode = VIRATE_HD,
		tmdsFreq = 74250000,
		spec_adv7611_mode = 0x1e,
		spec_amba_mode = AMBA_VIDEO_MODE_1080P,
		pixelRepeat = false,
		aspect_ratio = RATIO_16_9,
		video_system = VIDEO_SYSTEM_AUTO
	},		
	
--1080p24
	{
		standardId = {32},
		formatId = "1080p24",
		formatName = "1080p 24Hz",
		frameRate = 24,
		hTotal = 2750,
		hAct = 1920,
		vTotal0 = 1125,
		vAct0 = 1080,
		vTotal1 = 1125,
		vAct1 = 1080,
		interlaced = false,
		rateMode = VIRATE_HD,
		tmdsFreq = 74250000,
		spec_adv7611_mode = 0x1e,
		spec_amba_mode = AMBA_VIDEO_MODE_1080P,
		pixelRepeat = false,
		aspect_ratio = RATIO_16_9,
		video_system = VIDEO_SYSTEM_AUTO
	},	
	
--1080p23
	{
		standardId = {32},
		formatId = "1080p23",
		formatName = "1080p 23.976Hz",
		frameRate = 23.976,
		hTotal = 2750,
		hAct = 1920,
		vTotal0 = 1125,
		vAct0 = 1080,
		vTotal1 = 1125,
		vAct1 = 1080,
		interlaced = false,
		rateMode = VIRATE_HD,
		tmdsFreq = 74250000,
		spec_adv7611_mode = 0x1e,
		spec_amba_mode = AMBA_VIDEO_MODE_1080P,
		pixelRepeat = false,
		aspect_ratio = RATIO_16_9,
		video_system = VIDEO_SYSTEM_NTSC
	},
	
--640x480p60
	{
		standardId = {1},
		formatId = "640x480p60",
		formatName = "640x480@60Hz",
		frameRate = 60,
		hTotal = 800,
		hAct = 640,
		vTotal0 = 525,
		vAct0 = 480,
		vTotal1 = 525,
		vAct1 = 480,
		interlaced = false,
		rateMode = VIRATE_VGA,
		tmdsFreq = 25175000,
		spec_adv7611_mode = 0x08,
		spec_amba_mode = AMBA_VIDEO_MODE_VGA,
		pixelRepeat = false,
		aspect_ratio = RATIO_AUTO,
		video_system = VIDEO_SYSTEM_AUTO
	},

--640x480p75
	{
		standardId = {66},
		formatId = "640x480p75",
		formatName = "640x480@75Hz",
		frameRate = 75,
		hTotal = 840,
		hAct = 640,
		vTotal0 = 500,
		vAct0 = 480,
		vTotal1 = 500,
		vAct1 = 480,
		interlaced = false,
		rateMode = VIRATE_VGA,
		tmdsFreq = 31500000,
		spec_adv7611_mode = 0x0a,
		spec_amba_mode = AMBA_VIDEO_MODE_VGA,
		pixelRepeat = false,
		aspect_ratio = RATIO_AUTO,
		video_system = VIDEO_SYSTEM_AUTO
	},
   
--800x600p60
	{
		standardId = {69},
		formatId = "800x600p60",
		formatName = "800x600@60Hz",
		frameRate = 60,
		hTotal = 1056,
		hAct = 800,
		vTotal0 = 628,
		vAct0 = 600,
		vTotal1 = 628,
		vAct1 = 600,
		interlaced = false,
		rateMode = VIRATE_VGA,
		tmdsFreq = 40000000,
		spec_adv7611_mode = 0x01,
		spec_amba_mode = AMBA_VIDEO_MODE_SVGA,
		pixelRepeat = false,
		aspect_ratio = RATIO_AUTO,
		video_system = VIDEO_SYSTEM_AUTO
	},

--800x600p75
	{
		standardId = {71},
		formatId = "800x600p75",
		formatName = "800x600@75Hz",
		frameRate = 75,
		hTotal = 1056,
		hAct = 800,
		vTotal0 = 625,
		vAct0 = 600,
		vTotal1 = 625,
		vAct1 = 600,
		interlaced = false,
		rateMode = VIRATE_VGA,
		tmdsFreq = 49500000,
		spec_adv7611_mode = 0x03,
		spec_amba_mode = AMBA_VIDEO_MODE_SVGA,
		pixelRepeat = false,
		aspect_ratio = RATIO_AUTO,
		video_system = VIDEO_SYSTEM_AUTO
	},

--1024x768p60
	{
		standardId = {73},
		formatId = "1024x768p60",
		formatName = "1024x768@60Hz",
		frameRate = 60,
		hTotal = 1344,
		hAct = 1024,
		vTotal0 = 806,
		vAct0 = 768,
		vTotal1 = 806,
		vAct1 = 768,
		interlaced = false,
		rateMode = VIRATE_VGA,
		tmdsFreq = 65000000,
		spec_adv7611_mode = 0x0c,
		spec_amba_mode = AMBA_VIDEO_MODE_XGA,
		pixelRepeat = false,
		aspect_ratio = RATIO_AUTO,
		video_system = VIDEO_SYSTEM_AUTO
	},
	
 --1024x768p75
	{
		standardId = {75},
		formatId = "1024x768p75",
		formatName = "1024x768@75Hz",
		frameRate = 75,
		hTotal = 1312,
		hAct = 1024,
		vTotal0 = 800,
		vAct0 = 768,
		vTotal1 = 800,
		vAct1 = 768,
		interlaced = false,
		rateMode = VIRATE_VGA,
		tmdsFreq = 78800000,
		spec_adv7611_mode = 0x0e,
		spec_amba_mode = AMBA_VIDEO_MODE_XGA,
		pixelRepeat = false,
		aspect_ratio = RATIO_AUTO,
		video_system = VIDEO_SYSTEM_AUTO
	},
   
--1280x1024p60
	{
		standardId = {85},
		formatId = "1280x1024p60",
		formatName = "1280x1024@60Hz",
		frameRate = 60,
		hTotal = 1688,
		hAct = 1280,
		vTotal0 = 1066,
		vAct0 = 1024,
		vTotal1 = 1066,
		vAct1 = 1024,
		interlaced = false,
		rateMode = VIRATE_VGA,
		tmdsFreq = 108000000,
		spec_adv7611_mode = 0x05,
		spec_amba_mode = AMBA_VIDEO_MODE_SXGA,
		pixelRepeat = false,
		aspect_ratio = RATIO_AUTO,
		video_system = VIDEO_SYSTEM_AUTO
	},   
	
--1280x1024p75
	{
		standardId = {86},
		formatId = "1280x1024p75",
		formatName = "1280x1024@75Hz",
		frameRate = 75,
		hTotal = 1688,
		hAct = 1280,
		vTotal0 = 1066,
		vAct0 = 1024,
		vTotal1 = 1066,
		vAct1 = 1024,
		interlaced = false,
		rateMode = VIRATE_VGA,
		tmdsFreq = 135000000,
		spec_adv7611_mode = 0x06,
		spec_amba_mode = AMBA_VIDEO_MODE_SXGA,
		pixelRepeat = false,
		aspect_ratio = RATIO_AUTO,
		video_system = VIDEO_SYSTEM_AUTO
	}, 
}

function _M.Init( graphSystem )
	if graphSystem and graphSystem == "STDI" then
		local x
		for x=1,#_M.format_map do
			_M.format_map[x].BL = MAIN_FREQ / (( _M.format_map[x].vTotal0 * _M.format_map[x].frameRate)/8)
			_M.format_map[x].LCF = _M.format_map[x].vTotal0
			_M.format_map[x].FCL = MAIN_FREQ / _M.format_map[x].frameRate / 256
			_M.format_map[x].LCVS = 0 --TODO: Now ignore this argument
		end
		
		_M.STDI_Initialized = true
	end
end

function _M.Standard_Match( standardId )
	local x, y
	for x=1, #_M.format_map do
		if _M.format_map[x].standardId then
			for y = 1, #_M.format_map[x].standardId do
				if _M.format_map[x].standardId[y] == standardId then
					return _M.format_map[x]
				end
			end
		end
	end
	return nil, "No matched"
end

local function NTSCFormatDoubleCheck(idx, FCL)
	if FCL <= 0 or _M.format_map[idx].rateMode == VIRATE_VGA then
		return idx
	end

	local frate = _M.format_map[idx].frameRate
	local tmdsFreq = _M.format_map[idx].tmdsFreq
	if tmdsFreq >= 74000000 and (frate == 60 or frate == 30 or frate == 24) then
		local freq = MAIN_FREQ/256/FCL
		if freq <= (frate - 0.02) then
			return idx + 1
		else
			return idx
		end
	else
		return idx --no care
	end
end


function _M.STDI_Match( LCF, LCVS, FCL, BL, interlaced )
	if _M.STDI_Initialized then
		local x
		local diff_BL, diff_LCF, diff_FCL, diff_LCVS
		for x=1, #_M.format_map do
			diff_BL = math.abs( BL - _M.format_map[x].BL )
			diff_LCF = math.abs( LCF - _M.format_map[x].LCF )
			diff_FCL = math.abs( FCL - _M.format_map[x].FCL )
			diff_LCVS = 0	--TODO: Fix this
			if diff_BL <= 20 and diff_LCF <=2 and diff_FCL <=10 and diff_LCVS <=2 and ( nil == interlaced or interlaced == _M.format_map[x].interlaced ) then
				return _M.format_map[NTSCFormatDoubleCheck(x,FCL)]
			end
		end
		return nil, "No matched"
	else
		return nil, "No initialized STDI"
	end
end

function _M.Format_Match( tmdsFreqMHz , total_w, act_w, total_v0, act_v0, interlaced, total_v1, act_v1 )
	local diff_tmds, diff_tw, diff_w, diff_tv0, diff_v0, diff_tv1, diff_v1
	local x
	
	for x=1, #_M.format_map do
		if not tmdsFreqMHz or tmdsFreqMHz <= 0 then
			diff_tmds = 0
		else
			diff_tmds = math.abs( tmdsFreqMHz - _M.format_map[x].tmdsFreq/1000000 )
		end
		diff_tw = math.abs( total_w - _M.format_map[x].hTotal )
		diff_w = math.abs( act_w - _M.format_map[x].hAct )
		diff_tv0 = math.abs( total_v0 - _M.format_map[x].vTotal0 )
		diff_v0 = math.abs( act_v0 - _M.format_map[x].vAct0 )
		if interlaced and total_v1 and act_v1 then
			diff_tv1 = math.abs( total_v1 - _M.format_map[x].vTotal1 )
			diff_v1 = math.abs( act_v1 - _M.format_map[x].vAct1 )
		else
			diff_tv1 = 0
			diff_v1 = 0
		end
		
		if diff_tmds <=2 and
			diff_tw <= 8 and
			diff_w <= 4 and
			diff_tv0 <= 8 and
			diff_v0 <= 4 and
			( nil == interlaced or interlaced == _M.format_map[x].interlaced ) and
			diff_tv1 <= 8 and
			diff_v1 <= 4 then
				return _M.format_map[x]
		end
	end
	
	return nil, "No matched"
end

function _M.SetMainFreq( freq )
	MAIN_FREQ = freq
end

return _M
