local coaf=require "coaf.core"
local _dbg = require "coaf.debug"

local periphery = require "periphery"
local I2C = periphery.I2C

local I2C_REG = {
}

function I2C_REG:Open( i2c_dev )
	if not i2c_dev then
		return false
	end

	self.i2c_device = i2c_dev

	local done,err = pcall( function() self.i2c_handle = I2C( i2c_dev ) end )
	if not done then
		if type(err) == "table" then
			_dbg.Info("I2C-REG", "ERROR: Could not open I2C device for register access. ErrMsg:", err.message or "---" )
		else
			_dbg.Info("I2C-REG", "ERROR: Could not open I2C device for register access. ErrMsg:", err )
		end
		return false, err
	else
		return true
	end
end
	
-------------------------------------------------------
-- Object method: Register read
-------------------------------------------------------
function I2C_REG:ReadReg( map, addr, itemArr )
	if not self.i2c_handle then
		return nil, "Invalid I2C handle"
	end
	if not itemArr or type(itemArr) ~= "table" then
		itemArr = { 0xff, flags = I2C.I2C_M_RD }
	else
		if #itemArr < 1 then
			itemArr[1] = 0xff
		end
		if not itemArr.flags then
			itemArr.flags = I2C.I2C_M_RD
		else
			itemArr.flags = itemArr.flags | I2C.I2C_M_RD
		end
	end

	local msg = { {addr}, itemArr }
	local done,err = pcall( function() self.i2c_handle:transfer( map, msg ) end )
	if not done then
		if type(err) == "table" then
			--_dbg.Info("I2C-REG", string.format( "ReadReg fail: 0x%02x [0x%02x], Error:", map, addr), err.message or "---" )
			return nil, err.message or "---"
		else
			--_dbg.Info("I2C-REG", string.format( "ReadReg fail: 0x%02x [0x%02x], Error:", map, addr), err )
			return nil, err
		end
	else
		return itemArr
	end
end

------------------------------------------------------
-- Register write
------------------------------------------------------
function I2C_REG:WriteReg( map, addr, itemArr )
	if not self.i2c_handle then
		return false, "Invalid I2C handle"
	end
	if not itemArr or type(itemArr) ~= "table" or #itemArr < 1 then
		return false, "Invalid data"
	end

	table.insert( itemArr, 1, addr )
	local msg = { itemArr }
	local done,err = pcall( function() self.i2c_handle:transfer( map, msg ) end )
	if not done then
		if type(err) == "table" then
			--_dbg.Info("I2C-REG", string.format( "WriteReg fail: 0x%02x [0x%02x], Error:", map, addr), err.message or "---" )
			return nil, err.message or "---"
		else
			--_dbg.Info("I2C-REG", string.format( "WriteReg fail: 0x%02x [0x%02x], Error:", map, addr), err )
			return nil, err
		end
	else
		return true
	end
end

I2C_REG.__index = I2C_REG

--------------------------------------------------

local _M = {}

function _M.Create( i2c_dev )
	local obj = {
		i2c_handle = nil
	}

	setmetatable( obj, I2C_REG )
	if i2c_dev then
		obj:Open( i2c_dev )
	end
	return obj
end

return _M

