local periphery = require "periphery"
local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local client = require "coaf.client"
local coaf = require "coaf.core"

require "features"
local special_controls 
if _G.DEVICE_TYPE == "A1" then
	special_controls = require( "control_" .. _G.OEM_TYPE )
else
	special_controls = require ( "control_" .. (_G.DEVICE_TYPE or "NULL") )
end

local restore_module = require ( "kiloview." .. (_G.DEVICE_TYPE or "G1") .. ".restoreFactory" )

local I2C = periphery.I2C
local GPIO = periphery.GPIO
local SPI = periphery.SPI
------------------------------

local GPLED_OFF = 0
local GPLED_ON = 1
local GPLED_BLINK = 0x03		-- 0011
local GPLED_BLINK_FAST = 0x05	-- 0101
local GPLED_BLINK_COUNTER = 4

------------------------------

local CONTROL_OPTIONS = _G.IOLED_OPTIONS or { }

------------------------------
local ROLLING_SEQ = CONTROL_OPTIONS.ROLLING_SEQ or { "signal", "running" }
local ROLLING_LANG_STATUS = CONTROL_OPTIONS.ROLLING_LANG_STATUS or {
		[0x01] = "BLINK",
		[0x04] = "BLINK",
		[0x00] = "BLINK-FAST"
}
local RESET_HOLD = CONTROL_OPTIONS.RESET_HOLD or 5000
------------------------------

local GPIO_LEDS = {
	step = 0,
}
-------------------------
local FPGA_LEDS= {
	status = {},
	forced = {}
}
--------------------------
local FPGA_LED_STATUS_MAP = {
	["OFF"] = { 0xff, 0x00, 0x00 },
	["ON"] = { 0xff, 0x00, 0x01 },	--Green
	["BLINK"] = { 0xff, 0x00, 0x02 },	--Green
	
	["RED:ON"] = { 0xff, 0x01, 0x00 },	--Red
	["RED:BLINK"] = { 0xff, 0x02, 0x00 },	--Red
	
	["GREEN:ON"] = { 0xff, 0x00, 0x01 },	--Green
	["GREEN:BLINK"] = { 0xff, 0x00, 0x02 },	--Green
	
	["YELLOW:ON"] = { 0xff, 0x01, 0x01 },	--Yellow
	["YELLOW:BLINK"] = { 0xff, 0x02, 0x02 } --Yellow
}

-------------------------------

local _M = {
	fpga_i2c = nil,
	oled_spi = nil,
	oled_dc_pin = nil,

	lastIsRolling = false,
	resetButton = nil,
	resetHoldTimes = 0,

	pushButton = nil,
	pushButtonPressed = false,
	
	isRolling = false,
	rollingStatus = "ON",
	rollingIndex = 1,
	rollingNums = 1,

	userControl = true
}

-------------------------------
local function GetCoAFProxy( objPath, service, loc, timeout )
	local proxy
	local cli
	cli = client.New( service, loc or "local", nil, timeout or 3 ) --3 seconds timeout.
	if not cli then
		_dbg.Error( "IOLED", "Fail to create connection to service: " .. (loc or "local") .. "/" .. (service or "nil") )
		return nil
	end
	proxy = cli:CreateProxy(objPath, true) --Exclusive
	if not proxy then
		_dbg.Error( "IOLED", "Fail to create object proxy for '" .. (objPath or "unknown") .. "'" )
		return nil
	end
	return proxy
end
-------------------------------
local function GPIO_LedControl( id, status, noStore )
	local item = GPIO_LEDS[id]
	if item then
		status = status:upper()
		local ledStatus
		if status:match("OFF") then
			ledStatus = GPLED_OFF
		elseif status:match("ON") then
			ledStatus = GPLED_ON
		elseif status:match("FAST") then
			ledStatus = GPLED_BLINK_FAST
		elseif status:match("BLINK") then
			ledStatus = GPLED_BLINK
		else
			ledStatus = GPLED_OFF
		end
		if not item.forced or noStore then
			item.status = ledStatus
		end
		
		if not noStore then
			item.save_status = item.status
		end
	end
end
---------------------------
local function GPIO_LedForced( id, b )
	local item = GPIO_LEDS[id]
	if item then
		if not b then
			item.status = item.save_status
			item.forced = false
		else
			item.forced = true
		end
	end
end
---------------------------
local function FPGA_LedControl( id, status, noStore )
	local reg = FPGA_LEDS[id]
	if not reg then
		return
	end
	
	status = status:upper()
	local regvals = FPGA_LED_STATUS_MAP[status]
	if not regvals then
		return
	end
	
	if not noStore then
		--Save the status
		FPGA_LEDS.status[id] = status
	end
	
	if not FPGA_LEDS.forced[id] or noStore then
		if _M.fpga_i2c and CONTROL_OPTIONS.FPGA_DEV_ADDR then
			regvals[1] = reg	--Replace the reigster address
			local msg = { regvals }
			pcall(function()
				_M.fpga_i2c:transfer( CONTROL_OPTIONS.FPGA_DEV_ADDR>>1, msg )
			end)
		end
	end
end
------------------------------
local function FPGA_LedForced( id, b )
	local status = FPGA_LEDS.status[id]
	if status then
		FPGA_LEDS.forced[id] = b
		if not b then
			FPGA_LedControl( id, status, true )
		end
	end
end
------------------------------
local function LED_CONTROL( id, status, noStore )
	if CONTROL_OPTIONS.GPIO_LEDS and CONTROL_OPTIONS.GPIO_LEDS[id] then
		GPIO_LedControl( id, status, noStore )
	elseif CONTROL_OPTIONS.FPGA_LEDS and CONTROL_OPTIONS.FPGA_LEDS[id] then
		FPGA_LedControl( id, status, noStore )
	end
end
-------------------------------
local function LED_FORCED( id, b )
	if CONTROL_OPTIONS.GPIO_LEDS and CONTROL_OPTIONS.GPIO_LEDS[id] then
		GPIO_LedForced( id, b )
	elseif CONTROL_OPTIONS.FPGA_LEDS and CONTROL_OPTIONS.FPGA_LEDS[id] then
		FPGA_LedForced( id, b )
	end	
end
-------------------------------

function _M.Init()
	--忽略非1.6版本底板的WIFI灯控制
	local done,hwChk = pcall( require, "kiloview." .. _G.DEVICE_TYPE .. ".hardwareCheck" )
	if done and type(hwChk) == "table" then
		hwChk.Check()
		if _G.DEVICE_TYPE == "P1" and hwChk.hardwareVersionCode ~= 0x13 then
			if type(CONTROL_OPTIONS.GPIO_LEDS) == "table" and CONTROL_OPTIONS.GPIO_LEDS.wifi then
				CONTROL_OPTIONS.GPIO_LEDS.wifi = nil
			end
		end
		if _G.DEVICE_TYPE == "P2" and hwChk.hardwareVersionCode ~= 0x18 then
			if type(CONTROL_OPTIONS.GPIO_LEDS) == "table" and CONTROL_OPTIONS.GPIO_LEDS.wifi then
				CONTROL_OPTIONS.GPIO_LEDS.wifi = nil
			end
		end	
	end

	if CONTROL_OPTIONS.WITH_FPGA and  CONTROL_OPTIONS.FPGA_DEV_ADDR and  CONTROL_OPTIONS.FPGA_DEV_ADDR > 0 then
		pcall(function()
			_M.fpga_i2c = I2C( CONTROL_OPTIONS.FPGA_I2C_DEV or "/dev/i2c-3" )
		end)
		if not _M.fpga_i2c then
			print( "WARNING: Could not open FPGA I2C controller!" )
		end
	end

	if CONTROL_OPTIONS.OLED_SCREEN then
		_M.oled_spi = SPI( CONTROL_OPTIONS.OLED_SCREEN.DEVICE, 0, 1e6)
		if not _M.oled_spi then
			print( "WARNING: Could not open OLED SPI controller!" )
		else
			_M.dc_pin = GPIO( CONTROL_OPTIONS.OLED_SCREEN.DC_PIN, "out" )
			if not _M.dc_pin then
				print("WARNING: Could not open OLED DC# control pin!" )
			else
				_M.dc_pin:write( false ) --Default output low.
			end
		end
	end

	local k,v
	if type(CONTROL_OPTIONS.GPIO_LEDS) == "table" then
		for k,v in pairs(CONTROL_OPTIONS.GPIO_LEDS) do
			if type(v) == "table" then
				GPIO_LEDS[ k ] = {
					pin = v.pin,
					status = GPLED_OFF,
					save_status = GPLED_OFF
				}
				
				if v.iodrv then
					v.iodrv.Init()
					GPIO_LEDS[ k ].pin_ctrl = v.iodrv.Create( v.pin )
				else
					GPIO_LEDS[ k ].pin_ctrl = GPIO( v.pin, "out" )
				end

				if not GPIO_LEDS[ k ].pin_ctrl then
					print( "WARNING: Could not open GPIO LED pin of " .. v )
				end

			else
				GPIO_LEDS[ k ] = {
					pin = v,
					status = GPLED_OFF,
					save_status = GPLED_OFF
				}
				
				GPIO_LEDS[ k ].pin_ctrl = GPIO( v, "out" )
				if not GPIO_LEDS[ k ].pin_ctrl then
					print( "WARNING: Could not open GPIO LED pin of " .. v )
				end
			end
		end
	end
	
	if type(CONTROL_OPTIONS.FPGA_LEDS) == "table" then
		for k,v in pairs(CONTROL_OPTIONS.FPGA_LEDS) do
			FPGA_LEDS[k] = v
			FPGA_LEDS.status[k] = "OFF"
		end
	end

	if CONTROL_OPTIONS.RESET_PIN and CONTROL_OPTIONS.RESET_PIN >= 0 then
		_M.resetButton = GPIO( CONTROL_OPTIONS.RESET_PIN, "in" )
		if not _M.resetButton then
			print( "WARNING: Could not open RESET pin of " .. CONTROL_OPTIONS.RESET_PIN )
		end
	end

	if CONTROL_OPTIONS.PUSH_STOP_PIN and CONTROL_OPTIONS.PUSH_STOP_PIN >= 0 then
		_M.pushButton = GPIO( CONTROL_OPTIONS.PUSH_STOP_PIN, "in" )
		if not _M.pushButton then
			print( "WARNING: Could not open RESET pin of " .. CONTROL_OPTIONS.PUSH_STOP_PIN )
		end
	end

	--Remap the default FPGA led status registers.
	if type(CONTROL_OPTIONS.FPGA_LED_STATUS_MAP) == "table" then
		print( "*** Control module has special FPGA register maps." )
		FPGA_LED_STATUS_MAP = CONTROL_OPTIONS.FPGA_LED_STATUS_MAP
	end

	for k,v in pairs( CONTROL_OPTIONS.ACTION_STATUS.default ) do
		LED_CONTROL( k, v )
	end
end
-------------------------------
function _M.UpdateRolling()
--[[	
	local multiLangs = true
	local defaultLangs = 0x00
	if _M.fpga_i2c then
		local msg = { {0xf2}, {0x00, 0x00, 0x00, 0x00, flags = I2C.I2C_M_RD} }
		_M.fpga_i2c:transfer( CONTROL_OPTIONS.FPGA_DEV_ADDR >> 1, msg )
		local rslt = msg[2]
		if rslt[1] == 0x01 or rslt[1] == 0x02 or rslt[1] == 0x04 then
			multiLangs = false
		else
			multiLangs = true
		end
		
		if rslt[3] == 0x01 or rslt[3] == 0x04 then
			defaultLangs = rslt[3]
		else
			defaultLangs = 0x00
		end
	end
]]

	_M.rollingStatus = CONTROL_OPTIONS.ROLLING_LANG_STATUS[0] or "BLINK"
	_M.rollingIndex = 1
	_M.rollingNums = CONTROL_OPTIONS.ROLLING_MIN_NUMS or 1

--[[
	if multiLangs then	--Multi-language supports, so with more leds
		_M.rollingNums = _M.rollingNums + 1
	end
]]
end
-------------------------------
function _M.Check(period_ms)
	period_ms = period_ms or 250	--250ms defaults
	
	if special_controls then
		special_controls.Check(period_ms)
	end
	
	local k,v
	for k,v in pairs(GPIO_LEDS) do
		if type(v) == "table" and v.pin_ctrl then
			if v.status == GPLED_OFF then
				v.pin_ctrl:write( true )
			elseif v.status == GPLED_ON then
				v.pin_ctrl:write( false )
			else
				local mask = (1 << GPIO_LEDS.step)
				if (v.status & mask) ~= 0 then
					v.pin_ctrl:write( false )
				else
					v.pin_ctrl:write( true )
				end
			end -- Status
		end -- Pin control valid
	end -- For
	
	GPIO_LEDS.step = GPIO_LEDS.step + 1
	if GPIO_LEDS.step >= GPLED_BLINK_COUNTER then
		GPIO_LEDS.step = 0
	end
	
	-- Check the reset button
	if _M.resetButton then
		local st = _M.resetButton:read()
		if not st then	--Reset hold
			_M.resetHoldTimes = _M.resetHoldTimes + 1
			if (_M.resetHoldTimes*period_ms) >= RESET_HOLD then
				_M.Reset( "restore" )
				_M.DoRestore()
			elseif not _M.isRolling then
				_M.isRolling = true
				_M.UpdateRolling()	--Update rolling status
			end
		else
			_M.resetHoldTimes = 0
			_M.isRolling = false
		end
	end

	-- Check the push button
	if _M.pushButton then
		local st = _M.pushButton:read()
		-- print("st",st)
		if not st and not _M.pushButtonPressed then --忽略5秒内的重复按键
			_M.pushButtonPressed = true
			_M.DoPUSHSTOP()
			coaf.ScheduleDelayedTask( function()
				_M.pushButtonPressed = false
			end, 5)
		end
	end

	-- Rolling
	if _M.isRolling then
		_M.lastIsRolling = true
		local i, id
		
		--Kick off all first
		for i=1, #ROLLING_SEQ do
			LED_FORCED( ROLLING_SEQ[i], true )
			LED_CONTROL( ROLLING_SEQ[i], "OFF", true )	-- Not store status
		end
		for i=1, _M.rollingNums do 
			id = _M.rollingIndex + i - 1
			if id > #ROLLING_SEQ then
				id = id%(#ROLLING_SEQ)
			end
			LED_CONTROL( ROLLING_SEQ[id], _M.rollingStatus, true ) --Not store status
		end
		
		_M.rollingIndex = _M.rollingIndex + 1
		if _M.rollingIndex > #ROLLING_SEQ then
			_M.rollingIndex = 1
		end
	elseif _M.lastIsRolling then
		_M.lastIsRolling = false	--Mark no rolling
		--Restore status saved
		local i
		for i=1, #ROLLING_SEQ do
			LED_FORCED( ROLLING_SEQ[i], false )
		end
	end
end
-------------------------------
function _M.DoRestore()
	if restore_module then
		local action = restore_module.RestoreFactory()
		if action == "reboot" then
			os.execute( "/sbin/reboot" )
			os.exit(0)	--Quit this application
		end --Else ... TODO
	end
end
-------------------------------
function _M.Reset( action )
	action = action or "normal"
	if action == "normal" then
		local k
		for k in pairs(CONTROL_OPTIONS.GPIO_LEDS) do
			LED_FORCED( k, false )
		end
		
		for k in pairs(CONTROL_OPTIONS.FPGA_LEDS) do
			LED_FORCED( k, false )
		end
	else
		local ctrls = CONTROL_OPTIONS.ACTION_STATUS[action]
		if not ctrls then
			ctrls = CONTROL_OPTIONS.ACTION_STATUS["default"]
			if not ctrls then
				return
			end
		end
		local k,v
		for k,v in pairs(ctrls) do
			LED_CONTROL( k, v, true )		--Not save
		end
	end
end
-------------------------------
function _M.LedControl( id, status )
	LED_CONTROL( id, status )
end
-------------------------------
function _M.SetUserControl( en )
	_M.userControl = en
end
-------------------------------
function _M.DoPUSHSTOP()
	local svcProxy = GetCoAFProxy( "/", "streamer")
	if not svcProxy then
		_dbg.Error( "IOLED", "Could not get streamer proxy" )
	else
		local r,done = svcProxy:PushControl()
		svcProxy:Destroy()
		if not r or not done then
			_dbg.Error( "IOLED", "Push Control failed!" )
		end
	end
end
-------------------------------
return _M
