local checker = require "statusChecker"

local _M = {}

--The G1 device IO/LEDs control parameters
_G.IOLED_OPTIONS = 
{
	WITH_FPGA = true,	--With FPGA
	FPGA_DEV_ADDR = 0x34,	--FPGA I2C device address
	WITH_FPGA_LEDS	= true,	--With FPGA LEDs

	OLED_SCREEN = {
		CONTROL = "spi",
		DEVICE = "/dev/spidev0.0",
		DC_PIN = 17
	},

	------
	RESET_PIN = 59,	--GPIO59 as RESET button
	------
	GPIO_LEDS = {
	},
	------
	FPGA_LEDS = {
		["record"] = 0x12,	--Register address [ all same bellow ]
		["power"] = 0x10,
	},
	------
	--NOTE: For compatible reason, though the LED is green, but set it be RED.
	FPGA_LED_STATUS_MAP= {
		["OFF"] = { 0xff, 0x00 },
		["ON"] = { 0xff, 0x01 },	--Green
		["BLINK"] = { 0xff, 0x02 },	--Green
		
		["RED:ON"] = { 0xff, 0x01 },	--Green
		["RED:BLINK"] = { 0xff, 0x02 },	--Green

		["GREEN:ON"] = { 0xff, 0x01 },	--Green
		["GREEN:BLINK"] = { 0xff, 0x02 },	--Green

		["YELLOW:ON"] = { 0xff, 0x01 },	--Green
		["YELLOW:BLINK"] = { 0xff, 0x02 },	--Green
	},
	------
	--Rolling options: When pressed (but no arrived reset time) RESET button, rolling LEDs
	ROLLING_SEQ = { "record", "power" },
	
	ROLLING_LANG_STATUS = {
		[0x01] = "BLINK",		--Default language is CN
		[0x04] = "BLINK",	--Default language is EN
		[0x00] = "BLINK"	--Other languages
	},
	
	ROLLING_MIN_NUMS = 2,	--How many leds at least show when rolling (Multiple languages support should +1)
	------
	ACTION_STATUS = {
		["default"] = {
			["power"] = "ON",
			["record"] = "OFF"
		},
	
		["restore"] = {
			["power"] = "BLINK",
			["record"] = "BLINK"
		},
	
		["reboot"] = {
			["power"] = "BLINK",
			["record"] = "OFF"
		},
		
		["reset"] = {
			["power"] = "BLINK",
			["record"] = "OFF"
		}
	},
	
	RESET_HOLD = 5000	--Hold RESET button more than this value(milliseconds), should do restore factory action!
}

local ctrl_module = nil
local oled_show = false

local function OledShowDefault(ctrlM)
	
	if not ctrlM.dc_pin or not ctrlM.oled_spi then
		return
	end

	ctrlM.dc_pin:write( false )
	ctrlM.oled_spi:transfer({0xd5,0x80})	--Set Display Clock Divide Ratio/Oscillator Frequency
	ctrlM.oled_spi:transfer({0xa8,0x3f})	--Set Multiplex Ratio
	ctrlM.oled_spi:transfer({0xd3,0x00})	--Set Display Offset
	ctrlM.oled_spi:transfer({0x40})		--Set Display Start Line
	ctrlM.oled_spi:transfer({0xa1})		--Set Segment Re-Map
	ctrlM.oled_spi:transfer({0xc8})		--Set COM Output Scan Direction
	ctrlM.oled_spi:transfer({0xda,0x12})	--Set COM Pins Hardware Configuration
	ctrlM.oled_spi:transfer({0x81,0x66})	--Set Contrast Control
	ctrlM.oled_spi:transfer({0xd9,0xF1})	--Set Pre-Charge Period
	ctrlM.oled_spi:transfer({0xdb,0x30})	--Set VCOMH Deselect Level
	ctrlM.oled_spi:transfer({0xa4})		--Set Entire Display On/Off
	ctrlM.oled_spi:transfer({0xa6})		--Set Normal/Inverse Display
	ctrlM.oled_spi:transfer({0x8d,0x14})	--Set Charge Pump
	
	ctrlM.oled_spi:transfer({0x20,0x00})	--Set Memory Addressing Mode: 0x02 = Page Address
	ctrlM.oled_spi:transfer({0x22,0x00,0x07})	--Set Page Address: 0~7
	
	ctrlM.dc_pin:write( true )
	local file = io.open( "image.bin", "rb" )
	if file then
		local bytes = file:read("a")
		file:close()
		ctrlM.oled_spi:transfer({bytes:byte(1,#bytes)})
	end
	ctrlM.dc_pin:write( false )
	ctrlM.oled_spi:transfer({0xaf})		--Set Display On
end

function _M.Check(period_ms)
	if not ctrl_module then
		ctrl_module = require( "controlModule" )
	end
	
	if not ctrl_module or ctrl_module.userControl then
		return
	end
	
	if not oled_show then
		OledShowDefault( ctrl_module )
		oled_show = true
	end

	if not checker then
		return
	end
end

-----------

return _M
