local checker = require "statusChecker"
local periphery = require "periphery"
local I2C = periphery.I2C

local _M = {}

local EXT_GPIO = {
	I2C_DEV = "/dev/i2c-2",
	I2C_ADDR = 0x40,
	STATUS = {0xff, 0xff},
	i2c_handle = nil
}

EXT_GPIO.Init = function()
	if not EXT_GPIO.i2c_handle then
		EXT_GPIO.i2c_handle = I2C( EXT_GPIO.I2C_DEV )
		if not EXT_GPIO.i2c_handle then
        	print( "ERROR: Open ext-gpio i2c fail!" )
			return
		end
	end
end

EXT_GPIO.Create = function(pin)
	if pin < 0 or pin > 15 then
		print( "ERROR: ext-gpio: Invalid PIN id." )
		return nil
	end

	if not EXT_GPIO.i2c_handle then
		print( "ERROR: Un-initialized EXT_GPIO module!" )
		return nil
	end

	local obj = {
		pin = pin
	}

	function obj:read()
		local msg = { {0x00, 0x00, flags = I2C.I2C_M_RD} }
    	EXT_GPIO.i2c_handle:transfer( EXT_GPIO.I2C_ADDR >> 1, msg )
		if self.pin < 8 then
			return (msg[1][1] & (1 << self.pin)) ~= 0
		else
			return (msg[1][2] & (1 << (self.pin-8))) ~= 0
		end
	end

	function obj:write(val)
		if self.pin < 8 then
			if val then
				EXT_GPIO.STATUS[1] = EXT_GPIO.STATUS[1] | (1 << self.pin)
			else
				EXT_GPIO.STATUS[1] = EXT_GPIO.STATUS[1] & ( ~(1<<self.pin) & 0xff )
			end
		else
			if val then
				EXT_GPIO.STATUS[2] = EXT_GPIO.STATUS[2] | (1 << (self.pin-8))
			else
				EXT_GPIO.STATUS[2] = EXT_GPIO.STATUS[2] & ( ~(1<<(self.pin-8)) & 0xff )
			end
		end
		EXT_GPIO.i2c_handle:transfer( EXT_GPIO.I2C_ADDR >> 1, { EXT_GPIO.STATUS } )
	end

	return obj
end


--The G1 device IO/LEDs control parameters
_G.IOLED_OPTIONS = 
{
	WITH_FPGA = true,	--With FPGA controls
	FPGA_DEV_ADDR = 0x34,	--FPGA I2C device address
	WITH_FPGA_LEDS	= false,	--Without FPGA LEDs

	OLED_SCREEN = {
		CONTROL = "spi",
		DEVICE = "/dev/spidev0.0",
		DC_PIN = 17
	},

	------
	RESET_PIN = 9,	--GPIO9 as RESET button
	------
	GPIO_LEDS = {
		["signal"] = { pin = 7, iodrv = EXT_GPIO },
		["stream"] = { pin = 6, iodrv = EXT_GPIO },
		["old_off_screen"] = { pin = 13, iodrv = EXT_GPIO },
		["screen"] = { pin = 14, iodrv = EXT_GPIO },
		["reset_screen"] = { pin = 15, iodrv = EXT_GPIO },
		["stop_fan"] = 58
	},
	------
	FPGA_LEDS = {
	},
	------
	
	--Rolling options: When pressed (but no arrived reset time) RESET button, rolling LEDs
	ROLLING_SEQ = { "signal", "stream" },
	
	ROLLING_LANG_STATUS = {
		[0x01] = "BLINK",	--Default language is CN
		[0x04] = "BLINK",	--Default language is EN
		[0x00] = "BLINK"	--Other languages
	},
	
	ROLLING_MIN_NUMS = 2,	--How many leds at least show when rolling (Multiple languages support should +1)
	------
	ACTION_STATUS = {
		["default"] = {
			["signal"] = "OFF",
			["stream"] = "OFF",
			["screen"] = "OFF",
			["old_off_screen"] = "ON",
			["reset_screen"] = "OFF",
			["stop_fan"] = "OFF",
		},
	
		["restore"] = {
			["signal"] = "BLINK",
			["stream"] = "BLINK",
			["old_off_screen"] = "ON",
		},
	
		["reboot"] = {
			["signal"] = "BLINK",
			["stream"] = "OFF",
			["old_off_screen"] = "ON",
		},
		
		["reset"] = {
			["signal"] = "BLINK",
			["stream"] = "OFF",
			["old_off_screen"] = "ON",
		}
	},
	
	RESET_HOLD = 5000	--Hold RESET button more than this value(milliseconds), should do restore factory action!
}

local ctrl_module = nil
local oled_show = false

local function OledShowDefault(ctrlM)
	if not ctrlM.dc_pin or not ctrlM.oled_spi then
		return
	end
	ctrlM.dc_pin:write( false )                      
	ctrlM.oled_spi:transfer({0xae})
	ctrlM.oled_spi:transfer({0xb0})
	ctrlM.oled_spi:transfer({0x00})
	ctrlM.oled_spi:transfer({0x10})
	ctrlM.oled_spi:transfer({0x00})
	ctrlM.oled_spi:transfer({0xd5})
	ctrlM.oled_spi:transfer({0x50})
	ctrlM.oled_spi:transfer({0xd9})
	ctrlM.oled_spi:transfer({0x22})
	ctrlM.oled_spi:transfer({0x40})
	ctrlM.oled_spi:transfer({0x81})
	ctrlM.oled_spi:transfer({0x01})
	ctrlM.oled_spi:transfer({0xa0})
	ctrlM.oled_spi:transfer({0xc0})
	ctrlM.oled_spi:transfer({0xa4})
	ctrlM.oled_spi:transfer({0xa6})
	ctrlM.oled_spi:transfer({0xa8})
	ctrlM.oled_spi:transfer({0x3f})
	ctrlM.oled_spi:transfer({0xad})
	ctrlM.oled_spi:transfer({0x80})
	ctrlM.oled_spi:transfer({0xd3})
	ctrlM.oled_spi:transfer({0x00})            
	ctrlM.oled_spi:transfer({0xdb})
	ctrlM.oled_spi:transfer({0x30})
	ctrlM.oled_spi:transfer({0xdc})
	ctrlM.oled_spi:transfer({0x30})
	ctrlM.oled_spi:transfer({0x33})

	--ctrlM.oled_spi:transfer({0xaf})
	local file = io.open( "image.bin", "rb" )
	if file then
		local bytes = file:read("a")
		file:close()
		local i,j,n,k
		local v = {}
		local bb = {}
		local b1,b2

		for i=0,7 do
			for n=0,127,2 do
				b1 = bytes:byte(i*128 + n + 1, i*128 + n + 1)
				b2 = bytes:byte(i*128 + n + 2, i*128 + n + 2)

				j = math.floor(n/2) + 1

				bb[1] = ((b1&0x01) ~= 0 and 0x80 or 0x00)    
				bb[1] = bb[1] | ((b2&0x01) ~= 0 and 0x08 or 0x00)
				bb[2] = ((b1&0x02) ~= 0 and 0x80 or 0x00)
				bb[2] = bb[2] | ((b2&0x02) ~= 0 and 0x08 or 0x00)
				bb[3] = ((b1&0x04) ~= 0 and 0x80 or 0x00)              
				bb[3] = bb[3] | ((b2&0x04) ~= 0 and 0x08 or 0x00)
				bb[4] = ((b1&0x08) ~= 0 and 0x80 or 0x00)              
				bb[4] = bb[4] | ((b2&0x08) ~= 0 and 0x08 or 0x00)
				bb[5] = ((b1&0x10) ~= 0 and 0x80 or 0x00)              
				bb[5] = bb[5] | ((b2&0x10) ~= 0 and 0x08 or 0x00)
				bb[6] = ((b1&0x20) ~= 0 and 0x80 or 0x00)              
				bb[6] = bb[6] | ((b2&0x20) ~= 0 and 0x08 or 0x00)                  
				bb[7] = ((b1&0x40) ~= 0 and 0x80 or 0x00)              
				bb[7] = bb[7] | ((b2&0x40) ~= 0 and 0x08 or 0x00)
				bb[8] = ((b1&0x80) ~= 0 and 0x80 or 0x00)              
				bb[8] = bb[8] | ((b2&0x80) ~= 0 and 0x08 or 0x00)

				if i <= 1 then
					if n < 64 then
						for k=0,7 do
							v[k*128 + j] = bb[k+1]
							v[k*128 + 64 + j] = 0
						end
					else
						for k=0,7 do
							v[k*128 + j] = 0
							v[k*128 + 64 + j] = bb[k+1]
						end
					end
				else
					for k=0,7 do
						v[k*128 + math.ceil(j/2)] = 0
						v[k*128 + 32 + j] = bb[k+1]
						v[k*128 + 96 + math.ceil(j/2)] = 0
					end
				end
			end                                                                    
			ctrlM.dc_pin:write( false )                                            
			ctrlM.oled_spi:transfer({0xb0,i*8})                                    
			ctrlM.oled_spi:transfer({0x00,0x10})                                   

			ctrlM.dc_pin:write( true )                                             
			ctrlM.oled_spi:transfer(v)                                            
		end                                                                            
	end                                                                                    
	ctrlM.dc_pin:write( false )                                                                    
	ctrlM.oled_spi:transfer({0xaf})         --Set Display On                               

end

function _M.Check(period_ms)
	if not ctrl_module then
		ctrl_module = require( "controlModule" )
	end
	
	if not ctrl_module or ctrl_module.userControl then
		return
	end
	
	if not checker then
		return
	end


	if not oled_show then
		OledShowDefault( ctrl_module )
		oled_show = true
		ctrl_module.LedControl( "screen", "ON" )
		ctrl_module.LedControl( "old_off_screen", "OFF" )
	end


	--Signal checking
	local signal = checker.CheckSignal()
	if signal then
		ctrl_module.LedControl( "signal", "ON" )
	else
		ctrl_module.LedControl( "signal", "OFF" )
	end

	--Streaming checking
	local status = checker.CheckStreaming()
	local ledStatus
	if status == "OFF" then
		ledStatus = "OFF"
	elseif status == "CONNECTING" then
		ledStatus = "GREEN:BLINK"
	elseif status == "CONNECTED" then
		ledStatus = "YELLOW:ON"
	elseif status == "PUSHING" then
		ledStatus = "GREEN:ON"
	elseif status == "STOPPED" then
		ledStatus = "YELLOW:BLINK"
	else
		ledStatus = "RED:ON"
	end
	ctrl_module.LedControl( "stream", ledStatus )

end

-----------

return _M
