local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local run_daemon = require "coaf.run_daemon"

local _M = {
	dhcp_service = nil,
	dhcp_pubs = {}
}

local function update_dhcp_service()
	if _M.dhcp_service then
		_M.dhcp_service:Kill()
		_M.dhcp_service:Destroy()
		_M.dhcp_service = nil
	end

	if #_M.dhcp_pubs > 0 then
		local args = {}
		local i, j
		for i=1, #_M.dhcp_pubs do
			for j=1, #_M.dhcp_pubs[i] do
				table.insert( args, _M.dhcp_pubs[i][j] )
			end
		end
		table.insert( args, "-l" )
		table.insert( args, "/tmp/dhcp.leases" )
		table.insert( args, "-p" ) -- -p 0 to disable DNS features.
		table.insert( args, "0" )
		table.insert( args, "-d" )

		local test_exist_f = io.open("/etc/dnsmasq.conf", "r")
		if test_exist_f then
			test_exist_f:close()
			--Don't use it.
			os.execute("/bin/mv -f /etc/dnsmasq.conf /etc/dnsmasq.conf.bak")
		end

		_M.dhcp_service = run_daemon.Create({
			name = "dhcp-server-dnsmasq",
			run = "/usr/sbin/dnsmasq",
			args = args,
			on_terminate = "restart"
		})
	
		if _M.dhcp_service then
			_M.dhcp_service:Start()
		end
	end
end

function _M.StartDHCPService(ifname, ifaddr, addr_prefix, options)
	--TODO: (this is the new feature):
	-- If you configured to enable DHCP service for specified interface 
	-- ('ifname'), it will call this function to start DHCP service.
	-- 'ifaddr' is the configured IP address for 'ifname', and 
	-- 'addr_prefix' is the address prefix (number).
	--
	-- 'options' is a table, contains:
	-- {
	--    dynamic_start = "DHCP start IP address",
	--    dynamic_end = "DHCP end IP address",
	--    gw = "gateway",
	--    dns = { array of dns list },
	--
	--    --NOTE: below is according to DHCP ruler to set DHCP option
	--    dhcp_options = {
	--    	[option-number] = "value",
	--    	...
	--    }
	-- }
	--
	-- you can run the DHCP service program by yourself according to the
	-- passed arguments, or do nothing to ignore that.
	local i, item
	for i=1, #_M.dhcp_pubs do
		if _M.dhcp_pubs[i].ifname == ifname then
			item = { ifname = ifname }
			_M.dhcp_pubs[i] = item
			break
		end
	end
	if not item then
		item = { ifname = ifname }
		table.insert( _M.dhcp_pubs, item )
	end
	item[1] = "-i"
	item[2] = ifname
	item[3] = "--listen-address"
	item[4] = ifaddr
	item[5] = "--dhcp-host"
	item[6] = ifaddr
	item[7] = "--dhcp-range"
	item[8] = options.dynamic_start .. "," .. options.dynamic_end .. ",8h"

	local last_idx = 9
	--local options_set = {}
	if options.gw then
		--options_set[3] = true
		--options_set["3"] = true
		item[last_idx] = "--dhcp-option"
		item[last_idx+1] = "3," .. options.gw
		last_idx = last_idx + 2
	else
		item[last_idx] = "--dhcp-option"
		item[last_idx+1] = "3"
		last_idx = last_idx + 2
	end

	if type(options.dns) == "string" or type(options.dns) == "table" then
		local dns_items
		if type(options.dns) == "table" then
			for _idx,_val in ipairs(options.dns) do
				if _idx > 1 then
					dns_items = dns_items .. "," .. _val
				else
					dns_items = _val
				end
			end
		else
			dns_items = options.dns
		end

		if dns_items then
			--options_set[6] = true
			--options_set["6"] = true
			item[last_idx] = "--dhcp-option"
			item[last_idx+1] = "6," .. dns_items
			last_idx = last_idx + 2
		else
			item[last_idx] = "--dhcp-option"
			item[last_idx+1] = "6"
			last_idx = last_idx + 2
		end
	else
		item[last_idx] = "--dhcp-option"
		item[last_idx+1] = "6"
		last_idx = last_idx + 2
	end
	
	if type(options.dhcp_options) == "table" then
		for _key,_val in pairs(options.dhcp_options) do
			item[last_idx] = "--dhcp-option"
			if _val == "" then
				item[last_idx + 1] = "" .. _key
			else
				item[last_idx + 1] = "" .. _key .. "," .. _val
			end
			last_idx = last_idx + 2
		end
	end
	
	update_dhcp_service()
end

function _M.StopDHCPService(ifname)
	--TODO: (this is the new feature)
	--      Stop the DHCP service for specified interface 'ifname'.
	
	for i=1, #_M.dhcp_pubs do
		if _M.dhcp_pubs[i].ifname == ifname then
			table.remove(_M.dhcp_pubs,i)
			break
		end
	end

	update_dhcp_service()
end

return _M

