local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local run_daemon = require "coaf.run_daemon"
local pf_config = require "portfwd_config"
local thread = require "coaf.thread"
local nm = require "mod.networkManager"
-- local pt = require "pl.pretty"

local _M = {
	running = {}, --save all running daemons.
	config = nil,
	init_thread = {}
}

local function GetPPPThenStart(thrObj, over, listen, protocol, args)
    local newover = nil
    while not thrObj.terminated do
        if over and over ~= "" then
			newover = nm.GetPPPbyName(over)
			if newover then
				-- _dbg.Info( "PORT-FWD", "******newover:",newover)
				table.insert(args, "--over")
				table.insert(args, newover)
				break
			end
        else
            -- do nothing...
            break
        end
        coaf.Sleep(5)
	end
	
	if not thrObj.terminated then
		-- pt.dump(args)
        local task = run_daemon.Create({
            name = "port-forward-" .. protocol .. "-" .. listen,
            run = "/usr/local/bin/port_fwd",
            args = args,
            on_terminate = function( task_obj, flag, reason, code )
                if reason ~= "kill" then --and reason ~= "exec-error" then
                    coaf.Sleep(1)
                    return "restart"
                else
                    --_M.running[ listen ] = nil
                    return nil
                end
            end
		})

        if task then
            _M.running[ listen ] = task
            task:Start()
            return true
        else
            return false, "create-failed"
        end
    end
end

function _M.__Create( protocol, listen, target, over, pkt_size, rx_buf, tx_buf )
	if not listen or listen == "" or not target or target == "" then
		_dbg.Error( "PORT-FWD", "Invalid listen address:", listen, "or invalid target address:", target )
		return nil, "invalid"
	end

	protocol = protocol or "tcp"
	if protocol == "udp" or protocol == "UDP" then
		protocol = "--udp"
	else
		protocol = "--tcp"
	end

	pkt_size = tostring(math.ceil(tonumber(pkt_size) or 64))
	rx_buf = tostring(math.ceil(tonumber(rx_buf) or 256))
	tx_buf = tostring(math.ceil(tonumber(tx_buf) or 256))

	local args = {
		protocol,
		"--listen",
		listen,
		"--to",
		target,
		"--psize",
		pkt_size,
		"--rbuf",
		rx_buf,
		"--wbuf",
		tx_buf
	}

	local newthread = thread.New(GetPPPThenStart)
    if newthread then
        newthread:Start(over, listen, protocol, args)
        _M.init_thread[listen] = newthread
    else
        _dbg.Error( "DEVICE", self.name, "Create init thread error!!!" )
    end
    return true
end

function _M.__Destroy( listen )
	if listen then
		local _thread = _M.init_thread[listen]
        if _thread then
            _thread:Terminate()
            _M.init_thread[listen] = nil
		end
		
		local task = _M.running[listen]
		if task then
			task:Destroy()
			_M.running[listen] = nil
		end
	end
end

function _M.Destroy()
	local k,v
	for k,v in pairs(_M.config) do
		_M.__Destroy( v.listen )
	end
end

function _M.Init()
	os.execute("/usr/bin/killall port_fwd")
	run_daemon.Init()
	if not _M.config then
		_M.config = pf_config.Load()
		
		local k,v
		for k,v in pairs(_M.config) do
			_M.__Create( v.protocol, v.listen, v.target, v.over, v.pkt_size, v.rx_buf, v.tx_buf )
		end
	end
end

function _M.Add( protocol, listen, target, over, pkt_size, rx_buf, tx_buf )
	if not listen or listen == "" or not target or target == "" then
		return false, "invalid-arguments"
	end
	protocol = string.lower(protocol or "tcp")
	if protocol == "" then
		protocol = "tcp"
	end
	if protocol ~= "tcp" and protocol ~= "udp" then
		return false, "invalid-protocol"
	end

	local addr,listen_port = listen:match("(.*):(%d+)$")
	listen_port = tonumber(listen_port)
	if not addr or not listen_port or addr == "" or listen_port <= 0 or listen_port > 65535 then
		return false, "invalid-listen-address"
	end
	local target_port
	addr, target_port = target:match("(.*):(%d+)$")
	target_port = tonumber(target_port)
	if not addr or not target_port or addr == "" or target_port <= 0 or target_port > 65535 then
		return false, "invalid-target-address"
	end

	if _M.config[listen_port] then
		return false, "exist"
	end

	local r, reason = _M.__Create( protocol, listen, target, over, pkt_size, rx_buf, tx_buf )
	if not r then
		return false, reason
	end

	_M.config[listen_port] = {
		protocol = protocol,
		listen = listen,
		target = target,
		over = over,
		pkt_size = tonumber(pkt_size),
		rx_buf = tonumber(rx_buf),
		tx_buf = tonumber(tx_buf)
	}
	_M.config:__TRIGGER(true)
	return true
end

function _M.Remove( listen )
	if not listen then
		return false, "invalid-arguments"
	end
	local addr,listen_port = listen:match("(.*):(%d+)$")
	listen_port = tonumber(listen_port)
	if not addr or not listen_port or listen_port <= 0 or listen_port > 65535 then
		return false, "invalid-listen-address"
	end

	if _M.init_thread[listen] then
        _M.init_thread[listen]:Terminate()
        _M.init_thread[listen] = nil
	end
	
	if _M.running[listen] then
		_M.running[listen]:Destroy()
		_M.running[listen] = nil
	end
	
	if _M.config[listen_port] then
		_M.config[listen_port] = nil
		_M.config:__TRIGGER(true)
		return true
	else
		return false, "not-exist"
	end
end

function _M.Clear()
	local k,v
	for k,v in pairs(_M.running) do
		v:Destroy()
	end
	_M.running = {}
	_M.config:TAB_BIND({})
	_M.config:__TRIGGER(true)
	os.execute("/usr/bin/killall port_fwd")
end

function _M.GetAll()
	local k,v
	local ret = {}
	for k,v in pairs(_M.config) do
		local status = "error"
		local task = _M.running[v.listen or "#bad"]
		if task then
			status = task:Status() or "error"
		end

		table.insert( ret, {
			Protocol = v.protocol,
			Listen = v.listen,
			__Port = tonumber(k) or 0,
			Target = v.target,
			Over = v.over,
			MaxPacketSize = v.pkt_size,
			SocketRxBuffer = v.rx_buf,
			SocketTxBuffer = v.tx_buf,
			Status = status
		})
	end
	return ret
end

return _M

