pcall( require, "oeminfo" )
pcall( require, "features" )
pcall( require, "version" )
pcall( require, "sys_setting")

local done,hwChk = pcall( require, "kiloview." .. _G.DEVICE_TYPE .. ".hardwareCheck" )

if done and type(hwChk) == "table" then
	hwChk.Check()
else
	hwChk = nil
end

_G.HARDWARE_VERSION = hwChk and hwChk.hardwareVersion or _G.HARDWARE_VERSION or "1.4"

local device_oem_info

if OEM_INFO and OEM_TYPE then
	device_oem_info = OEM_INFO[OEM_TYPE]
	if device_oem_info then
		device_oem_info =device_oem_info["en"]	--Use english languages.
	end
end

-------- Get the serial-no.

local function GetSerialNo()
	if type(_G.KV) == "table" and type(_G.KV.SERIAL_HISTORIES) == "table" and #_G.KV.SERIAL_HISTORIES > 0 then
		return _G.KV.SERIAL_HISTORIES[ #_G.KV.SERIAL_HISTORIES ].serial
	else
		return "0000000000"
	end
end

-------- Generate the hardware ID.

local hwid = GetSerialNo()
if hwid and hwid ~= "" and hwid ~= "0000000000" then
	if #hwid < 12 then
		hwid = string.rep('a',12-#hwid) .. hwid
	elseif #hwid > 12 then
		hwid = string.sub(hwid, (#hwid - 12) + 1)
	end
	
	hwid = "urn:uuid:95254bde-5e68-4fd3-a9e5-" .. hwid
else
	local file = io.popen( "/usr/bin/uuidgen", "r" )
	if file then
		local line = file:read("l")
		if line and line ~= "" then
			hwid = "urn:uuid:" .. line
		else
			hwid = "urn:uuid:95254bde-5e68-4fd3-a9e5-bad0bad0bad0"
		end
		file:close()
	else
		hwid = "urn:uuid:95254bde-5e68-4fd3-a9e5-bad0bad0bad0"
	end
end
--------

local _M = {
	DEVICE_OEM_INFO = device_oem_info or {
		DEVICE_NAME = "Professional Encoder",
		DEVICE_VENDOR = "",
		DEVICE_COPY = "Copyright 2016, All rights reserved",
		DEVICE_URL = "http://www.kiloview.com/",
		DEVICE_TYPE = "ENC-1"
	},
	
	SERIAL_NO = GetSerialNo(),
	HARDWARE_ID = hwid,
	
	HARDWARE_VERSION = _G.HARDWARE_VERSION or [[0.0]],
	SOFTWARE_VERSION = _G.SOFTWARE_VERSION or [[3.00.0001]],
	FIRMWARE_VERSION = _G.FIRMWARE_VERSION or [[0.0.00]],
	
	OEM_TYPE = _G.OEM_TYPE or "ENCODER",
	OEM_DEVICE = _G.OEM_DEVICE or "encoder"
}

------ Get Endpoint types
function _M.GetEndpointTypes(cast)
	if cast then
		return "tdn:NetworkVideoTransmitter tds:Device"
	else
		return "tdn:NetworkVideoTransmitter"
	end
end

----- Get scopes.
function _M.GetEndpointScopes(asArray)
	local fname, devType
	
	if _G.onvif.deviceName == "" then
		fname = _M.DEVICE_OEM_INFO.DEVICE_NAME
	else
		fname = _G.onvif.deviceName
	end

	if _G.FEATURE_NDI_HX then
		if _G.DEVICE_TYPE == "G1" or _G.DEVICE_TYPE == "P1" or _G.DEVICE_TYPE == "A1" or _G.DEVICE_TYPE == "R3C1" then
			devType = "G1_NewTek"
		else
			devType = "U2_NewTek"
		end
	else
		if _G.onvif.deviceType == "" then
			devType = OEM_TYPE or "ENC1"
		else
			devType = _G.onvif.deviceType
		end
	end
	
	fname = fname:gsub( "(%s)", "_" )
	fname = fname:gsub( "(/)", "|" )
	
	devType = devType:gsub( "(%s)", "_" )
	devType = devType:gsub( "(/)", "|" )
	if asArray then
		return {
			"onvif://www.onvif.org/type/Network_Video_Transmitter",
			"onvif://www.onvif.org/hardware/" .. devType,
			"onvif://www.onvif.org/location/" .. onvif.location,
			"onvif://www.onvif.org/name/" .. fname
		}
	else
		return "onvif://www.onvif.org/type/Network_Video_Transmitter onvif://www.onvif.org/hardware/" .. devType .. " onvif://www.onvif.org/location/" .. onvif.location .. " onvif://www.onvif.org/name/" .. fname
	end
end

function _M.GetHostAddress( specPort )
	specPort = specPort or _G.onvif.control.port
	
	local peerAddr = OnvifSvc.Net.GetCurrentSrcAddr()
	local allInterfaces = OnvifSvc.Net.GetAllInterfaces()
	if not allInterfaces then
		return "127.0.0.1:" .. specPort
	end
	
	local firstAddr
	local i,item
	for i,item in ipairs(allInterfaces) do
		if item.name ~= "lo" and item.family == "AF_INET" and item.address then
			if not firstAddr then
				firstAddr = item.address
			end
			
			if peerAddr and OnvifSvc.Net.AddressMatch( peerAddr, item.address, item.netmask ) then
				firstAddr = item.address
				break
			end
		end
	end
	
	if not firstAddr then
		return "127.0.0.1:" .. specPort
	else	
		return firstAddr .. ":" .. specPort
	end
end

----- Get service address.
function _M.GetServiceAddress()
	--TODO:
	return "http://" .. _M.GetHostAddress() .. "/onvif/device_service"
end

--~~~~~~~~~~~~~~~~~~~~~~~~

return _M



