local getset = require "kiloview.getSetHelper"
require "OnvifSvc"

local MAP_LOCATION = "./services"

local CONFIG_FILE = "/data/configs/onvif.conf"

require "configuredb"
require "onvif_config"

---------------------------------------------------
_G.onvif = CONFIGS.InstanceOf( ONVIF_Template, "onvif" )
_G.ONVIF_CONFIG_LOADING = true	--Lock to avoid trigger events
_G.onvif:LoadFile( CONFIG_FILE )
_G.ONVIF_CONFIG_LOADING = false
_G.onvif:ResetChanges()

---------------------------------------------------

local _M = {
	service_running = false
}

---------------------------------------------------

local Onvif_SVC = {
	Identifier = "Onvif",
	FriendlyName = "Onvif Service",
	Version = "2.0",
	ConfigPrefix = "onvif",
}

function Onvif_SVC:Activate()
	--Nothing
end

function Onvif_SVC:Start()
end

function Onvif_SVC:Stop()
end

function Onvif_SVC:Reset()
--TODO: Nothing
end

function Onvif_SVC:Terminate()

	if _M.service_running then
		OnvifSvc.Discovery.SendBye({
			wsa__EndpointReference = {
				Address = GLOBAL_INFO.HARDWARE_ID
			},
			Types = GLOBAL_INFO.GetEndpointTypes(true),
			Scopes = {
				{ __item = GLOBAL_INFO.GetEndpointScopes() },
			},

			XAddrs = GLOBAL_INFO.GetServiceAddress(),
			MetadataVersion = 1
		})

		OnvifSvc.Terminate()
	end
	
	os.exit(0)
end

function Onvif_SVC:Control(action,params)
	if action == "activate" then
		self:Activate()
		return 0, {}
	elseif action == "start" then
		self:Start()
		return 0, {}
	elseif action == "stop" then
		self:Stop()
		return 0, {}
	elseif action == "Reset" then
		self:Reset()
		return 0, {}
	elseif action == "terminate" then
		self:Terminate()
		return 0, {}
	else
		return -1, {}
	end
end

function Onvif_SVC:Set(params)
	local cnt = getset.SetConfigureValues( self.ConfigPrefix, params )
	return cnt
end

function Onvif_SVC:SetEnv(params)
	local cnt = getset.SetValues( "_ENV", params )
	return cnt
end

function Onvif_SVC:Get(inParams)
	local cnt = getset.GetValues( self.ConfigPrefix, inParams )
	return cnt,inParams
end

function Onvif_SVC:GetEnv(inParams)
	local cnt = getset.GetValues( "_ENV", inParams )
	return cnt,inParams
end

function Onvif_SVC:RestoreDefault()
	--TODO:
	return 0
end

function Onvif_SVC:LoadConfigureFile(filename)
	CONFIGS.BeginUpdate()
	local r = pcall( dofile, filename )
	CONFIGS.Submit()
	if r then
		return 1
	else
		return 0
	end
end

---------------------------------------------------
local function MapFile(routine)
	local filename = MAP_LOCATION .. "/" .. routine .. ".lua"
	local file = io.open( filename, "r" )
	if file then
		file:close()
		return filename
	else
		return MAP_LOCATION .. "/NOIMPL.lua"
	end
end

---------------------------------------------------
local auth_ignore_routines = {
	["__wsdd__Probe"] = true,
	["__wsdd__Resolve"] = true,
	["__wsdd__ProbeMatches"] = true,
	["__wsdd__ResolveMatches"] = true,
	["__wsdd__Hello"] = true,
	["__wsdd__Bye"] = true,
	["__tdn__Hello"] = true,
	["__tdn__Bye"] = true,
	["__tdn__Probe"] = true,
	["__tds__GetCapabilities"] = true
}

----------------

local function Authenticate(routine)
	--print( "Call:", routine )
	if onvif.auth_req == 0 or auth_ignore_routines[routine] then
		return nil	--Ignore.
	end
	
	local username, errcode = OnvifSvc.GetCurrentAuthUser()
	if not username then
		return errcode
	end
	
	if username == "admin" then
		return OnvifSvc.VerifyCurrentPassword( onvif.adminPassword )
	else
		local i
		for i=0,9 do
			if onvif.users[i].username ~= "" and onvif.users[i].username == username then
				return OnvifSvc.VerifyCurrentPassword( onvif.users[i].password )
			end
		end
		return 401
	end
end

---------------------------------------------------

function _M.BasicInit()
	OnvifSvc.SetLuaMapping( MapFile )
	OnvifSvc.SetAuthenticateCallback( Authenticate )
	OnvifSvc.SetHardwareID(GLOBAL_INFO.HARDWARE_ID)
end
---------------------------------------------------

function _M.Config()
	OnvifSvc.SetServicePort( onvif.control.port )
	if _M.service_running then
		OnvifSvc.StopDiscoverService()
	end
	OnvifSvc.SetDiscoverPort( onvif.control.discoverPort )

	if _M.service_running then
		--OnvifSvc.Terminate()
		
		_M.service_running = false

		OnvifSvc.Discovery:SendBye({
			wsa__EndpointReference = {
				Address = GLOBAL_INFO.HARDWARE_ID
			},
			Types = GLOBAL_INFO.GetEndpointTypes(true),
			Scopes = {
				{ __item = GLOBAL_INFO.GetEndpointScopes() },
			},
	
			XAddrs = GLOBAL_INFO.GetServiceAddress(),
			MetadataVersion = 1
		})

		OnvifSvc.StopService()
		--OnvifSvc.StopDiscoverService() ???
	end
	
	if onvif.control.enabled ~= 0 then
		_M.service_running = true

		OnvifSvc.StartDiscoverService()
		OnvifSvc.StartService()
	
		--What's wrong with this?
		pcall( function()
		OnvifSvc.Discovery:SendHello({
			wsa__EndpointReference = {
				Address = GLOBAL_INFO.HARDWARE_ID
			},
			Types = GLOBAL_INFO.GetEndpointTypes(true),
			Scopes = {
				{ __item = GLOBAL_INFO.GetEndpointScopes() },
			},
	
			XAddrs = GLOBAL_INFO.GetServiceAddress(),
			MetadataVersion = 1
		})
		end)
	end
end
---------------------------------------------------

_M.Reconfig = _M.Config

---------------------------------------------------

function _M.Init( cosvc )
	_M.BasicInit()
	_M.Config()
	if cosvc then
		cosvc:ExposeObject( "/", Onvif_SVC )
	end
	print( "Onvif CoAF service online." )
end

---------------------------------------------------

return _M
