--[==[

The TCP Co-Socket for CoAF (Co-Application Framework)

The cosocket module is similar to lua-socket, but, it works
asynchronously. If you are calling cosocket APIs in a 
coroutine context and called API may be blocked, it 
should yield current coroutine, util unblocked or
timeouted, it returns.

The cosocket module depends 'lua-ev' module.

Copyright (C) 2018, KILOVIEW Electronics, Co., Ltd.

]==]

local socket = require "socket"

local _M = {}

local tcp_meta = {}

function tcp_meta.getfd(tcp)
	return tcp.sock:getfd()
end

function tcp_meta.setfd(tcp, fd)
	return tcp.sock:setfd(fd)
end

function tcp_meta.dirty(tcp)
	return tcp.sock:dirty()
end

function tcp_meta.accept(tcp)
	return tcp.socket:accept()
end

function tcp_meta.bind(tcp, addr, port)
	if addr and addr:match("^unix:") then
		addr = addr:match("^unix:(.+)")
		if tcp.metatype ~= "unix" then
			local unix_sock = require "socket.unix"
			if not unix_sock then
				return nil, "fail to initialize unix socket"
			end
			local sock = unix_sock()
			if not sock then
				return nil, "fail to create unix socket"
			end
			if tcp.sock then
				tcp:close()
				collectgarbage()
			end
			tcp.sock = sock
			tcp.metatype = "unix"
		end
		tcp.unixaddr = addr
	end
	tcp.sock:setoption("reuseaddr", true)
	return tcp.sock:bind(addr,port)
end

function tcp_meta.cleanup(tcp, mode)
	--TODO:
end

function tcp_meta.close(tcp)
	tcp:cleanup("both")
	tcp.sock:close()
end

function tcp_meta.connect(tcp,addr,port)
	if addr and addr:match("^unix:") then
		addr = addr:match("^unix:(.+)")
		if tcp.metatype ~= "unix" then
			local unix_sock = require "socket.unix"
			if not unix_sock then
				return nil, "fail to initialize unix socket"
			end
			local sock = unix_sock()
			if not sock then
				return nil, "fail to create unix socket"
			end
			if tcp.sock then
				tcp:close()
				collectgarbage()
			end
			tcp.sock = sock
			tcp.metatype = "unix"
		end
		tcp.unixaddr = addr
	end

	return tcp.sock:connect(addr,port)
end

function tcp_meta.getpeername(tcp)
	if tcp.metatype == "unix" then
		return tcp.unixaddr, tcp.sock:getfd(), "unix"
	else
		return tcp.sock:getpeername()
	end
end

function tcp_meta.getsockname(tcp)
	if tcp.metatype == "unix" then
		return tcp.unixaddr, tcp.sock:getfd(), "unix"
	else
		return tcp.sock:getsockname()
	end
end

function tcp_meta.getstats(tcp)
	return tcp.sock:getstats()
end

function tcp_meta.listen(tcp,backlogs)
	return tcp.sock:listen(backlogs)
end

function tcp_meta.receive(tcp, pattern, prefix)
	return tcp.sock:receive(pattern, prefix)
end

function tcp_meta.send(tcp, data, i, j)
	return tcp.sock:send( data, i, j )
end

function tcp_meta.setoption(tcp,option,val)
	return tcp.sock:setoption(option,val)
end

function tcp_meta.setstats(tcp,received,sent,age)
	return tcp.sock:setstats(received,sent,age)
end

function tcp_meta.settimeout(tcp,value,mode)
	return tcp.sock:settimeout(value,mode)
end

function tcp_meta.shutdown(tcp,mode)
	--TODO: close tasks.
	tcp:cleanup(mode)
	return tcp.sock:shutdown(mode)
end

function _M.build_socket(tcp,metatype)
	local tcp_sock = {
		sock = tcp,
		metatype = metatype or "tcp",
		unixaddr = '',
		timeout = 30	-- The default action timeout!
	}
	return setmetatable(tcp_sock, {__index = tcp_meta})
end

return _M
