local busOprs = require "busOprs"
local devices = busOprs.GetIfDevices("*")	--Get all devices.
local k,v

local allDNS = {}
local manualDNS = {}

local file = io.open( "/etc/resolv.conf", "r" )
if file then
	local line = file:read("l")
	while line do
		if not line:match( "^%s*#.*" ) then	--Not be comment
			local addr = line:match( "nameserver%s+([0-9%.]+)" )
			if addr then
				table.insert( allDNS, addr )
			end
		end
		line = file:read( "l" )
	end
	file:close()
end

local dhcp = -1

for k,v in pairs(devices) do
	if (v.TypeName == "ETHERNET" or v.TypeName == "WIFI") and v.State > 10 then
		local ifconfig = busOprs.CheckIfConfig( v.Object, v.TypeName == "ETHERNET" and "WIRE" or "WIFI" )
		if ifconfig then
			if ifconfig.dhcp then
				if v.TypeName == "ETHERNET" then
					dhcp = true
				elseif dhcp == -1 then
					dhcp = true
				end
			else
				if v.TypeName == "ETHERNET" then
					dhcp = false
				elseif dhcp == -1 then
					dhcp = false
				end
			end
			
			if ifconfig.ip4dns1 and ifconfig.ip4dns1 ~= "" then
				manualDNS[ifconfig.ip4dns1] = true
			end
			
			if ifconfig.ip4dns2 and ifconfig.ip4dns2 ~= "" then
				manualDNS[ifconfig.ip4dns2] = true
			end
		end
	end
end

local dhcpDNS = {}
local mDNS = {}
for k,v in pairs(allDNS) do
	if manualDNS[v] then
		table.insert( mDNS, { Type = tt__IPType__IPv4 , IPv4Address = v } )
	else
		table.insert( dhcpDNS, { Type = tt__IPType__IPv4 , IPv4Address = v } )
	end
end

return {
	DNSInformation = {
		FromDHCP = dhcp and 1 or 0,
		SearchDomain = allDNS,
		DNSManual = mDNS,
		DNSFromDHCP = dhcpDNS
	}
}

--[==[ *ABSTRACT* The 'response' arguments:
{
	DNSInformation: (!)
	{
		FromDHCP: (!)
		(
			xsd__boolean__false_: 0,
			xsd__boolean__true_: 1,
		)
		__sizeSearchDomain: <int>
		SearchDomain: <char**>
		__sizeDNSFromDHCP: <int>
		DNSFromDHCP[...]: 
		{
			Type: (!)
			(
				tt__IPType__IPv4: 0,
				tt__IPType__IPv6: 1,
			)
			IPv4Address: <char*>
			IPv6Address: <char*>
		}
		__sizeDNSManual: <int>
		DNSManual[...]: 
		{
			Type: (!)
			(
				tt__IPType__IPv4: 0,
				tt__IPType__IPv6: 1,
			)
			IPv4Address: <char*>
			IPv6Address: <char*>
		}
		Extension: 
		{
			__size: <int>
			__any: <_XML>
		}
		__anyAttribute: <_XML>
	}
}
]==]

--NOTE: SVC_OK = 0, SVC_NOIMPL = 1

