local busOprs = require "busOprs"
local args = {...}
local request = args[1]
if not request or not request.InterfaceToken or not request.NetworkInterface then
	error( "Invalid request argument passed!" )
end

--local pt = require "pl.pretty"
--pt.dump(request)

if request.NetworkInterface.Enabled and request.NetworkInterface.IPv4 then
	local devices = busOprs.GetIfDevices(request.InterfaceToken)	--Get special
	if devices and #devices > 0 then
		local k,v
		for k,v in pairs(devices) do
			if v.TypeName == "ETHERNET" and v.State > 10 then
				local ifconfig = busOprs.CheckIfConfig( v.Object, "WIRE" )
				if ifconfig then
					local args
					if request.NetworkInterface.IPv4.DHCP ~= 0 then
						args = {
							obj = v.Object,
							uuid = ifconfig.uuid,
							dynamic = 'y',
							specConn = ifconfig.specConn
						}
					elseif type(request.NetworkInterface.IPv4.Manual) == "table" then
						args = {
							obj = v.Object,
							uuid = ifconfig.uuid,
							dynamic = 'n',
							ip = request.NetworkInterface.IPv4.Manual[1].Address or ifconfig.ip4addr,
							mask = request.NetworkInterface.IPv4.Manual[1].PrefixLength and OnvifSvc.Net.AddressMatch( "255.255.255.255", "255.255.255.255", request.NetworkInterface.IPv4.Manual[1].PrefixLength)  or ifconfig.ip4mask,
							gw = ifconfig.ip4gateway,
							dns1 = ifconfig.ip4dns1,
							dns2 = ifconfig.ip4dns2,
							specConn = ifconfig.specConn
						}
					end
					
					if args then
						--pt.dump(args)
						busOprs.SetWireIfConfig(args)
						busOprs.UpdateIfConfig(v.Object)
					end
				end --of Get 'ifconfig'
			end --of ethernet
		end --of for
	else
		print( "***WARNING: Invalid network interface of '" .. request.InterfaceToken .. "'" )
	end --of get 'devices'
end --of Enabled (but disable ethernet is not allowed.)

return {
	RebootNeeded = 0
}


--[[
{
  NetworkInterface = {
    __anyAttribute = "",
    IPv4 = {
      DHCP = 1,
      __sizeManual = 0,
      Enabled = 1
    },
    MTU = 1472,
    Enabled = 1
  },
  InterfaceToken = "eth0"
}
]]

--[==[ *ABSTRACT* The 'request' arguments:
{
	InterfaceToken: <char*> (!)
	NetworkInterface: (!)
	{
		Enabled: 
		(
			xsd__boolean__false_: 0,
			xsd__boolean__true_: 1,
		)
		Link: 
		{
			AutoNegotiation: (!)
			(
				xsd__boolean__false_: 0,
				xsd__boolean__true_: 1,
			)
			Speed: <int> (!)
			Duplex: (!)
			(
				tt__Duplex__Full: 0,
				tt__Duplex__Half: 1,
			)
		}
		MTU: <int*>
		IPv4: 
		{
			Enabled: 
			(
				xsd__boolean__false_: 0,
				xsd__boolean__true_: 1,
			)
			__sizeManual: <int>
			Manual[...]: 
			{
				Address: <char*> (!)
				PrefixLength: <int> (!)
			}
			DHCP: 
			(
				xsd__boolean__false_: 0,
				xsd__boolean__true_: 1,
			)
		}
		IPv6: 
		{
			Enabled: 
			(
				xsd__boolean__false_: 0,
				xsd__boolean__true_: 1,
			)
			AcceptRouterAdvert: 
			(
				xsd__boolean__false_: 0,
				xsd__boolean__true_: 1,
			)
			__sizeManual: <int>
			Manual[...]: 
			{
				Address: <char*> (!)
				PrefixLength: <int> (!)
			}
			DHCP: 
			(
				tt__IPv6DHCPConfiguration__Auto: 0,
				tt__IPv6DHCPConfiguration__Stateful: 1,
				tt__IPv6DHCPConfiguration__Stateless: 2,
				tt__IPv6DHCPConfiguration__Off: 3,
			)
		}
		Extension: 
		{
			__size: <int>
			__any: <_XML>
			__sizeDot3: <int>
			Dot3[...]: 
			{
				__size: <int>
				__any: <_XML>
				__anyAttribute: <_XML>
			}
			__sizeDot11: <int>
			Dot11[...]: 
			{
				SSID: (!)
				{
					__ptr: <unsigned char*>
					__size: <int>
				}
				Mode: (!)
				(
					tt__Dot11StationMode__Ad_hoc: 0,
					tt__Dot11StationMode__Infrastructure: 1,
					tt__Dot11StationMode__Extended: 2,
				)
				Alias: <char*> (!)
				Priority: <char*> (!)
				Security: (!)
				{
					Mode: (!)
					(
						tt__Dot11SecurityMode__None: 0,
						tt__Dot11SecurityMode__WEP: 1,
						tt__Dot11SecurityMode__PSK: 2,
						tt__Dot11SecurityMode__Dot1X: 3,
						tt__Dot11SecurityMode__Extended: 4,
					)
					Algorithm: 
					(
						tt__Dot11Cipher__CCMP: 0,
						tt__Dot11Cipher__TKIP: 1,
						tt__Dot11Cipher__Any: 2,
						tt__Dot11Cipher__Extended: 3,
					)
					PSK: 
					{
						Key: 
						{
							__ptr: <unsigned char*>
							__size: <int>
						}
						Passphrase: <char*>
						Extension: 
						{
							__size: <int>
							__any: <_XML>
						}
						__anyAttribute: <_XML>
					}
					Dot1X: <char*>
					Extension: 
					{
						__size: <int>
						__any: <_XML>
						__anyAttribute: <_XML>
					}
					__anyAttribute: <_XML>
				}
				__size: <int>
				__any: <_XML>
				__anyAttribute: <_XML>
			}
			Extension: 
			{
				__size: <int>
				__any: <_XML>
			}
		}
		__anyAttribute: <_XML>
	}
}
]==]

--[==[ *ABSTRACT* The 'response' arguments:
{
	RebootNeeded: (!)
	(
		xsd__boolean__false_: 0,
		xsd__boolean__true_: 1,
	)
}
]==]


