local args = {...}
local request = args[1]
if not request then
	error( "Invalid request argument passed!" )
end

local busOprs = require "busOprs"

local multicast = (request.StreamSetup.Stream == tt__StreamType__RTP_Multicast)

local isMainStream = false
if not request.ProfileToken or request.ProfileToken:match( "Profile_Main" ) then
	isMainStream = true
end

local svcPort
local sessId

if isMainStream then
	local cc, gets = busOprs.Get( {
		["streamer.stream[0].Rtsp_server.port"] = 554,
		["streamer.stream[0].Rtsp_server.session"] = "ch01"
	} )

	svcPort = gets["streamer.stream[0].Rtsp_server.port"] or 554
	sessId = gets["streamer.stream[0].Rtsp_server.session"] or "ch01" 
else
	local cc, gets = busOprs.Get( {
		["streamer.stream[1].Rtsp_server.port"] = 554,
		["streamer.stream[1].Rtsp_server.session"] = "sub01"
	} )

	svcPort = gets["streamer.stream[1].Rtsp_server.port"] or 554
	sessId = gets["streamer.stream[1].Rtsp_server.session"] or "sub01" 
end

if request.StreamSetup.Transport.Protocol == tt__TransportProtocol__UDP then
	local rtspUri = "rtsp://" .. GLOBAL_INFO.GetHostAddress( svcPort ) .. "/" .. sessId 
	
	if multicast then
		rtspUri = rtspUri .. "/ssm"
	end
	return {
		MediaUri = {
			Uri = rtspUri,
			InvalidAfterConnect = 0,
			InvalidAfterReboot = 0,
			Timeout = 0x7fffffff
		}
	}
elseif request.StreamSetup.Transport.Protocol == tt__TransportProtocol__TCP then
	local rtspUri = "rtsp://" .. GLOBAL_INFO.GetHostAddress( svcPort ) .. "/" .. sessId 
	
	if multicast then
		rtspUri = rtspUri .. "/ssm"
	end
	return {
		MediaUri = {
			Uri = rtspUri,
			InvalidAfterConnect = 0,
			InvalidAfterReboot = 0,
			Timeout = 0x7fffffff
		}
	}
elseif request.StreamSetup.Transport.Protocol == tt__TransportProtocol__RTSP then
	local rtspUri = "rtsp://" .. GLOBAL_INFO.GetHostAddress( svcPort ) .. "/" .. sessId 
	
	if multicast then
		rtspUri = rtspUri .. "/ssm"
	end
	
	return {
		MediaUri = {
			Uri = rtspUri,
			InvalidAfterConnect = 0,
			InvalidAfterReboot = 0,
			Timeout = 0x7fffffff
		}
	}
elseif request.StreamSetup.Transport.Protocol == tt__TransportProtocol__HTTP then
	return {
		MediaUri = {
			Uri = "", --TODO
			InvalidAfterConnect = 0,
			InvalidAfterReboot = 0,
			Timeout = 0x7fffffff
		}
	}
else
	return {
		MediaUri = {
			Uri = "", --TODO
			InvalidAfterConnect = 1,
			InvalidAfterReboot = 1,
			Timeout = 0
		}
	}
end

--TODO: Check the 'request' arguments, and fill the 'response' arguments, see <same-name>.txt file for reference.

--[==[ *ABSTRACT* The 'request' arguments:
{
	StreamSetup: (!)
	{
		Stream: (!)
		(
			tt__StreamType__RTP_Unicast: 0,
			tt__StreamType__RTP_Multicast: 1,
		)
		Transport: (!)
		{
			Protocol: (!)
			(
				tt__TransportProtocol__UDP: 0,
				tt__TransportProtocol__TCP: 1,
				tt__TransportProtocol__RTSP: 2,
				tt__TransportProtocol__HTTP: 3,
			)
			Tunnel: 
			{...}
		}
		__size: <int>
		__any: <_XML>
		__anyAttribute: <_XML>
	}
	ProfileToken: <char*> (!)
}
]==]

--[==[ *ABSTRACT* The 'response' arguments:
{
	MediaUri: (!)
	{
		Uri: <char*> (!)
		InvalidAfterConnect: (!)
		(
			xsd__boolean__false_: 0,
			xsd__boolean__true_: 1,
		)
		InvalidAfterReboot: (!)
		(
			xsd__boolean__false_: 0,
			xsd__boolean__true_: 1,
		)
		Timeout: <LONG64> (!)
		__size: <int>
		__any: <_XML>
		__anyAttribute: <_XML>
	}
}
]==]


