local _M = {}

---------------------------------------------------
-- RTP pushing service operations
---------------------------------------------------

local rtp_svc_methods = {}
rtp_svc_methods.__index = rtp_svc_methods

function rtp_svc_methods:get_ServiceType()
	return "RTP/PUSH"
end

function rtp_svc_methods:ServiceName()
	return "RTP pushing of stream " .. self.appId .. "-" .. self.streamId
end

function rtp_svc_methods:get_Enabled()
	local thisConfig = _G.streamer.stream[self.cfgId].Dynamic_services[self.dynamicIndex].Rtp_push
	if not thisConfig then
		return false
	end
	return _G.streamer.stream[self.cfgId].enabled ~= 0 and thisConfig.enabled ~= 0
end

function rtp_svc_methods:get_ServiceUri()
	local thisConfig = _G.streamer.stream[self.cfgId].Dynamic_services[self.dynamicIndex].Rtp_push
	if not thisConfig then
		return "offline"
	elseif self.streamer then
		if 1 then
			return "rtp://" .. thisConfig.target_video_IP .. ":" .. thisConfig.target_video_port
		end
		return "none"
	else
		return "offline"
	end
end

function rtp_svc_methods:Enable(en)
	local thisConfig = _G.streamer.stream[self.cfgId].Dynamic_services[self.dynamicIndex].Rtp_push
	if not thisConfig then
		return false
	end
	CONFIGS.BeginUpdate()
	thisConfig.enabled = en and 1 or 0
	CONFIGS.Submit()
	return thisConfig.enabled ~= 0 and _G.streamer.stream[self.cfgId].enabled ~= 0
end

function rtp_svc_methods:Update()
	local streams = require "streams"
	streams.UpdateStreamService( self.streamId, "Dynamic_" .. string.format( "%02d", self.dynamicIndex ), self.dynamicIndex )
end

function rtp_svc_methods:GetStatus()
	if not self.streamer then
		return 1, { ["stream.status"] = "offline" }
	else
		return 1, { ["stream.status"] = "online" }
	end
end

------------------------------

function _M.ServiceMethods()
	return rtp_svc_methods
end

return _M
