require "features"
local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
require "kmCore"

----------------------------------------------------

local _M = {
	servers = {}
}

----------------------------------------------------
local function onLookupRtspAuthUser( rtsp_server, digist, realm, username )
	if not username or username == "" then
		return nil
	end

	local i
	for i=0,15 do
		if _G.media_users.users[i].username == username then
			return _G.media_users.users[i].password
		end
	end
	return nil
end
----------------------------------------------------

local rtsp_meta = {}

function rtsp_meta:AddSession( idr, mediaSource, session_id, options, open_ndi )
	if not mediaSource or not session_id then
		_dbg.Error( "RTSP", "Invalid RTSP session arguments (MediaSource/Session ID ...). Stream:", self.stream_id, ", IDR:", idr )
		return nil
	end

	if self.session then
		_dbg.Warn( "RTSP", "The RTSP session is already created. Stream:", self.stream_id, ", IDR:", idr )
		return self.session
	end

	if self.server.server:AddSession( session_id, mediaSource, options ) 
	then
		_dbg.Info( "RTSP", "Session of stream '" .. idr .. "' created/updated. [session=" .. session_id .. "]"  )
		self.session = session_id
		if (_G.FEATURE_NEWTEK or _G.FEATURE_NDI_HX) and open_ndi then
            self.server.server:AddSession( session_id .. "-raw", mediaSource, nil )
        end
	else
		_dbg.Error( "RTSP", "Session of stream '" .. idr .. "' add fail! session id:", session_id )
	end
	return self.session
end

function rtsp_meta:RemoveSession()
	if self.session then
		if _G.FEATURE_NEWTEK or _G.FEATURE_NDI_HX then
			self.server.server:RemoveSession( self.session .. "-raw" )
		end
		self.server.server:RemoveSession( self.session )
		self.session = nil
		collectgarbage()
	end

	return true
end

function rtsp_meta:StartSession()
	self.server.server:Start()
end

function rtsp_meta:StopSession()
	--TODO: Nothing.
end

function rtsp_meta:SetDropADTSHeader(en)
	if self.session then
		self.server.server:SetDropADTSHeader( self.session, en )
		return true
	else
		_dbg.Warn( "RTSP", "Set rtsp session option 'DropADTSHeader' but session is not created! Stream:", self.stream_id )
		return false
	end
end

function rtsp_meta:EnableScrambling(en,options)
	if self.session then
		self.server.server:EnableScrambling( self.session, en, options )
		return true
	else
		_dbg.Warn( "RTSP", "Set rtsp session option 'Scrambling' but session is not created! Stream:", self.stream_id )
		return false
	end
end

function rtsp_meta:EnableAuth(en)
	return self.server.server:EnableAuth(en)
end

function rtsp_meta:Destroy()
	self:RemoveSession()
	if self.server.counter > 0 then
		self.server.counter = self.server.counter - 1
		--TODO: Destroy the RTSP server?
		--[[
		if self.server.counter <= 0 then
			self.server.server:Stop()
			self.server.server = nil

			local i
			for i=1, #_M.servers do
				if _M.servers[i] == self.server then
					table.remove( _M.servers, i )
					break
				end
			end

			collectgarbage()
		end
		]]
	end
end

rtsp_meta.__index = rtsp_meta

----------------------------------------------------
function _M.EnableAuth(en)
	local servers = _M.servers
	local i
	for i=1, #servers do
		servers[i].server:EnableAuth(en)
	end
end

----------------------------------------------------
function _M.GetService( stream_id, options )
	stream_id = tonumber(stream_id)
	stream_id = string.format( "%02d", stream_id )
	if not options then
		options = {
			port = 554
		}
	end

	local fetched_server
	local servers = _M.servers
	local i
	for i=1, #servers do
		if servers[i].port == options.port then
			servers[i].counter = servers[i].counter + 1
			fetched_server = servers[i]
			break
		end
	end

	if not fetched_server then
		local rtsp_server = KM_STREAMING.Rtsp.Server.createNew( "KILOVIEW Rtsp Media Server (" .. options.port ..")", "*", options.port, options.httpTunnelPort or 0 )
		if not rtsp_server then
			_dbg.Error( "RTSP", "Could not create RTSP server at port:", options.port, "! - This effects stream:", stream_id )
			return nil
		else
			_dbg.Info( "RTSP", "Created a RTSP server instance, Port=", options.port )
			rtsp_server:On( "authorization", onLookupRtspAuthUser )
		end

		fetched_server = {
			port = options.port,
			httpTunnelPort = options.httpTunnelPort,
			counter = 1,
			server = rtsp_server
		}
		table.insert( servers, fetched_server )
	end

	local service = {
		stream_id = stream_id,
		server = fetched_server
	}

	return setmetatable( service, rtsp_meta )
end

----------------------------------------------------
return _M

