require "kiloview.configure_helper"
local coaf = require "coaf.core"

math.randomseed(os.time())

_G.STREAMS_CONFIG_LOADING = false
_G.LOCAL_SIP_ADDRESS = "127.0.0.1"

--------------
local function getStreamEnable( owner, pObj )
	local rootObj = owner:Root()
	if rootObj then
		local thisStream = pObj:Parent():Name()
		if thisStream == "2" then
			pObj:InternalSetInt(1)
		elseif rootObj.stream_mode == "main" then
			if thisStream == "0" then
				pObj:InternalSetInt(1)
			else
				pObj:InternalSetInt(0)
			end
		elseif rootObj.stream_mode == "main+sub" then
			pObj:InternalSetInt(1)
		else
			pObj:InternalSetInt(0)
		end
	else
		pObj:InternalSetInt(0)
	end
end

----------

local function getRtspSession( owner, pObj )
	if pObj:InternalGetString() == "" then	--Not set, so assume defaults
		local thisStream = pObj:Parent():Parent():Name()
		if thisStream == "0" then
			pObj:InternalSetString("ch01")
		elseif thisStream == "1" then
			pObj:InternalSetString("sub01")
		else
			pObj:InternalSetString("mjpeg01")
		end
	end
end

----------

local function getKmpPullingChannel( owner, pObj )
	local rootObj = owner:Root()
	local thisStream = pObj:Parent():Parent():Name()
	if rootObj.kmp_ordering == "snap_first" then	--Make the snap stream as 0(default) KMP channel.
		thisStream = tonumber(thisStream)
		if thisStream == 2 then
			pObj:InternalSetInt(0)
		else
			pObj:InternalSetInt( thisStream + 1 )
		end
	else
		pObj:InternalSetInt( tonumber(thisStream) )
	end
end

----------

local function getConnectionTimeout( owner, pObj )
	local v = pObj:InternalGetInt() 
	if v <= 0 or v > 60 then
		pObj:InternalSetInt(15)
	end
end

----------

local function getReconnectRetry( owner, pObj )
	local v = pObj:InternalGetInt() 
	if v < 0 then
		pObj:InternalSetInt(3)
	end
end

---------

local function onStreamModeChanged( owner, pObj )
	if _G.STREAMS_CONFIG_LOADING then
		return
	end
	coaf.ScheduleDelayedTask( function()
		local streams = require "streams"
		streams.UpdateConfigures()
	end)
end

---------

local function delayed_EnableAuth(en)
	coaf.ScheduleDelayedTask( function()
		local stream_rtsp_server = require "stream.rtsp_server"
		stream_rtsp_server.EnableAuth( en )
	end)
end

local function onRtspAuthChanged( owner, pObj )
	if _G.STREAMS_CONFIG_LOADING then
		return
	end
	delayed_EnableAuth( pObj:InternalGetInt() ~= 0 )
end

local function onKmpOrderingChanged( owner, pObj )
	if _G.STREAMS_CONFIG_LOADING then
		return
	end

	coaf.ScheduleDelayedTask( function()
		local streams = require "streams"
		--Update the KMP pulling servers.
		
		--Stop them first.
		streams.UpdateKmpPullingServer( false, "00" )
		streams.UpdateKmpPullingServer( false, "01" )
		streams.UpdateKmpPullingServer( false, "02" )
	
		--Then try start.
		streams.UpdateStreamService( "00", "Kmp_pullingServer" )
		streams.UpdateStreamService( "01", "Kmp_pullingServer" )
		streams.UpdateStreamService( "02", "Kmp_pullingServer" )
	end)
end

---------
local function delayed_UpdateStreamService( stream_id, serviceName, serviceId )
	coaf.ScheduleDelayedTask( function()
		local streams = require "streams"
		streams.UpdateStreamService( stream_id, serviceName, serviceId )
	end)
end

local function onServiceParameterChanged( obj )
	local streamId = obj:Parent():Name()
	local selfName = obj:Name()
	delayed_UpdateStreamService( "0" .. streamId, selfName )
end

local function onNDIServerChanged()
	coaf.ScheduleDelayedTask( function()
		local streams = require "streams"
		streams.Update_NDI()
	end)
end

---------

local function onDynamicServiceParameterChanged( obj )
	local streamId = obj:Parent():Parent():Parent():Name()
	local serviceId = obj:Parent():Name()
	--print( "STREAM_ID:", streamId, "SERVICE_ID:", serviceId )
	delayed_UpdateStreamService( "0" .. streamId,  "Dynamic_" .. string.format( "%02d", tonumber(serviceId)), tonumber(serviceId) )
end

---------

local function onDynamicServiceTypeChanged( owner, pObj )
	if _G.STREAMS_CONFIG_LOADING then
		return
	end
	local streamId = owner:Parent():Parent():Name()
	local serviceId = owner:Name()
	--print( "STREAM_ID:", streamId, "SERVICE_ID:", serviceId )
	delayed_UpdateStreamService( "0" .. streamId, "Dynamic_" .. string.format( "%02d", tonumber(serviceId)), tonumber(serviceId) )
end

---------
local function delayed_UpdateAudioChannel( stream_id )
	coaf.ScheduleDelayedTask( function()
		local streams = require "streams"
		streams.UpdateAudioChannel( stream_id )
	end)
end

local function onAudioChannelChanged( owner, pObj )
	if _G.STREAMS_CONFIG_LOADING then
		return
	end
	
	local streamId = owner:Name()
	--print( "STREAM_ID:", streamId )
	delayed_UpdateAudioChannel( "0" .. streamId )
end

---------

--[[
local function getDynamicServiceEnable( owner, pObj )
	local serviceType = owner:Parent().serviceType
	local myServiceType = owner:Name()
	if serviceType == myServiceType then
		pObj:InternalSetInt(1)
	else
		pObj:InternalSetInt(0)
	end
end
]]
---------

local function getVideoRtspMcPort( owner, pObj )
	if pObj:InternalGetInt() == 0 then
		local streamId = tonumber( owner:Parent():Name() )
		pObj:InternalSetInt( 3000 + 100*streamId )
	end
end

local function getAudioRtspMcPort( owner, pObj )
	if pObj:InternalGetInt() == 0 then
		local streamId = tonumber( owner:Parent():Name() )
		pObj:InternalSetInt( 3000 + 100*streamId + 2 )
	end
end

local function getHLSsession(owner, pObj )
	if pObj:InternalGetString() == "" then
		local thisStream = owner:Parent():Parent():Parent():Name()
		local stream_idx = owner:Parent():Name()
		if thisStream == "0" then
			pObj:InternalSetString("main" .. stream_idx )
		elseif thisStream == "1" then
			pObj:InternalSetString("sub" .. stream_idx)
		end		
	end
end

local function delayed_UpdateScrambling( stream_id )
	coaf.ScheduleDelayedTask( function()
		local streams = require "streams"
		streams.UpdateScrambling( stream_id )
	end)
end

local function onScramblingChanged( obj )
	local streamId = obj:Parent():Name()
	delayed_UpdateScrambling( "0" .. streamId )
end

local function delayed_UpdateSipEndpoint( stream_id )
	coaf.ScheduleDelayedTask( function()
		local stream_sip = require "stream.sip"
		stream_sip.UpdateSipEndpoint()
	end)
end

local function onSipEpConfigChanged( obj )
	delayed_UpdateSipEndpoint()
end

local function onEnableStreaming( owner, pObj )
	if _G.STREAMS_CONFIG_LOADING then
		return
	end

	coaf.ScheduleDelayedTask( function()
		local streams = require "streams"
		streams.UpdateEnableStreaming()
	end)
end

---------

Streamer_Template = OBJECT({
	stream_mode = { type="string", value="", noPersist=true, onChanged = onStreamModeChanged },
	kmp_ordering = { type="string", value="default", onChanged = onKmpOrderingChanged },
	rtsp_auth = { type="integer", value=0, onChanged = onRtspAuthChanged },
	http_tunnel_port = { type="integer", value=0 },
	rtsp_service_port = { type="integer", value=554 },
	no_video = { type="integer", value=0 },
	enable_streaming = { type="integer", value=1, noPersist=true, onChanged = onEnableStreaming },

	sipEp = OBJECT({
		enabled = { type="integer", value=0 },
		localAccount = { type="string", value="" },
		localPort = { type="integer", value=5060 },
		
		--注意：  北京米提sip注册 <sip:xx@miti.com> 而非sip:xx@192.168.2.168:5060
		localUri = { type="string", value="", noPersist=true, onRead = function(owner,pObj)
			pObj:InternalSetString(owner.localAccount)
			local s = "<" .. owner.localAccount .. ">"
			pObj:InternalSetString( s )
		end },
--[[
		localUri = { type="string", value="", noPersist=true, onRead = function(owner,pObj)
			local s = "sip:" .. owner.localAccount .. "@" .. _G.LOCAL_SIP_ADDRESS
			if owner.localPort ~= 5060 then
				s = s .. ":" .. owner.localPort
			end
			pObj:InternalSetString( s )
		end },
--]]
		server = { type="string", value="" },
		serverPort = { type="integer", value=5060 },
		realm = { type="string", value="*" },

		username = { type="string", value="", onRead = function(owner,pObj)
			if pObj:InternalGetString() == "" then
				pObj:InternalSetString( owner.localAccount )
			end
		end },

		password = { type="string", value="" },
	}, onSipEpConfigChanged),

	stream = ARRAY(
		{ 0,1,2 },

		OBJECT({
			enabled = { type="integer",  noPersist=true, writeEnable=false, onRead = getStreamEnable },
			audioChannel = { type="integer", value=1, onChanged = onAudioChannelChanged },

			Scrambling = OBJECT({
				enabled = { type="integer", value=0 },
				seed = { type="string", value="", noPersist=true, onRead=function(owner,pObj)
					if pObj:InternalGetString() == "" then
						local v1 = math.random(0,255)
						local v2 = math.random(0,255)
						local v3 = math.random(0,255)
						local v4 = math.random(0,255)
						pObj:InternalSetString( string.format("0x%02x%02x%02x%02x", v1,v2,v3,v4) )
					end
				end
				},

				depth = { type="string", value="", noPersist=true, onRead=function(owner,pObj)
					if pObj:InternalGetString() == "" then
						local v1 = math.random(0,255)
						local v2 = math.random(0,255)
						local v3 = math.random(0,255)
						local v4 = math.random(0,255)
						pObj:InternalSetString( string.format("0x%02x%02x%02x%02x", v1,v2,v3,v4) )
					end
				end
				}
			}, onScramblingChanged),

			Rtsp_server = OBJECT({

				port = { type="integer", value=554, noPersist=true, onRead = function(owner,pObj)
						pObj:InternalSetInt( owner:Root().rtsp_service_port )
					end,

					onWrite = function(owner,pObj,val)
						if _G.STREAMS_CONFIG_LOADING then
							return
						end
						if owner:Root().rtsp_service_port ~= val then
							owner:Root().rtsp_service_port = val
							pObj:InternalSetChanged(true)
						else
							pObj:InternalSetChanged(false)
						end
					end
				},
				
				httpTunnelPort = { type="integer", value=0, noPersist=true, onRead = function(owner,pObj)
						pObj:InternalSetInt( owner:Root().http_tunnel_port )
					end,

					onWrite = function(owner,pObj, val)
						if _G.STREAMS_CONFIG_LOADING then
							return
						end
						if owner:Root().http_tunnel_port ~= val then
							owner:Root().http_tunnel_port = val
							pObj:InternalSetChanged(true)
						else
							pObj:InternalSetChanged(false)
						end
					end
				},

				session = { type="string", value="", onRead = getRtspSession },
				auth = { type="integer", value=0 },
				ssm = { type="integer", value=0 },
				multicast_addr = { type="string", value="224.0.0.1" },
				multicast_video_port = { type="integer", value=0, onRead = getVideoRtspMcPort },
				multicast_audio_port = { type="integer", value=0, onRead = getAudioRtspMcPort },
				multicast_ttl = { type="integer", value=127 },
				no_adts = { type="integer", value=0 }
			}, onServiceParameterChanged),

			NDI_server = OBJECT({
				enabled = { type = "integer", value = 1},
				device_name = { type = "string", value = "Chan 1"},
				host_name = { type = "string", value = ""},
				group_name = { type = "string", value = ""},
				connection = { type = "string", value = "tcp"},
				mcast_prefix = { type = "string", value = "239.255.0.0"},
				mcast_mask = { type = "string", value = "255.255.0.0"},
				server_addr = { type = "string", value = ""}
			}, onNDIServerChanged),

			Kmp_pullingServer = OBJECT({
				enabled = { type="integer", value = 0 },
				timeShift = { type="integer", value=3000 },
				channel = { type="integer", noPersist=true, writeEnable=false, onRead = getKmpPullingChannel },
			}, onServiceParameterChanged),

			--TODO: The RTMP server
			
			Dynamic_services = ARRAY(
				{0,1,2,3,4,5,6,7,8},		--Max 8 dynamic services
				
				OBJECT({
					serviceType = { type="string", value="", onChanged = onDynamicServiceTypeChanged },
					--RTMP pushing configures.
					Rtmp_push = OBJECT({
						enabled = { type="integer", value=0 },
						control_enabled = { type="integer", value=0 },
						pubPoint = { type="string", value= "" },
						username = { type="string", value= "" },
						password = { type="string", value= "" },
						timeout = { type="integer", value= 15, onRead = getConnectionTimeout },
						reconnect_interval = { type="integer", value=3, onRead = getReconnectRetry },
						compatible_mode = { type="integer", value=1 },
						ntp_timestamp = { type="integer", value=-1 }
					}, onDynamicServiceParameterChanged),
			
					--General network pushing configures.
					Net_push = OBJECT({
						enabled = { type="integer", value=0 },
						control_enabled = { type="integer", value=0 },
						pubPoint = { type="string", value= "" }
					}, onDynamicServiceParameterChanged),

					--RTP pushing configures.
					Rtp_push = OBJECT({
						enabled = { type="integer", value=0 },
						control_enabled = { type="integer", value=0 },
						target_video_IP = { type="string", value="" },
						target_video_port = { type="integer", value= 1026 },
						--target_audio_IP = { type="string", value="" },
						--target_audio_port = { type="integer", value= 1026 },
						payload_type = { type="string", value="TS" },
						ts_pts_pcr_delay = { type="integer", value=500 },
						ts_service_name = { type="string", value="Encoder" },
						ts_service_provider = { type="string", value="Encoder device" }
					}, onDynamicServiceParameterChanged),
			
					--KMP pushing configures.
					Kmp_push = OBJECT({
						enabled = { type="integer", value=0 },
						control_enabled = { type="integer", value=0 },
						pubPoint = { type="string", value= "" },
						username = { type="string", value= "" },
						password = { type="string", value= "" },
						timeout = { type="integer", value= 15, onRead = getConnectionTimeout },
						reconnect_interval = { type="integer", value = 3, onRead = getReconnectRetry }
					}, onDynamicServiceParameterChanged),
			
					--SProxy pushing configures.
					Sproxy_push = OBJECT({
						enabled = { type="integer", value=0 },
						control_enabled = { type="integer", value=0 },
						pubPoint = { type="string", value= "" },
						username = { type="string", value= "" },
						password = { type="string", value= "" },
						timeout = { type="integer", value= 15, onRead = getConnectionTimeout },
						reconnect_interval = { type="integer", value = 3, onRead = getReconnectRetry }
					}, onDynamicServiceParameterChanged),
			
					--TS pushing configures.
					TS_push = OBJECT({
						enabled = { type="integer", value=0 },
						control_enabled = { type="integer", value=0 },
						targets = ARRAY(
							{ 1,2,3,4,5,6,7,8 },
							OBJECT({
								address = { type="string", value="" },
								port = { type="integer", value=0 },
								ttl = { type="integer", value=127 },
								ts_transport_stream_id = { type="integer", value=101 },
								ts_pts_pcr_delay = { type="integer", value=200 },
								ts_pmt_start_pid = { type="integer", value=480 },
								ts_start_pid = { type="integer", value=481 },
								ts_tables_version = { type="integer", value=6 },
								ts_service_name = { type="string", value="Encoder" },
								ts_service_provider = { type="string", value="Encoder device" },
								ts_null_multiple = { type="double", value=1.5 },
								ts_pcr_period = { type="integer", value=20 }
							})
						)
					}, onDynamicServiceParameterChanged),
					
					--SRT pushing configures.--add by youji@2019-7-15
					SRT_push = OBJECT({
						enabled = { type="integer", value=0 },
						control_enabled = { type="integer", value=0 },
						address = { type="string", value="" },
						srt_connect_mode = { type="string", value="caller" },
						port = { type="integer", value=1025 },
						srt_latency = { type="integer", value=125 },
						advanced_set = { type="integer", value=0 },
						srt_stream_id = { type="string", value="" },
						srt_encryption_key_len = { type="string", value="none" },
						srt_encryption_key = { type="string", value="" },
						srt_max_bw = { type="integer", value=-1 },
						srt_input_bw = { type="integer", value=0 },
						srt_bandwidth_overhead = { type="double", value=25.00 },
						srt_payload_size = { type="integer", value=1316 },
						ts_null_multiple = { type="double", value=0 },
						ts_pcr_period = { type="integer", value=20 },
						ts_pts_pcr_delay = { type="integer", value=500 },
						ts_transport_stream_id = { type="integer", value=101 },
					}, onDynamicServiceParameterChanged),
			
					--HLS service configures.
					HLS = OBJECT({
						enabled = { type="integer", value=0 },
						control_enabled = { type="integer", value=0 },
						session = { type="string", value = "", onRead = getHLSsession },
						segmentTime = { type="integer", value=5000 },
						maxSegments = { type="integer", value=4 },
					}, onDynamicServiceParameterChanged),

					--JUNSHI service configures.
					JUNSHI = OBJECT({
						enabled = { type="integer", value=0 },
						control_enabled = { type="integer", value=0 },
						host = { type="string", value="device.vgemv.com" },
						port = { type="integer", value=9200 },
						vender_key = { type="string", value="A0000002" },
						secure_key = { type="string", value="123456" },
					}, onDynamicServiceParameterChanged),

					Omnistream = OBJECT({
						enabled = { type="integer", value = 0 },
						control_enabled = { type="integer", value=0 },
						address = { type="string", value = "" },
						video_port = { type="integer", value = 0 },
						audio_port = { type="integer", value = 0 },
						error_correction = { type="string", value="none" }
					}, onDynamicServiceParameterChanged),

					SIP = OBJECT({
						enabled = { type="integer", value="0" },
						control_enabled = { type="integer", value=0 },
						callId = { type="string", value="" },
						callAccount = { type="string", value="", noPersist=true, onRead = function(owner,pObj)
							if owner.callId == "" then
								pObj:InternalSetString( "" )
							else
								local s = owner.callId
								if s:match( "sip:" ) then
									local pj_call = "<" .. s .. ">"
									--pObj:InternalSetString( s )
									pObj:InternalSetString( pj_call )
								elseif s:match( "@" ) then
									pObj:InternalSetString( "sip:" .. s )
								elseif owner:Root().sipEp.realm ~= "" and owner:Root().sipEp.realm ~= "*" then
									local uri = "sip:" .. s .. "@" .. owner:Root().sipEp.realm
									pObj:InternalSetString( uri )
								else
									local uri = "sip:" .. s .. "@" .. owner:Root().sipEp.server .. ":" .. owner:Root().sipEp.serverPort
									pObj:InternalSetString( uri )
								end
							end
						end },
						autoAnswer = { type="integer", value=1 }
					}, onDynamicServiceParameterChanged),
			
				}) -- Of dynamic service config object
			) -- Of dynamic service configs array
		}) -- Of stream config object
	) -- Of streams array
})



