local coaf = require "coaf.core"
local sysinfo = require "kiloview.sysInfo"
local lfs = require "lfs"
local periphery = require "periphery"
local SERIAL = periphery.Serial
local _dbg = require "coaf.debug"
local thread = require "coaf.thread"
--local logProc = require "kiloview.logProc"
local timedatectl = require "kiloview.systemd-timedate"
require "features"

local REBOOT_REQUIRED = false

local _M = {
	cpu = { load = 0, idle = 99 },
}

function _M.Reset()
	local x, reset = pcall( require, "kiloview." .. _G.DEVICE_TYPE .. ".reset" )
	if not x or not reset then   
		x, reset = pcall( require, "kiloview.reset" )
	end
	if reset then
		reset.Reset()
	end
	return true
end

function _M.Reboot()
	REBOOT_REQUIRED = true
	return true
end

function _M.RestoreFactory()
	local x, restore = pcall( require, "kiloview." .. _G.DEVICE_TYPE .. ".restoreFactory" )
	if not x or not restore then   
		x, restore = pcall( require, "kiloview.restoreFactory" )
	end
	if restore then
		x = restore.RestoreFactory()
		if x == "reboot" then
			REBOOT_REQUIRED = true
		end
		return true
	else
		return false
	end
end

function _M.GetTimeDateInfo()
	return timedatectl.GetTimeDateInfo()
end

function _M.GetNTPAddress()
	return timedatectl.GetNTPAddress()
end

function _M.SetTimeUpdateStatus(info)
	print(info)
	_M.setTimeUpdateStatus_ = info or "0"
end

function _M.GetTimeUpdateStatus()
	return _M.setTimeUpdateStatus_ or "0"
end

function _M.SetTimezone(zone)
	timedatectl.SetTimezone(zone)
	return true
end

function _M.SetNTP( en, args, noSave )
	timedatectl.SetNTP( en, args, noSave )
	return true
end

function _M.SetLocalTime(time)
	timedatectl.SetTime( time )
	return true
end

function _M.SetUTCTime(time)
	local tinfo = timedatectl.GetTimeDateInfo()
	local tz = tinfo and tinfo["zone"] or "UTC"
	timedatectl.SetTimezone( "UTC" )
	timedatectl.SetTime( time ) --Change time by UTC time.
	timedatectl.SetTimezone( tz )	--Set it back.
	return true
end

function _M.SetTime(method,args)
	if not method then
		return false, "Invalid argument"
	end

	if method == "ntp" then
		if not args then
			return false, "Invalid NTP server address"
		else
			if args == "@china" then
				timedatectl.SetNTP( "true", "time1.aliyun.com time2.aliyun.com time3.aliyun.com time4.aliyun.com time5.aliyun.com time6.aliyun.com time7.aliyun.com" )
			elseif args == "@global" then
				timedatectl.SetNTP( "true", "time1.google.com time2.google.com time3.google.com" )
			else
				timedatectl.SetNTP( "true", args )
			end
		end
	elseif method == "pc" or method == "manual" then
		if not args then
			return false, "Invalid time"
		else
			timedatectl.SetNTP( "false" ) --Disable NTP
			timedatectl.SetTime( args )
		end
	end
	return true
end

function _M.RemoveFile(file)
	local path = string.gsub(string.gsub(file,"(%[)","\\["),"(%])","\\]")
	local done,status,code = os.execute( "/bin/rm -Rf " .. path)
	return done and code == 0
end

function _M.RenameFile(from,to)
	local done,status,code = os.execute( "/bin/mv -f '" .. from .. "' '" .. to .. "'" )
	return done and code == 0
end

function _M.HistoryLogs()
	os.execute( "/bin/mkdir -p /tmp/runlog" )
	local r,stat,code = os.execute( "PATH=/bin:/usr/bin /bin/tar czf /tmp/runlog/history_log.tar.gz -C /data/log ." )
	if r and code == 0 then
		return true
	else
		return false
	end
end

function _M.CurrentLogs()
	os.execute( "/bin/mkdir -p /tmp/runlog" )
	local r,stat,code = os.execute( "PATH=/bin:/usr/bin /bin/tar czf /tmp/runlog/current_log.tar.gz /data/log/*.log" )
	if r and code == 0 then
		return true
	else
		return false
	end
end

function _M.GetIrConnectionParams()
	local _x, func = pcall( loadfile, "/data/configs/jianbo.conf" )
	local cfgData
	if _x and func then
		_x, cfgData = pcall( func )
	end
	if not cfgData then
		cfgData = {
			REMOTE_ADDR = "",
			REMOTE_PORT = 24379,
			TIMEOUT = 15,
			UPDATE_PERIOD = 5,
			REMOTE_AUTH = ""
		}
	end
	return cfgData
end

function _M.SetIrConnectionParams(args)
	if not args or type(args) ~= "table" then
		return false, "Invalid arguments"
	end

	local file = io.open( "/data/configs/jianbo.conf", "w" )
	if not file then
		return false, "Cannot save configurations"
	else
		file:write( "return {\r\n" )
		file:write( "REMOTE_ADDR = '" .. (args.REMOTE_ADDR or "") .. "',\r\n" )
		file:write( "REMOTE_PORT = " .. (tonumber(args.REMOTE_PORT) or "24379") .. ",\r\n" )
		file:write( "TIMEOUT = " .. (tonumber(args.TIMEOUT) or 15) .. ",\r\n" )
		file:write( "REMOTE_AUTH = '" .. (args.REMOTE_AUTH or "" ) .. "'\r\n" )
		file:write( "}" )
		file:close()
		os.execute( "/bin/sync && /bin/sync" )
	end

	os.execute( "/usr/local/bin/coctrl.lua stop ir-connection" )
	os.execute( "/usr/local/bin/coctrl.lua start ir-connection" )
	return true
end

function _M.GetCpuLoad()
	return _M.cpu.load, _M.cpu.idle
end	

function _M.GetMemUse()
	return sysinfo.GetMemUse()
end

function _M.GetDiskSpaces()
	return sysinfo.GetDiskSpaces()
end

function _M.GetPersisTime()
	return sysinfo.GetPersisTime()
end

function _M.Step()
	local load,idle = sysinfo.GetCpuLoad()
	_M.cpu.load = load or 0
	_M.cpu.idle = idle or 99
	if REBOOT_REQUIRED then
		local x, reset = pcall( require, "kiloview." .. _G.DEVICE_TYPE .. ".reset" )
		if not x or not reset then   
			x, reset = pcall( require, "kiloview.reset" )
		end
		if reset then
			reset.Reboot()
		end
	end
end

function _M.GetUSBSerials()
	local usb_drv
	local attrs

	local devices = {}
	--BUG fix: 
	if not lfs.attributes( "/sys/bus/usb-serial/drivers" ) then
		return devices
	end

	for usb_drv in lfs.dir("/sys/bus/usb-serial/drivers") do
		if usb_drv ~= "." and usb_drv ~= ".." and not usb_drv:match("option") then
			--attrs = lfs.attributes( "/sys/bus/usb-serial/drivers/" .. usb_drv )
			local item
			local drv_type = "GENERIC"
			if usb_drv:match("ftdi") then
				drv_type = "FTDI"
			elseif usb_drv:match("pl23") then
				drv_type = "PL230X"
			end

			for item in lfs.dir( "/sys/bus/usb-serial/drivers/" .. usb_drv ) do
				if item ~= "." and item ~= ".." and item:match("tty.+") then
					table.insert( devices, { device=item, driver=drv_type })
				end
			end
		end
	end

	return devices
end

function _M.EnableTelnet(en)
	if en == "true" or en == "yes" then
		os.execute( "/bin/ln -sf /usr/lib/systemd/system/telnet.socket /usr/lib/systemd/system/sockets.target.wants/telnet.socket" )
		os.execute( "/usr/bin/systemctl start telnet.socket" )
	else
		os.execute( "/bin/rm -f /usr/lib/systemd/system/sockets.target.wants/telnet.socket" )
		os.execute( "/usr/bin/systemctl stop telnet.socket" )
	end
end

function _M.GetTelnetEnable()
	local x = io.popen( "/usr/bin/systemctl status telnet.socket", "r" )
	if not x then
		return nil
	else
		local status = "false"
		local ln = x:read("*l")
		while ln do
			local flag = ln:match( "^%s*Active:%s*(%w*)%s*.*" )
			if flag then
				if flag == "active" then
					status = "true"
				else
					status = "false"
				end
				break
			end
			ln = x:read("*l")
		end
		x:close()
		return status
	end
end

--Get hostname
function _M.GetHostName()
	local file = io.open( "/etc/hostname", "r" )
	local hostname = ""
	if file then
		hostname = file:read("l")
		file:close()
		if not hostname then
			hostname = ""
		end
	end
	return hostname
end

local client = require "coaf.client"
local function GetCoAFProxy( objPath, service, loc, timeout )
	local proxy
	local cli

	cli = client.New( service, loc, nil, timeout or 3 ) --3 seconds timeout.
	if not cli then
		_dbg.Error( "STREAMER", "Fail to create connection to service '" .. (loc or "local") .. "/" .. (service or "nil") .. "'" )
		return nil
	end
	proxy = cli:CreateProxy(objPath, true) --Exclusive
	if not proxy then
		_dbg.Error( "STREAMER", "Fail to create object proxy for '" .. (objPath or "unknown") .. "'" )
		return nil
	end
	return proxy
end

--Set hostname
function _M.SetHostName( hostname )
	local file = io.open( "/etc/hostname", "w" )
	if file then
		file:write( hostname .. "\n" )
		file:close()
    end

	os.execute( "/bin/hostname '" .. hostname .. "'" )
	local proxy = GetCoAFProxy("/AVAHI", "nm_daemon")
	if proxy then
		proxy.start()
	else
		_dbg.Info("STREAMER", "Get nm_daemon proxy failed")
	end
end

local function oniperf3Notify( ctx, topic, msg, msgType )
	ctx.status = msg.status
	ctx.code = msg.code
	ctx.ret = msg.ret
end

function _M.iperf3( targetAddr )
	if not targetAddr then
		return false, false, ""
	end
	local ctx = {
		status = "wait",
		ret = "error"
	}

	local topic = tostring(ctx)
	local ticket = g_event:Subscribe( topic, oniperf3Notify, ctx )
	local pid = coaf.Fork( function()
		-- local pfile = io.popen( "/bin/iperf3 -c " .. targetAddr .. " -i 1 -w 8k -t 1", "r" )
		local pfile = io.popen( "/usr/bin/iperf3 -c " .. targetAddr .. " -i 0.5 -t 4", "r" )
		
		if not pfile then
			g_event:NotifyBack( topic, { status="fail" } )
		else
			local ret=""
			local start_i, end_j
			local unit = "M"
			local line = pfile:read("*l")
			local pt
			while line do
				--[  5]   0.00-1.00   sec  5.69 MBytes  47.8 Mbits/sec    0   14.3 KBytes
				pt = line:match( "([0-9%.]+)%s*.bits/sec" )					
				if pt then
					start_i, end_j = string.find(line, 'bits/sec')
					unit = string.sub(line, start_i-1, start_i-1)
					ret = pt .. unit .. "bits/sec"
					break
				end
				line = pfile:read("*l")
			end
			local done, status, code = pfile:close()
			if status ~= "exit" then
				g_event:NotifyBack( topic, { status="fail" } )
			else			
				g_event:NotifyBack( topic, { status="finished", code = code, ret = ret } )
			end
		end
		os.exit(0)
	end)

	local r = coaf.WaitPID( pid )
	if not r then
		g_event:Unsubscribe( ticket )
		return false, false, 100
	else
		if ctx.status == "wait" then
			coaf.Sleep(0.5)	--Wait a while
		end
		g_event:Unsubscribe( ticket )
		if ctx.status == "finished" then
			return true, ctx.code == 0, ctx.ret
		else
			return true, false, ""
		end
	end
end

function _M.BatterySendCommand(command)
	if _G.DEVICE_TYPE ~= "P1" and _G.DEVICE_TYPE ~= "P2" then
		return false, "Invalid DEVICE TYPE"
	end

	if _M.BatteryChecking then
		return false, "It is busying!"
	end

	_M.BatteryChecking = true

    local dev = {device="/dev/ttyS1", baudrate=9600, databits=8, parity="none", stopbits=1, xonxoff=false, rtscts=false}
    local status, err = pcall(function () _M.serial_port = SERIAL(dev) end)
    if not status then
		_M.BatteryChecking = nil
        _dbg.Info( "BatteryControl ERROR: Fail to open serial port!" )
        return false, "Failed to open serial port "
    end
    _dbg.Info("BatteryControl Init Successed!")

	if not _M.serial_port then
		_M.BatteryChecking = nil
        return false, "Serial port not open."
    end

    if not command or (command ~= "gi" and command ~= "sd") then
		_M.BatteryChecking = nil
        return false, "Invalid arguments"
    end

    if command ~= "gi" then
		local task = thread.NewTimer(function(thrObj)
			while not thrObj.terminated do
				_M.serial_port:write("<" .. command .. ">")
        		_M.serial_port:close()
			end
		end, 1)
		if task then
			task:start()
			_M.BatteryChecking = nil
			return true, "  "
		end
		_M.BatteryChecking = nil
		return false, "System Error!"
    end

	_dbg.Info("Send command : " .. command)
    _M.serial_port:write("<" .. command .. ">")

    local rxData
    local status, err = pcall( function() rxData = _M.serial_port:read( 128, 1000 ) end )
    if not status then
        _dbg.Info("BatteryControl. Serial I/O fail, aborted")
        if nil ~= _M.serial_port then
            _M.serial_port:close()
            _M.serial_port = nil
        end
        collectgarbage()
		_M.BatteryChecking = nil
        return false, "Failed to read serial port."
    end

	_M.BatteryChecking = nil
    return true, rxData
end

function _M.Init()
	for file in lfs.dir("/data/configs") do
		if file ~= "." and file ~= ".." and file == "timectl.conf" then
			os.execute("/bin/cp /data/configs/timectl.conf /etc/systemd/timesyncd.conf")
			break
		end
	end
	os.execute( "/usr/bin/timedatectl set-ntp FALSE" )
	if _G.cfg_timer.useNtp then
		os.execute( "/usr/bin/timedatectl set-ntp TRUE" )
	end
end

return _M

