#!/usr/bin/env lua
local CONFIG_FILE = "/data/configs/web.conf"

local coaf = require "coaf.core"
local _dbg = require "coaf.debug"

-- Change current dir to app location
coaf.ChDir(coaf.GetAppDir(arg))

require "features"


--NOTE: Since nginx need write temporary uploaded file to /tmp, so must set r/w previliges.
os.execute( "/bin/chmod 777 /tmp" )

local socket = require "cosocket.core"
local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
local service = require "coaf.service"
local roll = require "coaf.log.rollfile"

local event = require "coaf.event"
local thread = require "coaf.thread" 

local withLog = false
local _x
for _x=1, #arg do
	if arg[_x] == "--log" then
		withLog = true
		break
	end
end

if withLog then
	-- ** initialize the log file **
	local roll = require "coaf.log.rollfile"
	os.execute( "/bin/mkdir -p /data/log/" )
	local logF = roll.Create("/data/log/web_daemon.log")
	if not logF or not logF:init() then
		_dbg.Error("WEB_DAEMON", "Fail to open log file: /data/log/web_daemon.log, so the log messages should be output to console.")
		withLog = false
	else
		_dbg.AddLogger("logfile", logF, nil, nil)
	end
	-- ** log **
end

coaf.Init( "web_daemon" )

_G.print = function(...)
	_dbg.Info1( "WEB_DAEMON", ... )
end

if withLog then
	local stdio = require "coaf.log.stdio"
	stdio.SetDefaultFilterName( "WEB_DAEMON" )
	stdio.Start()
end

-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-- CoAF setup above.
-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-- DBus client setup.
local dbusCli = require "kiloview.dbusClients"
dbusCli.Init( "SYSTEM" )

-- CoAF serivce initialization.
service.InitSocket()
client.InitSocket( socket, adapter )

----------------------------------------
require "server_config"
local cfgM = require "kiloview.configManager"
local presetM = require "kiloview.presetManager"
local getset = require "kiloview.getSetHelper"
local upgradeProc = require "kiloview.upgradeProc"

--Using CoAF.
presetM.using_dbus = false

_G.WEB_CONFIG_CHANGED = false
_G.CONFIG_LOADING = true
_G.webConfig = CONFIGS.InstanceOf( WebConfig_Template, "webConfig" )
_G.webConfig:LoadFile( CONFIG_FILE )
_G.webConfig:ResetChanges()
_G.CONFIG_LOADING = false

local SERVER = require "server"
local HOSTAP_CTRL = require "kiloview.hostap_ctrl"
local SYS_STATUS = require "sysStatus"

_G._ENV = {}

local cosvc_WebDaemon
local period_timer

local compatible_svc = {
	Identifier = "Web",
	FriendlyName = "ENCODER Web service",
	ConfigPrefix = "webConfig",
	Version = "1.0"
}

function compatible_svc:Activate()
end

function compatible_svc:Start()
end

function compatible_svc:Stop()
end

function compatible_svc:Reset()
end

function compatible_svc:Terminate()
end

function compatible_svc:Control(action,params)
	if action == "activate" then
		self:Activate()
		return 0, {}
	elseif action == "start" then
		self:Start()
		return 0, {}
	elseif action == "stop" then
		self:Stop()
		return 0, {}
	elseif action == "Reset" then
		self:Reset()
		return 0, {}
	elseif action == "terminate" then
		self:Terminate()
		return 0, {}
	else
		return -1, {}
	end
end

function compatible_svc:Set(params)
	local cnt = getset.SetConfigureValues( self.ConfigPrefix, params )
	return cnt
end

function compatible_svc:SetEnv(params)
	local cnt = getset.SetValues( "_ENV", params )
	return cnt
end

function compatible_svc:Get(inParams)
	local cnt = getset.GetValues( self.ConfigPrefix, inParams )
	return cnt,inParams
end

function compatible_svc:GetEnv(inParams)
	local cnt = getset.GetValues( "_ENV", inParams )
	return cnt,inParams
end

function compatible_svc:RestoreDefault()
	return 0
end

----------------------------------------

local function periodTask()
	SYS_STATUS.Step()
	if _G.WEB_CONFIG_CHANGED then
		SERVER.UpdateNginxConfig()
		_G.WEB_CONFIG_CHANGED = false
	end
end
----------------------------------------

local function FileExist( fn )
	local file = io.open( fn, "rb" )
	if not file then
		return false
	else
		file:close()
		return true
	end
end

local function FileSize( fn )
	local file = io.open( fn, "rb" )
	if not file then
		return -1
	else
		local size = file:seek( "end" )
		file:close()
		return size
	end
end
--------------------------------
local timerService = require "sysctrl_timerService"
_G.cfg_timer = require("timer_config").Load()

timerService.Init()

---

local reboot_timer                                                                                                                           
                                                                                                                                             
local function loopReboot()                                                                                                                  
        if timerService then                                                                                                                 
                timerService.rebootTimer()                                                                                                   
        end                                                                                                                                  
end  

local function xpopen(cmd,param,reg)
	local f 
	print("cmd = ", cmd)
	f = io.popen( cmd, "r" )
	if f then
		local l = f:read("l")
		while l do
			print("lines = ", l)
			if l:match( param ) then
				local _s,_e,ret = string.find(l,reg)
				if ret then
					print("ret = ", ret)
					return true,ret
				end
			end
			l = f:read("l")
		end
		f:close()
	end
	return false,nil
end

local function poly4G_timer()
	local done,ret
	done,ret = xpopen('/usr/bin/ppp_talk -p "/dev/ttyUSB20" --say "at+sysinfo\\r" -e "(.*)/\\1/-" ',"SYSINFO",":(%d+),")
	if not done or (ret and tonumber(ret) ~= 2)  then
		return
	end
	coaf.Sleep(1)

	done,ret = xpopen('/usr/bin/ppp_talk -p "/dev/ttyUSB20" --say "at+rrcstate\\r" -e "(.*)/\\1/-" ',"RRCSTATE",":%s-(%d+),")
	if not done or (ret and tonumber(ret) ~= 0)  then
		return
	end
	coaf.Sleep(2)

	os.execute('/usr/bin/ppp_talk -p "/dev/ttyUSB20" --say "at+cgact=0,1\\r" -e "(.*)/\\1/-" ')
	os.execute('/usr/bin/ppp_talk -p "/dev/ttyUSB20" --say "at+cgdcont=1,\\"IP\\",\\"cmnet\\"\\r" -e "(.*)/\\1/-" ')
	coaf.Sleep(1)

	done,ret = xpopen('/usr/bin/ppp_talk -p "/dev/ttyUSB20" --say "at+cgact=1,1\\r" -e "(.*)/\\1/-" ',"OK","(OK)")
	coaf.Sleep(5)

	if done then
		os.execute('/usr/bin/ppp_talk -p "/dev/ttyUSB20" --say "at+dhcpctrl=1\\r" -e "(.*)/\\1/-" ')
	else
		os.execute('/usr/bin/ppp_talk -p "/dev/ttyUSB20" --say "at+cgact=0,1\\r" -e "(.*)/\\1/-" ')
		os.execute('/usr/bin/ppp_talk -p "/dev/ttyUSB20" --say "at+pwr\\r" -e "(.*)/\\1/-" ')
		return
	end

	coaf.Sleep(1)
	done,ret = xpopen('/usr/bin/ppp_talk -p "/dev/ttyUSB20" --say "at+cgpaddr\\r" -e "(.*)/\\1/-" ',"CGPADDR",",\"(%d.+)\",")
	if not done or (ret and #ret <= 0) then
		os.execute('/usr/bin/ppp_talk -p "/dev/ttyUSB20" --say "at+cgact=0,1\\r" -e "(.*)/\\1/-" ')
	end
end

---

----------------------------------------
local function Main()
	_G.g_event = event.createNew("web_daemon", true)
	HOSTAP_CTRL.Init()
	cosvc_WebDaemon = service.New( "web_daemon", "unix:/var/run/web_daemon.sock" )
	if cosvc_WebDaemon then
		_dbg.Info( "WEB_DAEMON", "CoService of 'web_daemon' is created." )
		cosvc_WebDaemon:CreateCoServerProxy()
		cosvc_WebDaemon:ExposeObject( "/", compatible_svc )
		cosvc_WebDaemon:ExposeObject( "/SERVER", SERVER )
		cosvc_WebDaemon:ExposeObject( "/HOSTAP_CTRL", HOSTAP_CTRL )
		cosvc_WebDaemon:ExposeObject( "/SYS_STATUS", SYS_STATUS )
		cosvc_WebDaemon:ExposeObject( "/CONFIG_MANAGER", cfgM )
		cosvc_WebDaemon:ExposeObject( "/PRESET", presetM )
		cosvc_WebDaemon:ExposeObject( "/timer", timerService )
		cosvc_WebDaemon:ExposeObject( "/UPGRADE", upgradeProc )
	else
		_dbg.Error( "WEB_DAEMON", "Fail created CoService!" )
	end

	if FileSize( "/data/log/web_error.log" ) >= 1024*1024 then
		os.execute( "/bin/rm -f /data/log/web_error.tar.gz" )
		os.execute( "PATH=/bin:/usr/bin /bin/tar czf /data/log/web_error.tar.gz /data/log/web_error.log" )
		os.execute( [[/bin/echo "~~ TRANCATED ( `/bin/date` ) ~~" > /data/log/web_error.log]] ) --Clear current
	end

	period_timer = coaf.NewTimer( periodTask, 0.01, 3 )
	if period_timer then
		period_timer:start()
	end

	if _G.FEATURE_4G_POLY then
		coaf.NewTimer( poly4G_timer, 10, 40 ):start()
	end

	coaf.Sleep(3)	--Wait for other services starting.
	SERVER.UpdateNginxConfig()
	SERVER.StartNginx()
	SYS_STATUS.Init()

        reboot_timer = thread.NewTimer( loopReboot, 0.01, 20)                                                          
                                                                                                                       
        if reboot_timer then                                                                                           
                reboot_timer:start()                                                                                   
        end               
end

----------------------------------------

coaf.Start(Main)

_dbg.Info( "WEB_DAEMON", "Web daemon service is terminated." )

