local cjson = require "cjson"
local _M = {
}
local function QuoteString(s)
	local x,n = string.gsub( s, '(%")', '\\%1' )
	if x then
		return x
	else
		return s
	end
end

local MSG = {}
local function sayFields(preCount, fields)
	local k,v
	for k,v in pairs(fields) do
		if preCount > 0 then
			ngx.say( ",")
		end
		if type(k) == "string" then
			ngx.say( '"', QuoteString(k), '":' )
		end

		if type(v) == "string" then
			ngx.say( '"', QuoteString(v), '"' )
		elseif type(v) == "boolean" then
			ngx.say( v and 'true' or 'false' )
		elseif type(v) == "table" then
			ngx.say( cjson.encode(v) )
		else
			ngx.say(v)
		end
		preCount = preCount + 1
	end
end


function MSG.OK(msg,fields)
	ngx.say( '{"result":"ok"' )
	if msg and msg ~= "" then
		ngx.say( ',"msg":"', QuoteString(msg), '"' )
	end

	if type(fields) == "table" then
		sayFields( 1, fields )
	end
	ngx.say( '}' )
end

function MSG.ERROR(msg,fields)
	ngx.say( '{"result":"error"' )
	if msg then
		ngx.say( ',"msg":"', QuoteString(msg), '"' )
	else
		ngx.say( ',"msg":"未知错误"' )
	end

	if type(fields) == "table" then
		sayFields( 1, fields )
	end
	ngx.say( '}' )
end


local args = {}
local args_get = ngx.req.get_uri_args()
ngx.req.read_body()
local args_post = ngx.req.get_post_args() or {}
local req_headers = ngx.req.get_headers()
args.head = { platform = req_headers and req_headers.platform or "" }
for k, v in pairs(req_headers or {}) do
    args.head[k] = tostring(v):lower()
end
local content_type = req_headers and (req_headers["Content-Type"] or req_headers["content-type"]) or nil
if content_type and content_type:match("application/json") then
    --Is JSON
    local cjson = require "cjson"
    local parsed_data={}
    for k,v in pairs(args_post) do
        local item=cjson.decode(k)
        if item then
            table.insert(parsed_data, item)
        end
    end
    if #parsed_data == 1 then
        args_post = parsed_data[1]
    else 
        args_post = parsed_data
    end
end

local arrIdx
for k,v in pairs(args_post) do
    if type(k) == "number" then
        arrIdx = nil
    else
        arrIdx = k:match("(.*)%[%]")
    end
    if arrIdx then
        args[arrIdx] = v
    else
        args[k] = v
    end
end

for k,v in pairs(args_get) do
	if type(k) == "number" then
		arrIdx = nil
	else
		arrIdx = k:match("(.*)%[%]")
	end
	if arrIdx then
		args[arrIdx] = v
	else
		args[k] = v
	end
end


if not args.user or not args.password or not args.url then
    MSG.ERROR("Invalid arguments")
    return
end

local http = require "resty.http"
local timeout = 6000

local client = http.new()
client:set_timeout(timeout)

local contentType = "Content-Type"
local jsonEncode = "application/json; charset=utf-8"
local formEncode = "application/x-www--form-urlencoded"

local res, err = client:request_uri("http://" .. args.url, {
    method = "GET",
    headers = {
        contentType = jsonEncode,
        Authorization = "Basic " .. ngx.encode_base64(args.user .. ":" .. args.password)
    }
})
if err or not res then
    MSG.ERROR("Authorization Failed!")
    return
end

if res.status == ngx.HTTP_OK then
    MSG.OK("", {data = {
        Authorization = ngx.encode_base64(args.user .. ":" .. args.password)
    }})
else
    MSG.ERROR("Authorization Failed!")
end