local cjson = require "cjson"
local _M = {
}

local function QuoteString(s)
	local x,n = string.gsub( s, '(%")', '\\%1' )
	if x then
		return x
	else
		return s
	end
end

local MSG = {}
local function sayFields(preCount, fields)
	local k,v
	for k,v in pairs(fields) do
		if preCount > 0 then
			ngx.say( ",")
		end
		if type(k) == "string" then
			ngx.say( '"', QuoteString(k), '":' )
		end

		if type(v) == "string" then
			ngx.say( '"', QuoteString(v), '"' )
		elseif type(v) == "boolean" then
			ngx.say( v and 'true' or 'false' )
		elseif type(v) == "table" then
			ngx.say( cjson.encode(v) )
		else
			ngx.say(v)
		end
		preCount = preCount + 1
	end
end


function MSG.OK(msg,fields)
	ngx.say( '{"result":"ok"' )
	if msg and msg ~= "" then
		ngx.say( ',"msg":"', QuoteString(msg), '"' )
	end

	if type(fields) == "table" then
		sayFields( 1, fields )
	end
	ngx.say( '}' )
end

function MSG.ERROR(msg,fields)
	ngx.say( '{"result":"error"' )
	if msg then
		ngx.say( ',"msg":"', QuoteString(msg), '"' )
	else
		ngx.say( ',"msg":"未知错误"' )
	end

	if type(fields) == "table" then
		sayFields( 1, fields )
	end
	ngx.say( '}' )
end

local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
adapter.timeoutUnit = 1000
client.InitSocket(nil, adapter)

local function GetProxy(objPath, service, loc, timeout, socket_addr)
	local proxy
	local cli

	cli = client.New( service, loc, socket_addr, timeout or 5 ) --5 seconds timeout.
	if not cli then
		_dbg.Error( "CO", "Fail to create connection to service '" .. (loc or "local") .. "/" .. (service or "nil") )
		return nil
	end
	proxy = cli:CreateProxy(objPath, true) --Exclusive
	if not proxy then
		_dbg.Error( "CO", "Fail to create object proxy for '" .. (objPath or "unknown") .. "'" )
		return nil
	end
	return proxy
end

local proxy = GetProxy("/discovery", "DiscoveryRE")
if not proxy then
    MSG.ERROR("Failed to connect Server")
    return
end

local is_ok, list = proxy.GetList()
proxy:Destroy()
if not is_ok then
    MSG.ERROR("Get list failed!")
    return
end

MSG.OK("", {data = list})