------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=IOLED}: 电池状态获取以及控制电池

##ARGUMENTS{GET,POST}: 
	@@CMD{STR,*}: 指定需要执行的相应命令，例如：
		%%gi: 获取电池信息
		%%sd: 关机
	~~

##RETURNS{DEFAULT}: 
    ++Data{OBJ}:
		**MV: 适配器电压
        **BV: 电池电压
        **BP: 电池电量
        **T1: T1温度
        **T2: T2温度
        **FN: 风扇状态,0为风扇关闭，1为风扇打开
	~~
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )
local args = require("M_Request").VerifyAndGetArgs()
if not args or not args.CMD or (args.CMD ~= "gi" and args.CMD ~= "sd" and args.CMD ~= "of" and args.CMD ~= "cf") then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid arguments"
}
]])
	return
end

local is_ok, skip, rxData = rpc.Call( "SYS_STATUS.BatterySendCommand", args.CMD )
if not is_ok then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Cannot connect service"
}
]])
	return
end

if not skip then
	cgilua.put([[
{
	"Result":400,
	"Status":"]] .. rxData .. [["
}
]])
    return
end

local function split( str,reps )
    local resultStrList = {}
    string.gsub(str,'[^'..reps..']+',function ( w )
        table.insert(resultStrList,w)
    end)
    return resultStrList
end

rxData = rxData:sub(1, #rxData - 1):sub(2)

local info = {}
local dataTbl = split(rxData, " ")
for _, v in ipairs(dataTbl) do
    local key, value = v:match("(.+)=(.+)")
    table.insert(info, '"' ..  key .. '":"' .. value .. '"')
end

local str = table.concat(info, ",\n        ")

cgilua.put([[{
    "Result":200,
    "Status":"OK",
    "Data":{
        ]] .. str .. "\n    }\n}")

