local rpc = require "kiloview.ringRpc"

local _M = {}

--Get hostname
function _M.GetHostName()
	local x,hostname = rpc.Call( "SYS_STATUS.GetHostName" )
	if x then
		return hostname or ""
	else
		return ""
	end
end

--Set hostname
function _M.SetHostName( hostname )
	rpc.Call( "SYS_STATUS.SetHostName", hostname )
	rpc.Call("BUS.Set", {
		["streamer.stream[0].NDI_server.host_name"] = hostname,
	})
end

--Get friendly name
function _M.GetFriendlyName()
	local r,cc,gets = rpc.Call( "BUS.Get", { ["onvif.deviceName"] = "" } )
	if not r or not gets then
		return nil
	else
		return gets["onvif.deviceName"]
	end
end

--Set friendly name
function _M.SetFriendlyName( fname )
	local r,x = rpc.Call( "BUS.Set", { ["onvif.deviceName"] = fname } )
	if not r or not x then
		return false
	else
		return true
	end
end

function _M.GetNDIConfig()
	local is_ok, skip, result = rpc.Call("BUS.Get", {
		["streamer.stream[0].NDI_server.device_name"] = "",
		["streamer.stream[0].NDI_server.group_name"] = "",
		["streamer.stream[0].NDI_server.connection"] = "",
		["streamer.stream[0].NDI_server.mcast_prefix"] = "",
		["streamer.stream[0].NDI_server.mcast_mask"] = "",
		["streamer.stream[0].NDI_server.server_addr"] = "",
		["streamer.stream[0].NDI_server.enabled"] = "",
	})
	if not is_ok or not skip then
		return {}
	end

	return result
end

function _M.SetNDIConfig(args)
	local config = {
		["streamer.stream[0].NDI_server.group_name"] = args.GROUP_NAME,
		["streamer.stream[0].NDI_server.connection"] = args.NDI_CONNECTION,
		["streamer.stream[0].NDI_server.mcast_prefix"] = args.NDI_MCAST_PREFIX,
		["streamer.stream[0].NDI_server.mcast_mask"] = args.NDI_MCAST_MASK,
		["streamer.stream[0].NDI_server.host_name"] = args.HOST_NAME,
		["streamer.stream[0].NDI_server.server_addr"] = args.NDI_DISCOVERY_SERVER,
		["streamer.stream[0].NDI_server.enabled"] = args.NDI_ENABLE,
		["streamer.stream[0].NDI_server.device_name"] = args.FRIENDLY_NAME
	}
	local is_ok, skip = rpc.Call("BUS.Set", config)
	if not is_ok or not skip then
		return false
	end
	return true
end

------------------------------------------

return _M

