------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=PORTFWD}: Add a port forwarding setting
##ARGUMENTS{GET,POST}: 
  @@Protocol{STR,*}: Specify network protocol, 'tcp' or 'udp' is valid.
  @@Listen{STR,*}: Listen at local network interface and port. Format is "address:port", which address can be '*' that means listen at any network interface.
  @@Target{STR,*}: Forward the packet to target, format is "address:port", which address is an IP or domain name.
  @@Over{STR}: Optional, but it's important for multiple interface transmission. Specify what network interface this forwarding works on. Refer to ^^REF{getNetInterface.lua} to get all valid interfaces, check the 'Type' field in result of ^^REF{getNetInterface.lua} to get the special interface name of what you want. For example, the interface type equals to 'MODEM' means it is a 3G/4G network interface.
  @@MaxPacketSize{INT}: Optional. The maximum packet size for data transmission, in KB. The default value is 64 (KB), this suits general applications. Do not set it less than MTU (1500 bytes) size is suggested!
  @@SocketRxBuffer{INT}: Optional. Socket receiving buffer size, in KB. The default value is 256 (KB). Don't make it to be too small.
  @@SocketTxBuffer{INT}: Socket sending buffer size, in KB. The default value is 256 (KB).

##RETURNS{DEFAULT}: 

]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args then
	cgilua.put([[
{
	"Result":400,
	"Status":"invalid-arguments"
}
]])
	return
end

local r,done,msg = rpc.Call( "PORT_FWD.Add", args.Protocol, args.Listen, args.Target, args.Over, args.MaxPacketSize, args.SocketRxBuffer, args.SocketTxBuffer )

if not r or not done then
	if msg == "exist" then
		cgilua.put([[
		{
			"Result":201,
			"Status":"exist"
		}
		]])
	else
		cgilua.put([[
		{
			"Result":400,
			"Status":"]] .. (msg or "unknown-error") .. [["
		}
		]])
	end
	return
end

cgilua.put([[
{
	"Result":200,
	"Status":"OK"
}
]])

