------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=ETHERNET}: 连接(或重新连接)一个以太网Connection
##ARGUMENTS{POST}: 
  @@DeviceObject: 可选参数。如果指定，则为网络接口的ObjectID（参见 ^^REF{getNetInterface.lua}）；如果没有指定，则为默认的以太网接口

##RETURNS{DEFAULT}: 

##NOTES:
  当调用^^REF{setEthernet.lua}修改以太网连接参数后，参数不会立即生效，除非重启设备或调用本API重新连接。
]]

local ETH = require "M_EthernetSet"
local cgilua = require "cgilua"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid request arguments"
}
]])
	return
end

local objectId = args.DeviceObject
if not objectId or objectId == "" or objectId == "/" then
	local device = ETH.GetDefaultEthernet()
	if not device then
		cgilua.put([[
	{
		"Result":500,
		"Status":"FAIL: Fault to get ethernet device"
	}
	]])
		return
	end
	objectId = device.Object
end

local r,msg = ETH.ConnectConnection( objectId )

cgilua.put([[
{
	"Result":]] .. (r and "200" or "400") .. [[,
	"Status":"]] .. (msg or "OK") .. [["
}
]])


