------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=AV-SOURCE}: 获得音频增益
##ARGUMENTS{GET,POST}: 
    @@Source: 指定要获取音频增益的信号源名称（来自于 ^^REF{getAudioSource.lua} 中返回的 Sources 列表）
            >> 注：不同产品型号，数字音频源的名称可能不一样，例如HDMI/SDI/...。
            >> 为了软件编程的一致，无论HDMI/SDI/...哪种
            音频源，均可以用DIGITAL来代表；同样，模拟音频源可能为LINE/MIC/...，可以用ANALOG来代表。

			>> <b>注意：</b>当前设备仅支持获取模拟音频输入的增益！对于非模拟音频输入的增益，返回值恒为0。

##RETURNS: 
	++Data{OBJ}:
        **Gain{NUM}: 增益值(db)
	~~
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
local source = "LINE"
if args and args.Source then
	source = args.Source
end
local gain = 0

source = source:lower()
if source == "line" or source == "analog" then
	local r,cc,rets = rpc.Call( "BUS.Get", { ["avio.audio.line_gain"] = 0 } )
	if r and rets then
		gain = rets["avio.audio.line_gain"]
	else
		gain = nil
	end
elseif source == "mic" then
	local r,cc,rets = rpc.Call( "BUS.Get", { ["avio.audio.mic_gain"] = 0 } )
	if r and rets then
		gain = rets["avio.audio.mic_gain"]
	else
		gain = nil
	end
end

if not gain then
	cgilua.put([[
	{
		"Result":400,
		"Status":"Get audio gain error"
	}
]])
else
	cgilua.put([[
	{
		"Result":200,
		"Status":"OK",
		"Data":{
			"Gain":]] .. string.format( "%.2f", tonumber(gain) ) .. [[
		}
	}
]])
end

