------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=ETHERNET}: 获得Ethernet接口的Failsafe IP地址配置
>>{BOLD}关于Failsafe IP地址：
>> 在Ethernet的IP地址配置过程中，难免存在以下情况：
>> (1) 忘记手动配置的IP地址
>> (2) 启用DHCP，但DHCP无法得到有效的地址
>> (3) 启用DHCP，虽然得到地址但无法获知该地址是多少
>> 为解决以上问题，您可以为Ethernet配置一个Failsafe（简单、易记）地址。当以上情况发生的时候，您可以通过访问
Failsafe地址继续管理设备。

##ARGUMENTS{GET,POST}: 
	@@DeviceObject: 可选参数。如果指定，则为网络接口的ObjectID（参见^^REF{getNetInterface.lua}）；如果没有指定，则为默认的Ethernet接口
	
##RETURNS: 
	++Data{OBJ}:
		**Enable: Failsafe地址功能开启。0表示关闭，1表示开启
		**IP: Failsafe IP地址
		**Netmask: Failsafe 子网掩码 (如255.255.255.0)
	~~
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local gets = { ["hostapm.localLink.ip"] = "", ["hostapm.localLink.netmask"] = "", ["hostapm.localLink.enable"] = "true" }
local r, cc
r,cc,gets = rpc.Call( "BUS.Get", gets )

if not r or not gets then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Fail to get arguments"
}
]])
else
	cgilua.put([[
{
	"Result":200,
	"Status":"OK",
	"Data":{
		"Enable":]] .. ( gets["hostapm.localLink.enable"] == "true" and 1 or 0 ) .. [[,
		"IP":"]] .. ( gets["hostapm.localLink.ip"] or "" ) .. [[",
		"Netmask":"]] .. ( gets["hostapm.localLink.netmask"] or "") .. [["
	}
}
]])

end

