------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=ONVIF}: 获取Onvif用户信息
>>{BOLD} 注意：出于安全性考虑，获取的信息中不会包含用户密码。

##ARGUMENTS:
	@@Username{STR}: 可选参数。指定要获取的用户名。如果没有指定，则获取所有的Onvif用户列表。

##RETURNS:
	++Data{ARR}: 数组的每一项为以下内容：
	++{OBJ}:
		**Username{STR}: 用户名
		**Name{STR}: Onvif用户的名称
		**Role{STR}: Onvif角色
	~~
	~~
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
local results = {}


local MAX_USERS=10

local gets = { ["onvif.adminName"] = "" }
local i

for i=0, MAX_USERS-1 do
	gets[ "onvif.users[" .. i .. "].username" ] = ""
	gets[ "onvif.users[" .. i .. "].name" ] = ""
	gets[ "onvif.users[" .. i .. "].role" ] = ""
end

local r,cc
r,cc,gets = rpc.Call( "BUS.Get", gets )
if not r or not gets then
	cgilua.put([[
	{
		"Result":405,
		"Status":"ERROR: Fail to get user(s)"
	}]] )

	return
end

cgilua.put([[
{
	"Result":200,
	"Status":"OK",
	"Data":[
]])

if args and args.Username and args.Username ~= "" then
	if args.Username == "admin" then
		cgilua.put( "	{ \"Username\":\"" .. args.Username .. "\", \"Name\":\"" .. ( gets["onvif.adminName"] or "" ) .. "\", \"Role\":\"administrator\" }\n" )
	else
		for i=0, MAX_USERS-1 do
			if gets[ "onvif.users[" .. i .. "].username" ] == args.Username then
				cgilua.put( "	{ \"Username\":\"" .. args.Username .. "\", \"Name\":\"" .. ( gets["onvif.users[" .. i .. "].name"] or "" ) .. "\", \"Role\":\"" .. ( gets["onvif.users[" .. i .. "].role"] or "" ) .. "\" }\n" )
				break
			end
		end
	end
else
	cgilua.put( "	{ \"Username\":\"admin\", \"Name\":\"" .. ( gets["onvif.adminName"] or "" ) .. "\", \"Role\":\"administrator\" }\n" )
	for i=0, MAX_USERS-1 do
		if gets[ "onvif.users[" .. i .. "].username" ] ~= "" then
			cgilua.put( "	,{ \"Username\":\"" .. gets[ "onvif.users[" .. i .. "].username"] .. "\", \"Name\":\"" .. gets["onvif.users[" .. i .. "].name"] .. "\", \"Role\":\"" .. gets["onvif.users[" .. i .. "].role"] .. "\" }\n" )
		end
	end
end

cgilua.put([[
	]
}]] )

