------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------
--[[

##ABSTRACT{G=IMAGE}: 获取视频后级（编码）图像裁切参数
>> 后级图像裁切，是相对于前级裁切（参考 ^^REF{getSourceCrop.lua} / ^^REF{setSourceCrop.lua}）而言的。后级图像裁切对KILOVIEW全系列编码器产品适用。
>> 后级图像裁切，是在视频即将进行编码时，可以对经过前级处理后的图像（对于G/M系列产品）或直接输入的图像（对于U系列产品）进行裁切，从而实现局部
感兴趣区域的编码。如果对于G/M系列产品，这也许和前级裁切功能有所重复，但也可以组合前级裁切完成更高级的图像处理，当然也可以只考虑二者之一的功能开启；对于
U系列产品，这是实现图像裁切功能的手段。

##ARGUMENTS{POST}: 
无

##RETURNS{DEFAULT}:
	++Data{OBJ}:	裁切状态和参数
		**Enable{INT}:	0|1。0表示关闭裁切；1表示打开裁切
		**Left{NUM}:	如果Enable=1，表示裁切位置的左边起始位置（Pixel）
		**Top{NUM}: 如果Enable=1，表示裁切位置的顶部起始位置（Line）
		**Width{NUM}: 如果Enable=1，表示裁切的画面宽度（Pixel）
		**Height{NUM}: 如果Enable=1，表示裁切的画面高度（Line）
	~~
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local argPrefix = "avio.video"
local reqParams = {
	[argPrefix .. ".out_crop.enable"] = 0,
	[argPrefix .. ".out_crop.cropX"] = 0,
	[argPrefix .. ".out_crop.cropY"] = 0,
	[argPrefix .. ".out_crop.cropW"] = 0,
	[argPrefix .. ".out_crop.cropH"] = 0
}

local r, cc, gets = rpc.Call( "BUS.Get", reqParams )
if not r or not gets then
cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Fail to get picture cropping configurations",
}]])
else
cgilua.put([[
{
	"Result":200,
	"Status":"OK",
	"Data":{
		"Enable":]] .. gets[argPrefix .. ".out_crop.enable"] .. [[,
		"Left":]] .. gets[argPrefix .. ".out_crop.cropX"] .. [[,
		"Top":]] .. gets[argPrefix .. ".out_crop.cropY"] .. [[,
		"Width":]] .. gets[argPrefix .. ".out_crop.cropW"] .. [[,
		"Height":]] .. gets[argPrefix .. ".out_crop.cropH"] .. [[
	}
}]])
end


