------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=ENCODING}: 获得指定码流（主码流/子码流/Motion-JPEG码流）的媒体服务信息。

>>{BOLD} 媒体服务，是指RTSP/RTMP/HLS/...等服务。
>> KILOVIEW编码器产品，支持多种流媒体服务功能，并且每一项功能均可设定开启/关闭，
或动态添加/删除服务[ 说明：RTSP服务和KMP服务为固定存在的服务，不能添加/删除]。

##ARGUMENTS{GET,POST}: 
    @@Stream{*}: 
        指定要获取媒体服务的码流。
        %%main: 主码流
        %%sub: 子码流
		%%jpeg: Motion-JPEG码流
    ~~

    @@Config{INT}: 可选。
        %%1: 指明需要获取媒体服务的配置参数。
        %%0: (或未指定，默认）则不返回配置参数。
    ~~

    @@Status{INT}: 可选。
        %%1: 指明需要获取媒体服务的状态（默认）。
        %%0: 不返回媒体服务的状态。
    ~~

##RETURNS: 

++Data{OBJ}:
    **Stream: 当前码流
        %%main: 主码流
        %%sub: 子码流
		%%jpeg: Motion JPEG码流
    ~~

    **Enable{INT}: 
        当前码流的开启状态。
        %%0: 关闭。如果值为0，则以下参数均不会返回。
        %%1: 开启。
    ~~

    ++ServiceConfigs{ARR}:  每一项代表一个媒体服务的配置参数。仅当请求参数Config=1时返回。
        ++{OBJ}:
            **ID: 媒体服务的Unique ID。值可能为
                %%Rtsp_server:
                %%Kmp_pullingServer:
                %%Dynamic_services[0]:
                %%Dynamic_services[1]:
                %%Dynamic_services[2]:
                %%Dynamic_services[3]:
                %%Dynamic_services[4]:
                %%Dynamic_services[5]:
                %%Dynamic_services[6]:
                %%Dynamic_services[7]:
            ~~

            **Type: 服务类型, 值可能为以下列表中的某一个。
                %%Rtsp_server: RTSP服务
                %%Kmp_pullingServer: KILOVIEW Media Protocol的拉取服务
                %%Rtmp_push: RTMP推送服务
                %%Rtp_push: RTP推送或组播服务
                %%Kmp_push: KILOVIEW Media Protocol的推送服务
                %%Sproxy_push: 特定OEM客户的推送服务，一般情况下请勿考虑
                %%TS_push: TS/UDP推送或组播服务
                %%HLS: HLS(HTTP Live Streaming)服务
            ~~

            这些服务类型可能随着功能的扩展而增加。

            ++Rtsp_server{OBJ}: 对应RTSP服务的配置参数 [仅当 Type = Rtsp_server 时返回]
				**port{INT}:    RTSP服务端口，默认为554
				**session:      RTSP服务的SessionID
				**ssm{INT}: 0|1,是否开启RTSP Multicast服务。1表示开启，0表示关闭。
				**multicast_addr:   RTSP组播的Multicast地址
				**multicast_video_port{INT}: multicast视频端口
				**multicast_audio_port{INT}: multicast音频端口
				**multicast_ttl{INT}: multicast TTL值
            ~~
            ~~

            ++Kmp_pullingServer{OBJ}: 对应Kmp_pullingServer服务的配置参数 [仅当 Type = Kmp_pullingServer 时返回]
				**enabled{INT}: 0|1，是否开启KMP拉流服务。0关闭，1开启。
            ~~
            ~~

            ++Rtmp_push:    对应Rtmp推送服务的配置参数 [仅当 Type = Rtmp_push 时返回]
				**enabled{INT}: 0|1, 是否开启RTMP推流。0关闭，1开启。
				**pubPoint: RTMP推流的发布地址URL
				**username: RTMP推流身份认证用户名
				**password: RTMP推流身份认证用户密码
				**timeout{INT}: RTMP连接超时（单位：秒）
				**reconnect_interval{INT}: RTMP断开后重连的间隔时间（单位：秒）
            ~~
            ~~

            ++Rtp_push:     对应Rtp推送服务的配置参数 [仅当 Type = Rtp_push 时返回]
				**enabled{INT}: 0|1, 是否开启Rtp推送功能。0关闭，1开启。
				**target_video_IP: 视频目标推送IP或组播IP
				**target_video_port{INT}: 视频目标推送端口
				**target_audio_IP: 音视频目标推送IP或组播IP
				**target_audio_port{INT}: 音频目标推送端口
                **ttl{INT}: 如果目标IP为Multicast地址，指定组播TTL值
            ~~
            ~~

            ++Kmp_push:     对应KMP推送服务的配置参数 [仅当 Type = Kmp_push 时返回]
				**enabled{INT}: 0|1, 是否开启KMP推送。0关闭，1开启。
				**pubPoint: KMP推流的发布地址URL
				**username: KMP推流身份认证用户名
				**password: KMP推流身份认证用户密码
				**timeout{INT}: KMP连接超时（单位：秒）
				**reconnect_interval{INT}: KMP断开后重连的间隔时间（单位：秒）
            ~~
            ~~

            ++Sproxy_push:  对应Sproxy推送服务的配置参数 [仅当 Type = Sproxy_push 时返回]
                **...: 特殊应用，参数暂未提供
            ~~
            ~~

            ++TS_push:  对应TS/UDP推送服务的配置参数 [仅当 Type = TS_push 时返回]
				**enabled{INT}: 0|1, 是否开启TS推送。0关闭，1开启。
                **address: 推送目标IP地址或组播IP
                **port{INT}: 推送端口
                **ttl{INT}: 组播的TTL
            ~~
            ~~

            ++HLS:  对应HLS服务的配置参数 [仅当 Type = HLS 时返回]
				**enabled{INT}: 0|1,是否开启HLS服务。0关闭,1开启。
				**session: HLS的会话ID, 决定HLS的URL路径。HLS的最终访问路径为 http://&lt;ip address&gt;/&lt;session&gt;/playlist.m3u8
				**segmentTime{INT}: HLS切片时长（近似值，单位毫秒），实际切片时长以该值为参照，但可能会因为编码数据的原因有所偏差
				**maxSegments{INT}: 在一个m3u8的playlist中，最多同时允许存在的切片数。按HLS规范，该值必须 &gt;=3
            ~~
            ~~
        ~~
    ~~

    ++ServiceStatus{ARR}:   每一项代表一个媒体服务的工作状态信息。
        ++{OBJ}:
            **ID: 媒体服务的Unique ID。值可能为
                %%Rtsp_server:
                %%Kmp_pullingServer:
                %%Dynamic_services[0]:
                %%Dynamic_services[1]:
                %%Dynamic_services[2]:
                %%Dynamic_services[3]:
                %%Dynamic_services[4]:
                %%Dynamic_services[5]:
                %%Dynamic_services[6]:
                %%Dynamic_services[7]:
            ~~

            **Type: 服务类型, 值可能为
                %%Rtsp_server: RTSP服务
                %%Kmp_pullingServer: KILOVIEW Media Protocol的拉取服务
                %%Rtmp_push: RTMP推送服务
                %%Rtp_push: RTP推送或组播服务
                %%Kmp_push: KILOVIEW Media Protocol的推送服务
                %%Sproxy_push: 特定OEM客户的推送服务，一般情况下请勿考虑
                %%TS_push: TS/UDP推送或组播服务
                %%HLS: HLS(HTTP Live Streaming)服务
            ~~

            **Enable{INT}: 0|1,返回当前服务的开启状态。1表示开启，0表示关闭。

            **Status: 工作状态，指示该媒体服务的实时状态
                %%error: 表示出错
                %%offline: 表示未开启/不在线
                %%connecting: 表示正在连接
                %%online: 表示在线、已开启
            ~~

            **URL: 该媒体服务的访问URL
        ~~
    ~~
~~
]]

local MAX_DYNAMIC_STREAMS = 8

local serviceArgsMap = {
	["SRT_push"] = {
		enabled = "",
		srt_connect_mode = "",
		address = "",
		port = "",
		srt_latency = "",
		advanced_set = "",
		srt_encryption_key_len = "",
		srt_encryption_key = "",
		srt_bandwidth_overhead = "",
		srt_payload_size = "",
	},
	["Rtmp_push"] = {
		[ "enabled"			   ] = 0,
		[ "pubPoint"		   ] = "",
		[ "username"		   ] = "",
		[ "password"		   ] = "",
		[ "timeout"			   ] = 0,
		[ "reconnect_interval" ] = 0
	},

	["Rtp_push"] = {
		[ "enabled"			  ] = 0,
		[ "target_video_IP"	  ] = "",
		[ "target_video_port" ] = 0,
		[ "target_audio_IP"	  ] = "",
		[ "target_audio_port" ] = 0,
		[ "ttl"				  ] = 127
	},

	["Kmp_push"] = {
		[ "enabled"			   ] = 0,
		[ "pubPoint"		   ] = "",
		[ "username"		   ] = "",
		[ "password"		   ] = "",
		[ "timeout"			   ] = 0,
		[ "reconnect_interval" ] = 0
	},

	["Sproxy_push"] = {
		[ "enabled"			   ] = 0,
		[ "pubPoint"		   ] = "",
		[ "username"		   ] = "",
		[ "password"		   ] = "",
		[ "timeout"			   ] = 0,
		[ "reconnect_interval" ] = 0
	},

	["TS_push"] = {
		[ "enabled" ] = 0,
		[ "targets[1].address" ] = "",
		[ "targets[1].port" ] = "",
		[ "targets[1].ttl" ] = 127,
	},

	["HLS"] = {
		[ "enabled"	    ] = 0,
		[ "session"	    ] = "",
		[ "segmentTime" ] = 0,
		[ "maxSegments" ] = 0
	},

	["JUNSHI"] = {
		[ "enabled"	   ] = 0,
		[ "host"	   ] = "",
		[ "port"	   ] = 0,
		[ "vender_key" ] = "",
		[ "secure_key" ] = ""
	},

	["Omnistream"] = {
		[ "enabled"	   ] = 0,
		[ "address"	   ] = "",
		[ "video_port" ] = 0,
		[ "audio_port" ] = 0,
		[ "error_correction" ] = ""
	},
}

local serviceTypeRevMap = {
	["RTMP/PUSH"] = "Rtmp_push",
	["KMP/PUSH"] = "Kmp_push",
	["SPROXY/PUSH"] = "Sproxy_push",
	["TS-UDP/PUSH"] = "TS_push",
	["HTTP/HLS"] = "HLS",
	["INTERACT/JUNSHI"] = "JUNSHI",
	["SRT/PUSH"] = "SRT_push",
	["SRT-TS/PUSH"] = "SRT_push",
}

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args or not args.Stream or args.Stream == "" then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid arguments"
}
]])
	return
end

local stream = 0
local getConfig = false
local getStatus = true

if args.Stream == "sub" then
	stream = 1
elseif args.Stream == "jpeg" then
	stream = 2
end

if tonumber(args.Config) == 1 then
	getConfig = true
end

if tonumber(args.Status) == 0 then
	getStatus = false
end

cgilua.put([[
{
	"Result":200,
	"Status":"OK",
	"Data":{
		"Stream":"]] .. (stream == 2 and "jpeg" or (stream == 1 and "sub" or "main")) .. [[",
]])

local argPrefix = "streamer.stream[" .. stream .. "]"
local r,cc,gets = rpc.Call( "BUS.Get", { [argPrefix .. ".enabled"] = 0 } )
local streamEn = false
if r and gets then
	streamEn = tonumber( gets[argPrefix .. ".enabled"] ) == 1
end

cgilua.put([[
		"Enable":]] .. (streamEn and 1 or 0) .. [[
]])

if getConfig and streamEn then
	cgilua.put([[
		, "ServiceConfigs":[
	]])

	local gets = {
		[argPrefix .. ".Rtsp_server.port"] = 554,
		[argPrefix .. ".Rtsp_server.session"] = "",
		[argPrefix .. ".Rtsp_server.ssm"] = 0,
		[argPrefix .. ".Rtsp_server.multicast_addr"] = "",
		[argPrefix .. ".Rtsp_server.multicast_video_port"] = 0,
		[argPrefix .. ".Rtsp_server.multicast_audio_port"] = 0,
		[argPrefix .. ".Rtsp_server.multicast_ttl"] = 127,
		[argPrefix .. ".Kmp_pullingServer.enabled"] = 0
	}

	r, cc, gets = rpc.Call( "BUS.Get", gets )
	if not r or not gets then
		cgilua.put([[
			{
				"ID":"Rtsp_server",
				"Type":"Rtsp_server",
				"ERROR":"Fail to get configuration"
			},
			{
				"ID":"Kmp_pullingServer",
				"Type":"Kmp_pullingServer",
				"ERROR":"Fail to get configuration"
			}
		]])
	else
		cgilua.put([[
			{
				"ID":"Rtsp_server",
				"Type":"Rtsp_server",
				"Rtsp_server":{
					"port":]] .. gets[argPrefix .. ".Rtsp_server.port"] .. [[,
					"session":"]] .. gets[argPrefix .. ".Rtsp_server.session"] .. [[",
					"ssm":]] .. gets[argPrefix .. ".Rtsp_server.ssm"] .. [[,
					"multicast_addr":"]] .. gets[argPrefix .. ".Rtsp_server.multicast_addr"] .. [[",
					"multicast_video_port":]] .. gets[argPrefix .. ".Rtsp_server.multicast_video_port"] .. [[,
					"multicast_audio_port":]] .. gets[argPrefix .. ".Rtsp_server.multicast_audio_port"] .. [[,
					"multicast_ttl":]] .. gets[argPrefix .. ".Rtsp_server.multicast_ttl"] .. [[
				}
			},
			{
				"ID":"Kmp_pullingServer",
				"Type":"Kmp_pullingServer",
				"Kmp_pullingServer":{
					"enabled":]] .. gets[argPrefix .. ".Kmp_pullingServer.enabled" ] .. [[
				}
			}
		]])
	end

	if stream < 2 then	--Only main/sub stream get these
		local i
		for i=0, MAX_DYNAMIC_STREAMS-1 do
			local pfx = "streamer.stream[" .. stream .. "].Dynamic_services[" .. i .. "]"
			r, cc, gets = rpc.Call( "BUS.Get", { [pfx .. ".serviceType"] = "" } )
			if r and gets then
				local svcType = gets[ pfx .. ".serviceType" ] or ""
				if svcType ~= "" then
					local argMap = serviceArgsMap[ svcType ]
					if type(argMap) == "table" then
						local k, v
						gets = {}
						for k,v in pairs(argMap) do
							gets[ pfx .. "." .. svcType .. "." .. k ] = v
						end

						r, cc, gets = rpc.Call( "BUS.Get", gets )
						if r and gets then
							cgilua.put( [[,
			{
				"ID":"Dynamic_services[]] .. i .. [[]",
				"Type":"]] .. svcType .. [[",
				"]] .. svcType .. [[": {
]])
							local xx = 0
							for k,v in pairs(argMap) do
								if xx > 0 then
									cgilua.put( [[					,]])
								end
								if svcType == "TS_push" then	--Some special processing
									if k == "enabled" then
										cgilua.put( [[					"]] .. k .. [[":]] .. gets[ pfx .. "." .. svcType .. "." .. k ] .. [[]] )
									elseif type(v) == "string" then
										cgilua.put( [[					"]] .. k:match(".+%.(.*)$") .. [[":"]] .. gets[ pfx .. "." .. svcType .. "." .. k ] .. [["]] )
									else
										cgilua.put( [[					"]] .. k:match(".+%.(.*)$") .. [[":]] .. gets[ pfx .. "." .. svcType .. "." .. k ] .. [[]] )
									end
								elseif type(v) == "string" then
									cgilua.put( [[					"]] .. k .. [[":"]] .. gets[ pfx .. "." .. svcType .. "." .. k ] .. [["]] )
								else
									cgilua.put( [[					"]] .. k .. [[":]] .. gets[ pfx .. "." .. svcType .. "." .. k ] .. [[]] )
								end
								xx = xx + 1
							end
							cgilua.put([[
				} 
			}
]])
						end --Get arguments.
					end --Get valid arguments map
				end --Valid service type
			end -- Get service type
		end --For each dynamic services
	end --Main/sub stream

	cgilua.put( [[
		]
]])
end

--Output service status
if getStatus and streamEn then
	cgilua.put([[
		, "ServiceStatus":[
	]])

	r, gets = rpc.Call( "BUS.GetStreamServiceStatus", string.format("%02d",stream), "Rtsp_server" )
	if r and gets then
		local url = gets["serviceUri"] or ""
		url = url:gsub( "(%%HOST%%)", cgilua.servervariable"SERVER_ADDR" )
		cgilua.put([[
			{
				"ID":"Rtsp_server",
				"Type":"Rtsp_server",
				"Enable":]] .. ((gets["enabled"] == true or gets["enabled"] ==1) and 1 or 0) .. [[,
				"Status":"]] .. gets["stream.status"] .. [[",
				"URL":"]] .. url .. [["
			}
		]])
	else
		cgilua.put([[
			{
				"ID":"Rtsp_server",
				"Type":"Rtsp_server",
				"Enable":0,
				"ERROR":"Fail get service status"
			}
		]])
	end

	r, gets = rpc.Call( "BUS.GetStreamServiceStatus", string.format("%02d",stream), "Kmp_pullingServer" )
	if r and gets then
		local url = gets["serviceUri"] or ""
		url = url:gsub( "(%%HOST%%)", cgilua.servervariable"SERVER_ADDR" )
		cgilua.put([[
			, {
				"ID":"Kmp_pullingServer",
				"Type":"Kmp_pullingServer",
				"Enable":]] .. ((gets["enabled"] == true or gets["enabled"] ==1) and 1 or 0) .. [[,
				"Status":"]] .. gets["stream.status"] .. [[",
				"URL":"]] .. url .. [["
			}
		]])
	else
		cgilua.put([[
			, {
				"ID":"Kmp_pullingServer",
				"Type":"Kmp_pullingServer",
				"Enable":0,
				"ERROR":"Fail get service status"
			}
		]])
	end

	if stream < 2 then	--Only main/sub stream get these
		local i
		for i=0, MAX_DYNAMIC_STREAMS-1 do	--Each dynamic streams
			r, gets = rpc.Call( "BUS.GetStreamServiceStatus", string.format("%02d",stream), "Dynamic_" .. string.format( "%02d", i ) )
			if r and gets and gets["serviceType"] and gets["serviceType"] ~= "NIL" and gets["serviceType"] ~= "nil" then
				local url = gets["serviceUri"] or ""
				url = url:gsub( "(%%HOST%%)", cgilua.servervariable"SERVER_ADDR" )
				cgilua.put([[
			, {
				"ID":"Dynamic_services[]] .. i .. [[]",
				"Type":"]] .. (serviceTypeRevMap[gets["serviceType"]] or "unknown") .. [[",
				"Enable":]] .. ((gets["enabled"] == true or gets["enabled"] ==1) and 1 or 0).. [[,
				"Status":"]] .. gets["stream.status"] .. [[",
				"URL":"]] .. url .. [["
			}
		]])
			end
		end --Each dynamic streams
	end --Main/Sub stream

	cgilua.put( [[
		]
]])
end

	cgilua.put( [[
	}
}
]])


