------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=AV-SOURCE}: 获得视频源信息
##ARGUMENTS: 
    无

##RETURNS: 
	++Data{OBJ}:
       **Sources{ARR}: [ "HDMI|VGA|SDI|CVBS", ... ]
       >> 返回所有设备支持的视频信号源列表
        
       **CurrentSource: HDMI|VGA|SDI|...|UNKNOWN|DETECTING
       >> 当前视频信号输入的源，如果为UNKNOWN，表示没有信号源输入；DETECTING表示正在检测（仅针对具有多个输入源的设备）

       **SignalLock{INT}:
       信号是否锁定（输入信号格式被识别）
           %%0: 未锁定
           %%1: 锁定
       ~~
               
       **Mode: 指示当前信号的视频模式。
       %%3g: 对于SDI信号输入，指示SDI的速率模式为3G-SDI
       %%hd: 对于SDI信号输入，指示SDI的速率模式为HD-SDI
       %%sd: 对于SDI信号输入，指示SDI的速率模式为SD-SDI
       %%hdmi: 对于HDMI/DVI输入，指示输入视频为HDMI模式
       %%dvi: 对于HDMI/DVI输入，指示输入视频为DVI模式
       %%vga: 输入为VGA
       ~~

       列表外的其它暂未定义。

       **Format: 1080p60|1080p59|1080p50|1080p30|1080p29|1080p25|1080p24|
                  720p60|720p59|720p50|720p30|720p29|720p25|720p24|
                  480p|576p|
                  480i|525i|576i|625i|
                  (w)x(h)@(Hz)|
                  UNKNOWN,

       >> 当前视频输入格式ID，如果格式为(w)x(h)@(Hz)（其中w表示宽度，h表示高度，Hz表示刷新率，如800x600@60），说明当前格式为非HDTV格式。
       >> Format = UNKNOWN 表示未知格式，说明没有检测到有效的视频格式。

       **FormatName: 视频格式的友好名称 (参见Format值)
       >> FormatName适用于显示一个友好的格式名称，例如Format为720p60，FormatName友好名称将显示为1280x720p 60Hz

	   **Width{INT}: 视频的宽度
	   **Height{INT}: 视频的高度
	   **Interlaced{INT}: 视频是否为interlaced（隔行）格式，1为True, 0为False
       **FrameRate{NUM}: 代表视频输入信号的实际帧率（如60.0/59.94）
	~~
]]

local cgilua = require "cgilua"
require("kiloview.globalConfig")()
local rpc = require "kiloview.ringRpc"

local vsource = "UNKNOWN"
local vlock = 0
local vformat = "UNKNOWN"
local vformat_name = "UNKNOWN"
local vframe_rate = 0
local vstd = ""
local vwidth = 0
local vheight = 0
local vint = 0
local r, info = rpc.Call( "BUS.GetSignalStatus" )

if r and info then
	vsource = info.video["video.source"] or "UNKNOWN"
	vlock = info.video["video.locked"] or 0
	vformat = info.video["video.format_code"] or "UNKNOWN"
	vformat_name = info.video["video.format_name"] or "UNKNOWN"
	vframe_rate = info.video["video.frame_rate"] or 0
	vstd = info.video["video.standard"] or ""
	vwidth = info.video["video.width"] or 0
	vheight = info.video["video.height"] or 0
	vint = info.video["video.fix_interlaced"] or info.video["video.interlaced"] or 0
end

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

cgilua.put([[
{
	"Result":200,
	"Status":"OK",
	"Data":{
		"Sources":[
]])

local cnt = 0
if _G.FEATURE_VIDEO_SDI then
	if cnt > 0 then
		cgilua.put( "," )
	end
	cgilua.put( "\"SDI\"" )
	cnt = cnt + 1
end

if _G.FEATURE_VIDEO_HDMI then
	if cnt > 0 then
		cgilua.put( "," )
	end
	cgilua.put( "\"HDMI\"" )
	cnt = cnt + 1
end

if _G.FEATURE_VIDEO_VGA then
	if cnt > 0 then
		cgilua.put( "," )
	end
	cgilua.put( "\"VGA\"" )
	cnt = cnt + 1
end

cgilua.put("],\n" )

cgilua.put([[
		"CurrentSource":"]] .. vsource .. [[",
		"SignalLock":]] .. vlock .. [[,
		"Mode":"]] .. vstd:lower() .. [[",
		"Format":"]] .. vformat .. [[",
		"FormatName":"]] .. vformat_name .. [[",
		"Width":]] .. vwidth .. [[,
		"Height":]] .. vheight .. [[,
		"Interlaced":]] .. vint .. [[,
		"FrameRate":]] .. vframe_rate .. [[
	}
}
]] )

