------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=ONVIF}: 删除ONVIF用户
##ARGUMENTS{GET,POST}: 
    @@Username{STR}: 要删除的用户名
    	
##RETURNS: 
	++Data{OBJ}:
		**Reason{STR}: 如果返回错误码为405，Reason说明删除用户失败的原因：
			%%reserved: 内建的默认用户(admin)不可以删除
			%%invalid: 无效的用户名
			%%notexist: 指定的用户不存在
		~~
	~~

##NOTES:
>>{BOLD} 安全性说明：
>> 调用该API必须通过HTTP的身份认证。身份认证用户名和密码同Web登录。

##__OTHERS:
>> 在本页面的API Call测试中，给定的用户名为admin，密码为admin（指定HTTP Request header中Authorization:"Basic YWRtaW46YWRtaW4="）。如果需要修改，请在下面指定：
<script type="text/javascript">
ajaxAuth = "Basic YWRtaW46YWRtaW4=";
function change_auth(val)
{
	ajaxAuth = val;
	alert('DONE');
}
</script>
>> 修改Authorization值: <input type="text" id="test_auth" size="28" value="Basic YWRtaW46YWRtaW4=" /> <input type="button" value="修改" onclick='change_auth($("#test_auth")[0].value);' />
<br /><br />
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args or not args.Username or args.Username == "" then
	cgilua.put([[
	{
		"Result":400,
		"Status":"ERROR: Invalid arguments"
	}]] )
	return
end

local MAX_USERS=10

local function removeOnvifUser( user )
	if user == "admin" then
		return false, "reserved"
	elseif type(user) ~="string" then
		return false, "invalid"
	elseif user == "" then
		return false, "invalid"
	else
		local gets = {}
		local i
		
		for i=0, MAX_USERS-1 do
			gets[ "onvif.users[" .. i .. "].username" ] = ""
		end

		local r,cc
		r,cc,gets = rpc.Call( "BUS.Get", gets )
		if not r or not gets then
			return false, "system"
		end

		local validSpace = -1
		for i=0, MAX_USERS-1 do
			if gets[ "onvif.users[" .. i .. "].username" ] == user then
				validSpace = i
				break
			end
		end
		
		if validSpace >= 0 then
			local sets = {["onvif.users[" .. validSpace .. "].username"] = "",
						  ["onvif.users[" .. validSpace .. "].password"] = "",
						  ["onvif.users[" .. validSpace .. "].name"] = "",
						  ["onvif.users[" .. validSpace .. "].role" ] = "user" }
			rpc.Call( "BUS.Set", sets)
		else
			return false, "notexist"
		end

		return true
	end
end

local code, reason = removeOnvifUser( args.Username )
if code then
	cgilua.put([[
	{
		"Result":200,
		"Status":"OK"
	}]] )
else
	cgilua.put([[
	{
		"Result":405,
		"Status":"ERROR: Fail to remove user",
		"Data":{
			"Reason":"]] .. (reason or "unknown") .. [["
		}
	}]] )
end

