------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=IOLED}: 设置由用户自行控制LED选项开关
>> 编码器默认由内部控制显示LED的状态，但是也允许用户自行控制LED的显示。
>> 如果需要用户控制LED显示，则需要调用本API打开用户控制LED选项。当打开用户控制的LED选项后，
编码器的LED将不再被内部控制。关闭用户控制LED选项后，会恢复默认行为。

>> 当打开LED用户控制后，可以调用 ^^REF{ledControl.lua}来控制LED的显示状态。

##ARGUMENTS{GET,POST}: 
	@@Enable{INT}: 1表示打开用户控制LED选项，0表示关闭用户LED控制。

##RETURNS{DEFAULT}: 
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args or not args.Enable or args.Enable == "" then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid request arguments"
}
]])
	return
end

local en = tonumber(args.Enable)

local r,x = rpc.Call( "BUS.SetLEDUserControl", en )

cgilua.put([[
{
	"Result":]] .. (r and "200" or "400") .. [[,
	"Status":"]] .. (x and "OK" or "ERROR: Fail to set LED user control") .. [["
}
]])

