------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=OSD}: 设置主码流/子码流的字符叠加选项

##ARGUMENTS{GET,POST}: 
    @@Stream{*}: 指定码流
        %%main: 主码流
        %%sub: 子码流
    ~~
    @@ID{INT,*}: 要设定OSD叠加项目的ID（请参见 ^^REF{getOSDItem.lua} 的返回值）

    @@Type:
    叠加内容类型： 
    %%text: 叠加指定的文字
    %%time: 显示系统时间
    %%none: 关闭叠加
    ~~
    @@Position:
    叠加位置
        %%left-top: 左上角
        %%top-center: 顶部居中
        %%right-top: 右上角
        %%left-bottom: 左下角
        %%bottom-center: 底部居中
        %%right-bottom: 右下角
        %%special: 手动指定位置
    ~~

    @@X{INT}:
    像素为单位，手动指定的水平叠加位置（仅当Position参数为special时有效）
    
    @@Y{INT}:
    像素为单位，手动指定的垂直叠加位置（仅当Position参数为special时有效）

    @@Font: 字体名称。
    当前支持的字体有
        %%default: 默认，英文
        %%song: 中文，宋体
        %%vera: 英文
        %%lucida: 英文
    ~~
    
    @@Size{INT}:
    值为0，表示小号字体；1表示中等字体；2表示大号字体；大于2表示指定的文字大小（px）
    
    @@Outline{INT}:
    指定在文字的外围加上边框的粗细；0表示不加边框，数值越大，边框越粗。
    
    @@Bold: 0|1。字体加粗，0表示不加粗，1表示加粗
    
    @@Italic: 0|1。斜体，0表示正常，1表示斜体（注：当前尚不支持斜体功能）
    
    @@Text: 叠加文字(UTF-8编码，仅当 Type 为 text 时有效）

##RETURNS{DEFAULT}: 
]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )

local args = require("M_Request").VerifyAndGetArgs()
if not args or not args.Stream or args.Stream == "" or tonumber(args.ID) == nil then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid arguments"
}
]])
	return
end

local stream = 0
if args.Stream == "sub" then
	stream = 1
end

local itemId = tonumber(args.ID)

local argsMap = {
	["Type"] = "",
	["Position"] = "",
	["X"] = 0,
	["Y"] = 0,
	["Font"] = "",
	["Size"] = 0,
	["Outline"] = 0,
	["Bold"] = 0,
	["Italic"] = 0,
	["Text"] = ""
}

local sets = {}
local k,v
for k,v in pairs(args) do
	if type(argsMap[k]) == "number" then
		sets[ "osd.stream[" .. stream .. "].items[" .. itemId .. "]." .. k:lower() ] = tonumber(v)
	elseif type(argsMap[k]) == "string" then
		if k == "Text" or v ~= "" then	--Only allow "Text" be empty
			if k == "Type" then
				sets[ "osd.stream[" .. stream .. "].items[" .. itemId .. "].osd_type" ] = v
			else
				sets[ "osd.stream[" .. stream .. "].items[" .. itemId .. "]." .. k:lower() ] = v
			end
		end
	end
end

local r,x = rpc.Call( "BUS.Set", sets )
if r and x then
	cgilua.put([[
{
	"Result":200,
	"Status":"OK"
}
]])

else
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Fail to set arguments"
}
]])

end

