------------------------------------------------------
-- KILOVIEW Device HTTP Api, v1.0
-- Copyright (c) 2016, KILOVIEW Electronics.
------------------------------------------------------

--[[

##ABSTRACT{G=TIME_LOCATION}: 设置设备时间

##ARGUMENTS{GET,POST}: 
    @@LocalTime: "yyyy-mm-dd hh:nn:ss", 可选。
        >> 设置设备的本地时间（注意与UTCTime之间，二选其一）
    @@UTCTime: "yyyy-mm-dd hh:nn:ss", 可选。
        >> 设置设备上的UTC时间（注意与LocalTime之间，二选其一）
    @@NTPEnable{INT}: 0|1, 可选。NTP同步使能，0为关闭，1为开启
    @@NTPServers: NTP服务器地址列表，多项之间用空格隔开。可选。

##RETURNS{DEFAULT}: 

]]

local cgilua = require "cgilua"
local rpc = require "kiloview.ringRpc"

local SAPI = SAPI or cgilua.SAPI
SAPI.Response.contenttype( "application/json; charset=utf-8" )
local args = require("M_Request").VerifyAndGetArgs()
if not args then
	cgilua.put([[
{
	"Result":400,
	"Status":"ERROR: Invalid arguments"
}
]])
	return
end

local _x, timedateinfo = rpc.Call( "SYS_STATUS.GetTimeDateInfo" )
local ntpEn = timedateinfo and timedateinfo["ntp_enabled"] or "no"

local setNtp = false
if tonumber(args.NTPEnable) then
	setNtp = true

	if tonumber(args.NTPEnable) == 1 then
		ntpEn = "yes"
	else
		ntpEn = "no"
	end
end

if args.NTPServers and args.NTPServers ~= "" then
	rpc.Call( "SYS_STATUS.SetNTP", ntpEn, args.NTPServers )
elseif setNtp then
	rpc.Call( "SYS_STATUS.SetNTP", ntpEn )
end

if ntpEn ~= "yes" then
	if args.LocalTime and args.LocalTime ~= "" then
		rpc.Call( "SYS_STATUS.SetLocalTime", args.LocalTime )
	elseif args.UTCTime and args.UTCTime ~= "" then
		rpc.Call( "SYS_STATUS.SetUTCTime", args.UTCTime )
	end
end


cgilua.put([[
{
	"Result":200,
	"Status":"OK"
}
]])
