local _M = {}

local metatable = {}

function metatable:__index(key)
	if key == "QUERY" then
		if ngx.ctx.QUERY then
			return ngx.ctx.QUERY
		else
			ngx.ctx.QUERY = ngx.req.get_uri_args()
			return ngx.ctx.QUERY
		end
	elseif key == "POST" then
		if ngx.ctx.POST then
			return ngx.ctx.POST
		else
			ngx.req.read_body()
			ngx.ctx.POST = ngx.req.get_post_args()
			return ngx.ctx.POST
		end
	else
		return nil
	end
end

function _M.put(...)
	ngx.print(...)
end

function _M.print(...)
	ngx.print(...)
end

function _M.redirect(location)
	ngx.redirect(location)
end

function _M.contentheader( prim, sub )
	if prim then
		if sub then
			ngx.header.content_type = prim .. "/" .. sub
		else
			ngx.header.content_type = prim
		end
	end
end

function _M.servervariable(name)
	if name == "SCRIPT_NAME" or name == "script_name" then
		return ngx.var.uri
	elseif ngx.var[name:lower()] then
		return ngx.var[name:lower()]
	else
		return ngx.var[name]
	end
end

function _M.checkKlnkProxy()
	return ngx.req.get_headers().klnkproxy
end

setmetatable(_M, metatable)

--------------------------------

local Request = {}
local Response = {}


--------------------------------
function Request.getUserAndPwd()
	local http_authorization = ngx.var.http_authorization
	if not http_authorization then
		if ngx.var.host == "10.10.10.10" then
			return "root", "unknown"
		elseif ngx.req.get_headers().klnkproxy then
			return "admin", "unknow"
		else
			return nil
		end
	else
		local scheme, credential = http_authorization:match("(%S+)%s(%S+)")
		if scheme and credential then
			local base64 = require "base64"
			return base64.decode(credential):match("(.*):(.*)")
		else
			return nil
		end
	end
end

function Request.savePostFile( loc, keepExt )
	local args = {}
	local resty_upload = require "kiloview.resty_upload"
	local force_ext
	if not keepExt then
		force_ext = ".tar.gz"
	end
	local file,err = resty_upload.GetUploadFile(args, loc, force_ext)
	if file then
		ngx.ctx.POST = args
		return true, (loc or "/tmp") .. "/" .. file
	else
		return false, err
	end
end

--------------------------------
function Response.contenttype( ctype )
	ngx.header.content_type = ctype
end

function Response.header( key, val )
	ngx.header[key] = val
end

function Response.setContentType( ctype )
	ngx.header.content_type = ctype
end
--------------------------------

_M.SAPI = {
	Request = Request,
	Response = Response
}

---------------------------------

return _M

