local cgilua = require "cgilua"

local contentTypeToMimeMap = {
	["html"] = "text/html",
	["json"] = "application/json",
	["script"] = "text/javascript",
	["text"] = "text/plain",
	["xml"] = "application/xml"
}

return function()
--{

local _M = {
	exports = {}
}

function _M.ProcessingCall()
	if cgilua.POST and cgilua.POST._KAJAX_INVOKE then	--Found calling tag.
		local callFuncName = cgilua.POST._KAJAX_INVOKE
		if _M.exports[callFuncName] then
			if _M.exports[callFuncName].resultType and contentTypeToMimeMap[_M.exports[callFuncName].resultType] then
				SAPI.Response.contenttype( contentTypeToMimeMap[_M.exports[callFuncName].resultType] )
			else
				SAPI.Response.contenttype( "text/html" )
			end

			local call = _M.exports[callFuncName].call
			if type(call) == "function" then
				if cgilua.POST._KAJAX_ARGS then
					if cgilua.POST._KAJAX_ARGS == "~ajax:args" then
						local passArgs = {}
						local n, v
						for n=0,100,1 do	--HEHE, 100 arguments? no possible.
							v = cgilua.POST["_KAJAX_ARG_"..n]
							if not v then
								break
							else
								table.insert( passArgs, v )
							end
						end
						cgilua.put( call( table.unpack(passArgs) ) )
					elseif cgilua.POST._KAJAX_ARGS == "~ajax:form" then --Submit a form
						cgilua.put( call( cgilua.POST ) )	--Using post table as arguments.
					end
				else
					cgilua.put( call() )
				end
			end
		end
		return true			
	else
		return false
	end
end

--handler: "jquery"
--	Now only support JQUERY.
--location: "scripts location", or nil means no '<script...>' in generated outputs.
function _M.GenerateScriptHeaders(lang,handler,location) 
	return ""
end

function _M.GenerateScripts(lang,handler,nocall_func)
	local k,v
	local result = ""

	if not nocall_func then

		result = result .. [[
			function kajax_do_call( callUrl, funcname, rtype, args)
			{
				if (callUrl.indexOf("/")==0)
				{
					callUrl = "."+callUrl;
				}
				var data = {};
				var postArgs;
				var postForm = false;
				if(args.length > 1 && args[0] == "~ajax:form" )	//Special post form arguments
				{
					postArgs = "~ajax:form";
					postForm = true;
					var argTable = args[1];
					for( var n=0; n<argTable.length; n++ )
					{
						data[argTable[n].name] = argTable[n].value;
					}
				}
				else
				{
					postArgs = "~ajax:args";
					for( var n = 0; n<args.length-1; n++ )	//Last argument is callback function.
					{
						data["_KAJAX_ARG_" + n.toString()] = args[n];
					}
				}

				var callfunc = args[args.length-1];

				data["_KAJAX_INVOKE"] = funcname;
				data["_KAJAX_ARGS"] = postArgs;
				data["language"] = "]] .. (lang or "en") .. [[";
				$.ajax({ 
					url		:	callUrl,
					async	:	true,
					dataType:	rtype,   
					timeout	:	60000,
					cache	:	false, 
					data	:	data,
					type	:	'POST',

					success : 	function( ret ) {
						if( callfunc instanceof Function ) {
							callfunc(ret);
						}
					},	//End of success callback function
				    
					error : function( XMLHttpRequest, textStatus, errorThrown ) {
					}, //End of error callback function

					beforeSend : function(){
					}, //End of beforeSend callback function

					complete : function(){
					} //End of complete callback function
				});  // $.ajax(
			}

			function ajaxSubmit(func,callback,subForm) {
				var formData = subForm.serializeArray();
				func( "~ajax:form", formData, callback );
			}
		]]
	end

	for k,v in pairs(_M.exports) do
		local func = string.format ([[
                function x_%s() {
                    kajax_do_call ( "]] .. cgilua.servervariable"SCRIPT_NAME" .. [[", "%s", "%s", x_%s.arguments);
                }
                ]], v["funcName"], v["funcName"], v["resultType"], v["funcName"] )

		result = result .. func
	end	

	return result
end

--contentType:
--	json
--	html
--	script
--	xml
--	text
function _M.ExportFunction( name, func, contentType )
	_M.exports[name] = {
		funcName = name,
		resultType = contentType,
		call = func
	}
end

function _M.ClearExports()
	_M.exports = {}
end

return _M

--}
end

