--local getset = require "kiloview.getSetHelper"
local _dbg = require "coaf.debug"
local client = require "kiloview.coafClientInit"

-- local function myprint(...)
-- 	_dbg.Error("BUSOPR",...)
-- end

-- print = myprint

local _M = {}

local function GetCoAFProxy( objPath, service, loc, timeout )
	local proxy
	local cli
	cli = client.New( service, loc or "local", nil, timeout or 3 ) --3 seconds timeout.
	if not cli then
		_dbg.Error( "WEB", "Fail to create connection to service: " .. (loc or "local") .. "/" .. (service or "nil") )
		return nil
	end
	proxy = cli:CreateProxy(objPath, true) --Exclusive
	if not proxy then
		_dbg.Error( "WEB", "Fail to create object proxy for '" .. (objPath or "unknown") .. "'" )
		return nil
	end
	return proxy
end

function _M.getBlockLoopOutConfig()
	local proxy = GetCoAFProxy("/BLO", "codec")
	if proxy then
		local done, ret = proxy.getConfig()
		proxy:Destroy()
		return ret
	else
		return false
	end
end

function _M.setBlockLoopOutConfig(args)
	local proxy = GetCoAFProxy("/BLO", "codec")
	if proxy then
		local done, ret = proxy.setConfig(args)
		done, ret = proxy.bindMonitoring()
		proxy:Destroy()
		return ret
	else
		return false
	end
end

function _M.RecordFTPUpload(args)
	local proxy = GetCoAFProxy( "/", "ftp_service", ngx.var.coaf_location, 7200 )
	if proxy then
		local done, r = proxy.manualUpload(args)
		proxy:Destroy()
		return r
	else
		return false
	end
end

function _M.getUploadStatus()
	local proxy = GetCoAFProxy( "/", "ftp_service", ngx.var.coaf_location, 5 )
	if proxy then
		local done, r = proxy.getUploadStatus()
		proxy:Destroy()
		return r
	else
		return false
	end
end


function _M.Reset()
	local proxy = GetCoAFProxy( "/SYS_STATUS", "web_daemon", ngx.var.coaf_location, 5 )
	if proxy then
		proxy.Reset()
		proxy:Destroy()
		return true
	else
		return false
	end
end

function _M.Reboot()
	local proxy = GetCoAFProxy( "/SYS_STATUS", "web_daemon", ngx.var.coaf_location )
	if proxy then
		proxy.Reboot()
		proxy:Destroy()
		return true
	else
		return false
	end
end
function _M.SetRebootTimerInfo( enable,date,hour,minute,policy,week,day,month,days )
	local proxy = GetCoAFProxy( "/timer", "web_daemon", ngx.var.coaf_location )
	if proxy then
		local r,done = proxy.SetRebootTimerInfo( enable,date,hour,minute,policy,week,day,month,days )
		proxy:Destroy()
		if r and done then
			return true
		else
			return false
		end
	else
		return false
	end
end

function _M.GetRebootTimerInfo()
	local proxy = GetCoAFProxy( "/timer", "web_daemon", ngx.var.coaf_location )
	if proxy then
		local r, data,err = proxy.GetRebootTimerInfo()
		proxy:Destroy()
		return data
	else
		return false
	end
end


function _M.AddNAS( id, name, type, host, volumn, options )
	local proxy = GetCoAFProxy( "/", "nas_manager", ngx.var.coaf_location, 15 )
	if not proxy then
		return false, "Fail to get NAS manager proxy"
	end
	local done, r, msg = proxy.Add( id, name, type, host, volumn, options )
	proxy:Destroy()
	if not done then
		return false, "System error:" .. (r or "unknown")
	else
		return r, msg
	end
end

function _M.RemoveNAS( id )
	local proxy = GetCoAFProxy( "/", "nas_manager", ngx.var.coaf_location, 15 )
	if not proxy then
		return false, "Fail to get NAS manager proxy"
	end
	local done, r, msg = proxy.Remove(id)
	proxy:Destroy()
	if not done then
		return false, "System error:" .. (r or "unknown")
	else
		return r, msg
	end
end

function _M.GetNASList()
	local proxy = GetCoAFProxy( "/", "nas_manager", ngx.var.coaf_location, 15 )
	if not proxy then
		return false, "Fail to get NAS manager proxy"
	end
	local done, r = proxy.GetList()
	proxy:Destroy()
	if not done then
		return false, "System error:" .. (r or "unknown")
	else
		return r
	end
end

function _M.AddIMAGE( id, file )
	local proxy = GetCoAFProxy( "/", "img_manager", ngx.var.coaf_location, 100 )
	if not proxy then
		return false, "Fail to get IMAGE manager proxy"
	end
	local done, r, msg = proxy.Add( id, file )
	proxy:Destroy()
	if not done then
		return false, "System error:" .. (r or "unknown")
	else
		return r, msg
	end
end

function _M.RemoveIMAGE( id )
	local proxy = GetCoAFProxy( "/", "img_manager", ngx.var.coaf_location, 30 )
	if not proxy then
		return false, "Fail to get IMAGE manager proxy"
	end
	local done, r, msg = proxy.Remove(id)
	proxy:Destroy()
	if not done then
		return false, "System error:" .. (r or "unknown")
	else
		return r, msg
	end
end

function _M.GetIMAGEList()
	local proxy = GetCoAFProxy( "/", "img_manager", ngx.var.coaf_location, 10 )
	if not proxy then
		return false, "Fail to get IMAGE manager proxy"
	end
	local done, r, list = proxy.GetList()
	proxy:Destroy()
	if not done or not r then
		return false, "System error:" .. (r or list or "unknown")
	else
		return list
	end
end

function _M.GetIMAGEDesc(id)
	local proxy = GetCoAFProxy( "/", "img_manager", ngx.var.coaf_location, 10 )
	if not proxy then
		return false, "Fail to get IMAGE manager proxy"
	end
	local done, r = proxy.GetImageDesc(id)
	proxy:Destroy()
	if not done or not r then
		return false, "System error:" .. (r or "Invalid")
	else
		return r
	end
end

function _M.ResizeIMAGE(id, w, h, a, tp_x, tp_y, tp_mode, tp_fuzz)
	local proxy = GetCoAFProxy( "/", "img_manager", ngx.var.coaf_location, 60 )
	if not proxy then
		return false, "Fail to get IMAGE manager proxy"
	end
	local done, r, msg = proxy.ResizeImage(id,w,h,a,tp_x, tp_y, tp_mode, tp_fuzz)
	proxy:Destroy()
	if not done then
		return false, "System error:" .. (r or "unknown")
	else
		local osdProxy = GetCoAFProxy( "/OSD", "codec", ngx.var.coaf_location, 10 )
		if osdProxy then
			osdProxy:Control("update",{})
			osdProxy:Destroy()
		end
		return r, msg
	end
end

function _M.SetIMAGEAlpha(id,alpha)
	local proxy = GetCoAFProxy( "/", "img_manager", ngx.var.coaf_location, 30 )
	if not proxy then
		return false, "Fail to get IMAGE manager proxy"
	end
	local done, r, msg = proxy.SetAlpha(id,alpha)
	proxy:Destroy()
	if not done then
		return false, "System error:" .. (r or "unknown")
	else
		local osdProxy = GetCoAFProxy( "/OSD", "codec", ngx.var.coaf_location, 10 )
		if osdProxy then
			osdProxy:Control("update",{})
			osdProxy:Destroy()
		end
		return r, msg
	end
end

-----------------------------------------
--- Get signal status
-----------------------------------------

function _M.GetSignalStatus()
	local vfeProxy = GetCoAFProxy( "/AVIO/00/VFE", "codec", ngx.var.coaf_location )
	if not vfeProxy then
		print( "*** ERROR: Could not get VFE proxy when GetSignalStatus()" )
		return {}
	end

	local result = {}
	local r, x, rtable = vfeProxy:Control( "detect-source", {source="any"} )
	vfeProxy:Destroy()
	if r and x and x >= 0 then
		result.video = rtable
		if rtable["video.locked"] == 1 and rtable["video.freerun"] and rtable["video.freerun"] == 1 then	--Processing the special case: freerun.
			rtable["video.locked"] = 2
		end
	else
		print( "***vfe:detect-source WRONG", x, rtable )
	end

	local afeProxy = GetCoAFProxy( "/AVIO/00/AFE", "codec", ngx.var.coaf_location )
	if not afeProxy then
		print( "*** ERROR: Could not get AFE proxy when GetSignalStatus()" )
		return result
	end

	r, x, rtable = afeProxy:Control( "detect-source", {source="any"} )
	afeProxy:Destroy()
	if r and x and x >= 0 then
		result.audio = rtable
	else
		print( "***afe:detect-source WRONG", x, rtable )
	end

	return result
end

-----------------------------------------
--- Get Stream Status
-----------------------------------------

function _M.VGASpecialCtrl(flag, special, params)
	if not flag or not special or type(params) ~= "table" then
		return nil
	end
	
	local vfeProxy = GetCoAFProxy( "/AVIO/00/VFE", "codec", ngx.var.coaf_location )
	if not vfeProxy then
		print( "*** ERROR: Could not get VFE proxy when GetSignalStatus()" )
		return {}
	end

	params["special"] = special
	params["source"] = "VGA"
	
	if flag == "set" or flag == "SET" then
		local r,x,rtable = vfeProxy:Control( "config-source", params )
		vfeProxy:Destroy()
		--print( r,x,rtable)
		if not r or not x then
			return nil
		else
			return rtable
		end
	else
		local r,x,rtable = vfeProxy:Control( "detect-source", params )
		vfeProxy:Destroy()
		if not r or not x then
			return nil
		else
			return rtable
		end
	end
end

-----------------------------------------
--- Update encoding stream
-----------------------------------------
function _M.UpdateEncoding(idr, target)
	local vidEncProxy = GetCoAFProxy( "/Codec/Encoders/" .. idr, "codec", ngx.var.coaf_location )
	if vidEncProxy then
		local r, x = vidEncProxy:UpdateEncoding(target or "all")
		vidEncProxy:Destroy()
		return x
	else
		return false
	end
end

-----------------------------------------
--- LED User controls
-----------------------------------------

function _M.SetLEDUserControl(en)
	local ioledProxy = GetCoAFProxy( "/", "ioled", ngx.var.coaf_location )
	if not ioledProxy then
		return false
	end

	local ctrl = "disable_uctrl"

	if en and en ~= 0 then
		ctrl = "enable_uctrl"
	end

	local r,x = ioledProxy:Control( ctrl, {} )
	ioledProxy:Destroy()
	if r and x then
		return true
	else
		return false
	end
end

function _M.LEDControl(id,status)
	local ioledProxy = GetCoAFProxy( "/", "ioled", ngx.var.coaf_location )
	if not ioledProxy then
		return false
	end

	local r,x = ioledProxy:Control( "led_uctrl", { id=id or "", status = status or "OFF" })
	ioledProxy:Destroy()
	if r and x then
		return true
	else
		return false
	end
end

function _M.PTZControl(action,args)
	local ptzProxy = GetCoAFProxy( "/PTZ", "onvif", ngx.var.coaf_location )
	if not ptzProxy then
		return false
	end

	local r,x = ptzProxy:Control( action, args)
	ptzProxy:Destroy()
	if r and x then
		return true
	else
		return false
	end
end

function _M.IRControl(action,args)
	local irProxy = GetCoAFProxy( "/", "ir", ngx.var.coaf_location )
	if not irProxy then
		return false
	end

	local r,x,rtable = irProxy:Control( action, args)
	irProxy:Destroy()
	if r and x then
		return x,rtable
	else
		return false
	end
end
-----------------------------------------
--- Get Record Status
-----------------------------------------

function _M.GetAllDisks()
	local diskProxy = GetCoAFProxy( "/", "record", ngx.var.coaf_location )
	if not diskProxy then
		return nil, nil
	end
	local r,st,info = diskProxy:DiskList()
	if not r or not st or type(info) ~= "table" then
		diskProxy:Destroy()
		return nil, nil
	end

	local rslt = {}
	local cc = 0
	local k,v
	for k,v in pairs(info) do
		local detail
		r,st,detail = diskProxy:DiskDetail(k)
		if r and st and type(detail) == "table" then
			rslt[k] = detail
			cc = cc + 1
		end
	end
	diskProxy:Destroy()
	return cc, rslt
end

function _M.DiskRecordList(dev,filter)
	local diskProxy = GetCoAFProxy( "/", "record", ngx.var.coaf_location )
	if not diskProxy or not dev or dev == "" then
		return nil, nil
	end
	local r,st,info = diskProxy:DiskRecordList(dev, filter or "")
	diskProxy:Destroy()
	if not r or not st or type(info) ~= "table" then
		return nil, nil
	end

	local rslt = {}
	local k,v
	for k,v in pairs(info) do
		if v.FILES then
			local files = v.FILES .. "\n"
			v.FILES = {}
			local f,s
			for f,s in files:gmatch("(.-)=(.-)\n") do
				table.insert( v.FILES, { FILENAME=f, SIZE=s } )
			end
			table.sort( v.FILES, function(a,b)
				return a.FILENAME < b.FILENAME
			end)
			table.insert( rslt, v)
		end
	end

	table.sort( rslt, function(a,b)
		return (a.START_TIME or 0) > (b.START_TIME or 0)
	end)
	return #rslt, rslt
end

function _M.GetRecordStatus( idr )
	local idrNum = tonumber(idr) or 0
	local recordProxy = GetCoAFProxy( "/", "record", ngx.var.coaf_location )
	if not recordProxy then
		return nil, nil
	end
	local r,st,info = recordProxy:GetStatus( idrNum == 1 and "sub" or "main" )
	recordProxy:Destroy()
	if not r or type(info) ~= "table" then
		return nil, nil
	else
		if info.FILES then
			local files = info.FILES .. "\n"
			info.FILES = {}
			local f,s
			for f,s in files:gmatch("(.-)=(.-)\n") do
				table.insert( info.FILES, { FILENAME=f, SIZE=s } )
			end
			table.sort( info.FILES, function(a,b)
				return a.FILENAME < b.FILENAME
			end)
		end
		return st, info
	end
end

function _M.RemoveRecordFile( file )
	local proxy = GetCoAFProxy( "/SYS_STATUS", "web_daemon", ngx.var.coaf_location, 5 )
	if proxy then
		proxy.RemoveFile(file)
		proxy:Destroy()
		return true
	else
		return false
	end
end

function _M.RenameRecordFile( from, to )
	local proxy = GetCoAFProxy( "/SYS_STATUS", "web_daemon", ngx.var.coaf_location, 5 )
	if proxy then
		proxy.RenameFile(from,to)
		proxy:Destroy()
		return true
	else
		return false
	end
end

-----------------------------------------
--- Get Stream Status
-----------------------------------------

function _M.GetStreamStatus( idr,coaf_local )
	local srcProxy = GetCoAFProxy( "/DefaultApp/" .. idr .. "/Source", "streamer", coaf_local or ngx.var.coaf_location )
	local vidEncProxy = GetCoAFProxy( "/Codec/Encoders/" .. idr, "codec", coaf_local or ngx.var.coaf_location )

	local result = {}

	local idx = tonumber(idr)
	-- TODO: Audio
	local params = {
		["encoder.stream[" .. idx .. "].enabled"] = 0,
		["encoder.stream[" .. idx .. "].video.profile"] = "baseline",
		["encoder.stream[" .. idx .. "].video.scale"] = "default",
		["encoder.stream[" .. idx .. "].video.codec"] = "NONE",
		["streamer.stream[" .. idx .. "].audioChannel"] = -1
	}
	
	local gotAudio = false
	local r
	r,params = _M.Get(params, coaf_local)
	
	if r and r >=0 and params then
		result["stream.enabled"] = params["encoder.stream[" .. idx .. "].enabled"]
		result["video.codec"] = params["encoder.stream[" .. idx .. "].video.codec"]
		result["video.scale"] = params["encoder.stream[" .. idx .. "].video.scale"]
		result["video.profile"] = params["encoder.stream[" .. idx .. "].video.profile"]
		local aChannel = params[ "streamer.stream[" .. idx .. "].audioChannel" ] or -1
		result["audio.channel"] = aChannel
		if aChannel >= 0 then
			params = {
				["audioEngine.encoders[" .. aChannel .. "].encoder.sampling"] = 48000,
				["audioEngine.encoders[" .. aChannel .. "].encoder.channels"] = 2,
				["audioEngine.encoders[" .. aChannel .. "].encoder.codec"] = "AAC",
				["audioEngine.encoders[" .. aChannel .. "].encoder.bitrate"] = 64000
			}
			r,params = _M.Get(params, coaf_local)
			if r and r >=0 and params then
				gotAudio = true
				result["audio.codec"] = params["audioEngine.encoders[" .. aChannel .. "].encoder.codec"] or "NONE"
				result["audio.sampling"] = params["audioEngine.encoders[" .. aChannel .. "].encoder.sampling"]  or 0
				result["audio.channels"] = params["audioEngine.encoders[" .. aChannel .. "].encoder.channels"] or 0
				result["audio.bitrate"] = params["audioEngine.encoders[" .. aChannel .. "].encoder.bitrate"] or 0
			end
		end
		
		if not gotAudio then
			--TODO:
			result["audio.codec"] = "NONE"
			result["audio.sampling"] = 0
			result["audio.channels"] = 0
			result["audio.bitrate"] = 0
		end
		
	else
		print( "*** Fail to get encoder settings of '" .. idr .. "'" )
	end

	if vidEncProxy then
		local r,x,params = vidEncProxy:GetVideoProperties()
		vidEncProxy:Destroy()
		if r and x and x >= 0 and params then
			result["video.width"] = params["video.width"]
			result["video.height"] = params["video.height"]
		else
			print( "*** Could not get video encoding informations of '" .. idr .. "'" )
		end
	else
		print( "*** Could not get encoder control proxy of '" .. idr .. "'" )
	end

	if srcProxy then
		local r,x,info = srcProxy:GetStatus()
		srcProxy:Destroy()
		if r and x and x >=0 and info then
			result["video.bitrate"] = info["video.bitrate"]
			result["video.frameRate"] = info["video.frameRate"]
		else
			print( "*** Stream source of '" .. idr .. "': Fail to get status" )
		end
	else
		print( "*** Could not get source stream proxy of '" .. idr .. "'" )
	end
	return result
end


-----------------------------------------
--- Stream snapshot
-----------------------------------------
function _M.SnapShot( idr, path, file, ext )
	if not idr then
		print( "*** Invalid IDR when snap stream" )
		return false
	end

	local moduleProxy = GetCoAFProxy( "/DefaultApp/" .. idr, "streamer", ngx.var.coaf_location )
	if not moduleProxy then
		print( "*** Could not get streaming service module proxy of stream '" .. idr .. "'" )
		return false
	end

	local r,cc,msg = moduleProxy:Control( "snap", {
		["path"] = path or "/tmp",
		["file"] = file or "WEB-SNAP",
		["suffix"] = ext or ".jpg",
		["count"] = 1
	} )

	moduleProxy:Destroy()

	if not r or not cc or cc < 0 then
		return false
	else
		return true
	end
end

-----------------------------------------
--- Get stream service status
-----------------------------------------
function _M.GetStreamServiceStatus( idr, service ,coaf_local)
	if not idr or not service then
		print( "*** Invalid IDR/Service name when getting stream service" )
		return {}
	end

	local svcProxy = GetCoAFProxy( "/DefaultApp/" .. idr .. "/" .. service, "streamer", coaf_local or ngx.var.coaf_location )
	if not svcProxy then
		print( "*** Could not get streaming service proxy of stream '" .. idr .. "', service='" .. service .. "'" )
		return {}
	end

	local r,x,status = svcProxy:GetStatus()
	if r and x and x >=0 and status then
		status["enabled"],status["serviceUri"],status["serviceType"] = svcProxy._P("Enabled","ServiceUri", "ServiceType")
		svcProxy:Destroy()
		return status
	else
		svcProxy:Destroy()
		print( "*** Could not get streaming service status of stream '" .. idr .. "', service='" .. service .. "'" )
		return {}
	end
end

-----------------------------------------
--- Get stream count
-----------------------------------------
function _M.GetStreamCount(streamType, maxCount)
	if streamType == "" then
		return false
	end
	local svcProxy = GetCoAFProxy( "/streams", "streamer", ngx.var.coaf_location )
	if not svcProxy then
		print( "*** Could not get streaming service proxy of GetStreamCount" )
		return false
	end
	local r, msg = svcProxy.GetStreamCount(streamType, maxCount)
	svcProxy:Destroy()
	if r and msg then
		return msg
	else
		print( "*** The stream count is greater than the maximum count" )
		return false
	end	
end
-----------------------------------------
--- Control stream service enable
-----------------------------------------
function _M.ControlStreamServiceEnable( idr, service, en )
	if not idr or not service then
		print( "*** Invalid IDR/Service name when getting stream service" )
		return false
	end

	local svcProxy = GetCoAFProxy( "/DefaultApp/" .. idr .. "/" .. service, "streamer", ngx.var.coaf_location )
	
	if not svcProxy then
		print( "*** Could not get streaming service proxy of stream '" .. idr .. "', service='" .. service .. "'" )
		return false
	end

	local r,msg = svcProxy:Enable(en)
	svcProxy:Destroy()
	if r then
		return true
	else
		print( "*** Could not control streaming service enable/disable of stream '" .. idr .. "', service='" .. service .. "'" )
		print(msg)
		return false
	end	
end
-----------------------------------------
function _M.LoadPreset( id, loc )
	local presetM = GetCoAFProxy( "/PRESET", "web_daemon", ngx.var.coaf_location )
	if not presetM then
		return false, "Fail to connect web_daemon service"
	end

	local x, done = presetM.LoadPreset( id, loc )
	presetM:Destroy()

	if not x then
		return false, "Load preset failed"
	else
		return done
	end
end

-----------------------------------------
--- Get/Set operations.
-----------------------------------------

local getSetTargets = {
	["encoder"] = { bus = "codec", obj = "/Codec/Encoders" },
	["streamer_global"] = { bus = "streamer", obj = "/" },
	["streamer"] = { bus = "streamer", obj = "/DefaultApp" },
	["media_users"] = { bus = "streamer", obj = "/MediaUsers" },
	["audioEngine"] = { bus = "streamer", obj = "/AudioEngine" },
	["avio"] = { bus="codec", obj = "/AVIO" },
	["osd"] = { bus="codec", obj = "/OSD" },
	["onvif"] = { bus="onvif", obj="/" },
	["ptz"] = { bus="onvif", obj="/PTZ" },
	["hostapm"] = { bus="nm_daemon", obj="/raw/HOSTAP" },
	["record_m"] = { bus="record", obj="/" },
	["webConfig"] = { bus="web_daemon", obj="/" }
}

local function ParseParamTarget(idr)
	if idr:match("^_ENV%.") then
		local targ = idr:match("^_ENV%.(.-)%.")
		if not targ or not getSetTargets[targ] then
			return false,nil
		else
			return true,targ
		end
	else
		local targ = idr:match("^(.-)%.")
		if not targ or not getSetTargets[targ] then
			return false,nil
		else
			return false,targ
		end
	end
end

---------------------------------------------------

local function GroupParams(params)
	local group = {}
	local k,v
	for k,v in pairs(params) do
		local isEnv, targ = ParseParamTarget(k)
		if targ then
			if not group[targ] then
				group[targ] = { env={}, env_cc = 0, conf={}, conf_cc = 0 }
			end

			if isEnv then
				group[targ].env[k] = v
				group[targ].env_cc = group[targ].env_cc + 1
			else
				group[targ].conf[k] = v
				group[targ].conf_cc = group[targ].conf_cc + 1
			end
		end
	end
	return group
end

-----------------------------------------------

function _M.Get(params, coaf_local)
	local group = GroupParams(params)
	local k,v
	local cc = 0

	for k,v in pairs(group) do
		if getSetTargets[k].isLocal then --Local get
			if v.conf_cc > 0 then
				--getset.GetValues( k, v.conf ) --TODO:
				local kk, vv
				for kk, vv in pairs(v.conf) do
					params[kk] = vv
					cc = cc + 1
				end
			end
			
			if v.env_cc > 0 then
				--getset.GetValues( "_ENV", v.env ) --TODO:
				local kk, vv
				for kk, vv in pairs(v.conf) do
					params[kk] = vv
					cc = cc + 1
				end
			end
		else --Remote GET
			local proxy = GetCoAFProxy( getSetTargets[k].obj, getSetTargets[k].bus, coaf_local or ngx.var.coaf_location )
			if proxy and v.conf_cc > 0 then
				local r,x,res = proxy:Get( v.conf )
				if r and x and x >= 0 and res then
					local kk,vv
					for kk,vv in pairs(res) do
						params[kk] = vv
						cc = cc + 1
					end
				end
			end

			if proxy and v.env_cc > 0 then
				local r,x,res = proxy:GetEnv( v.env )
				if r and x and x >= 0 and res then
					local kk,vv
					for kk,vv in pairs(res) do
						params[kk] = vv
						cc = cc + 1
					end
				end
			end

			if proxy then
				proxy:Destroy()
			end
		end
	end

	return cc, params
end
-----------------------------------------------

function _M.Set(params, coaf_local)
	local group = GroupParams(params)
	local k,v
	local result = false
	for k,v in pairs(group) do
		if getSetTargets[k].isLocal then --Local SET
			if v.conf_cc > 0 then
				--getset.SetConfigureValues( k, v.conf ) --TODO:
				result = true
			end
			
			if v.env_cc > 0 then
				--getset.SetValues( "_ENV", v.env ) --TODO:
				result = true
			end
		else --Remote SET
			local proxy = GetCoAFProxy( getSetTargets[k].obj, getSetTargets[k].bus, coaf_local or ngx.var.coaf_location )

			if proxy and v.conf_cc > 0 then
				proxy:Set( v.conf )
				result = true
			end

			if proxy and v.env_cc > 0 then
				proxy:SetEnv( v.env )
				result = true
			end

			if proxy then
				proxy:Destroy()
			end
		end
	end

	return result
end


return _M

