local _M = {}

local DeviceNames = {

	["cn"] = {
		["eth0"] = "默认以太网口",
		["eth*"] = "以太网口",
		["wlan0"] = "默认WIFI",
		["wlan*"] = "WIFI",
		["ra0"] = "默认WIFI",
		["ra*"] = "WIFI",
		["modem*"] = "3G/4G设备",
		["usb0"] = "扩展的USB网络连接",
		["usb*"] = "USB网络连接",
	},

	["en"] = {
		["eth0"] = "Default Ethernet",
		["eth*"] = "Ethernet",
		["wlan0"] = "Default WIFI",
		["wlan*"] = "WIFI",
		["ra0"] = "Default WIFI",
		["ra*"] = "WIFI",
		["modem*"] = "3G/4G Modem",
		["usb0"] = "Default USB External",
		["usb*"] = "USB External",
	},

	["jp"] = {
		["eth0"] = "Default Ethernet",
		["eth*"] = "Ethernet",
		["wlan0"] = "Default WIFI",
		["wlan*"] = "WIFI",
		["ra0"] = "Default WIFI",
		["ra*"] = "WIFI",
		["modem*"] = "3G/4G Modem",
		["usb0"] = "Default USB External",
		["usb*"] = "USB External",
	}
}

local EnsuredDeviceNames = {
	["cn"] = {},
	["en"] = {},
	["jp"] = {},

	counters = {
		["eth"] = 0,
		["wlan"] = 0,
		["ra"] = 0,
		["modem"] = 0,
		["ttyUSB"] = 0,
		["usb"] = 0
	}
}

function _M.GetDeviceName(name,lang)
	if lang == nil then
		lang = "en"
	end

	if not EnsuredDeviceNames[lang] then
		lang = "en"
	end
	
	if EnsuredDeviceNames[lang][name] then
		return EnsuredDeviceNames[lang][name]
	end

	local fname = DeviceNames[lang][name]
	if nil ~= fname then
		EnsuredDeviceNames[lang][name] = fname
		return fname
	end

	local k,v
	for k,v in pairs(DeviceNames[lang]) do
		local prefix = name:match( k )
		if prefix then	--Match xxx*
			local fname = v
			EnsuredDeviceNames.counters[prefix] = EnsuredDeviceNames.counters[prefix] + 1
			fname = fname .. " " .. EnsuredDeviceNames.counters[prefix]
			EnsuredDeviceNames[lang][name] = fname
			return fname
		end
	end

	return "UNKNOWN"
end

local StateReasons = {
	["cn"] = {
		[0] = "未知",
		[1] = "已连接",
		[2] = "正在启动",
		[3] = "不受管理的设备",
		[4] = "配置失败",
		[5] = "不可用",
		[6] = "已过期",
		[7] = "加密认证失败",
		[8] = "WIFI管理器失效",
		[9] = "WIFI管理器配置错误",
		[10] = "WIFI管理器失败",
		[11] = "WIFI管理器超时",
		[12] = "不能开启拨号",
		[13] = "拨号连接断开",
		[14] = "拨号失败",
		[15] = "不能开启DHCP地址获取",
		[16] = "DHCP获取地址错误",
		[17] = "DHCP失败",
		[18] = "共享开启失败",
		[19] = "共享失败",
		[20] = "自动IP开启失败",
		[21] = "自动IP错误",
		[22] = "自动IP失败",
		[23] = "Modem忙",
		[24] = "Modem没有拨号音",
		[25] = "Modem没有载波",
		[26] = "Modem拨号超时",
		[27] = "Modem拨号失败",
		[28] = "Modem初始化失败",
		[29] = "APN无效",
		[30] = "没有搜索到移动网络",
		[31] = "注册移动网络失败",
		[32] = "注册移动网络超时",
		[33] = "注册移动网络失败",
		[34] = "SIM卡PIN码验证错误",
		[35] = "设备固件错",
		[36] = "已移除",
		[37] = "休眠",
		[38] = "连接断开",
		[39] = "用户请求",
		[40] = "连接状态改变",
		[41] = "连接已确认",
		[42] = "WIFI管理器有效",
		[43] = "找不到Modem",
		[44] = "蓝牙失败",
		[45] = "没有插入SIM卡",
		[46] = "SIM卡要求PIN码验证",
		[47] = "SIM卡要求PUK",
		[48] = "SIM卡错误",
		[49] = "不支持连接(InfiniBand)",
		[50] = "连接的依赖条件不满足",
		[51] = "RFC2684错误",
		[52] = "Modem管理器无效",
		[53] = "不能连接到指定的SSID",
		[54] = "第二连接失败",
		[55] = "DCB或FCoE设置失败",
		[56] = "Teamd控制失败",
		[57] = "Modem失败",
		[58] = "Modem无效",
		[59] = "SIM卡PIN不正确",
		[60] = "正在建立新的连接",
		[61] = "Parent Changed",
		[62] = "Parent Managed Changed"
	},

	["en"] = {
		[0] = "Unknown",
		[1] = "Connected",
		[2] = "Managed",
		[3] = "Unmanaged",
		[4] = "Config Failed",
		[5] = "Config Unavilable",
		[6] = "Expired",
		[7] = "No Secrets",
		[8] = "Wifi Manager Disconnected",
		[9] = "Wifi Manager Config Fail",
		[10] = "Wifi Manager Failed",
		[11] = "Wifi Manager Timeout",
		[12] = "Start Dial Fail",
		[13] = "Dial Disconnected",
		[14] = "Dial Fail",
		[15] = "Start DHCP Fail",
		[16] = "DHCP Error",
		[17] = "DHCP Failed",
		[18] = "Shared Start Failed",
		[19] = "Shared Failed",
		[20] = "Auto-IP Start Failed",
		[21] = "AUTO-IP Error",
		[22] = "AUTO-IP Failed",
		[23] = "Modem Busy",
		[24] = "Modem No Dial Tone",
		[25] = "Modem No Carrier",
		[26] = "Modem Dial Timeout",
		[27] = "Modem Dial Failed",
		[28] = "Modem Init Failed",
		[29] = "APN Failed",
		[30] = "Registration Not Searching",
		[31] = "Registration Denied",
		[32] = "Registration Timeout",
		[33] = "Registration Failed",
		[34] = "PIN Checking failed",
		[35] = "Firmware Error",
		[36] = "Removed",
		[37] = "Sleeping",
		[38] = "Connection Removed",
		[39] = "User Requested",
		[40] = "Carrier Status Changed",
		[41] = "Connection Assumed",
		[42] = "WIFI Manager avaliable",
		[43] = "Modem Not Found",
		[44] = "Bluetooth Failed",
		[45] = "No SIM Card",
		[46] = "PIN Required",
		[47] = "PUK Required",
		[48] = "SIM Error",
		[49] = "InfiniBand Mode",
		[50] = "Dependency Failed",
		[51] = "RFC2684 Error",
		[52] = "Modem Manager Unavaliable",
		[53] = "SSID Not Found",
		[54] = "Secondary Connection Failed",
		[55] = "DCB or FCoE Setup Failed",
		[56] = "Teamd Control Failed",
		[57] = "Modem Failed",
		[58] = "Modem Unavaliable",
		[59] = "PIN Wrong",
		[60] = "New Connection",
		[61] = "Parent Changed",
		[62] = "Parent Managed Changed"
	},

	["jp"] = {
		[0] = "Unknown",
		[1] = "Connected",
		[2] = "Managed",
		[3] = "Unmanaged",
		[4] = "Config Failed",
		[5] = "Config Unavilable",
		[6] = "Expired",
		[7] = "No Secrets",
		[8] = "Wifi Manager Disconnected",
		[9] = "Wifi Manager Config Fail",
		[10] = "Wifi Manager Failed",
		[11] = "Wifi Manager Timeout",
		[12] = "Start Dial Fail",
		[13] = "Dial Disconnected",
		[14] = "Dial Fail",
		[15] = "Start DHCP Fail",
		[16] = "DHCP Error",
		[17] = "DHCP Failed",
		[18] = "Shared Start Failed",
		[19] = "Shared Failed",
		[20] = "Auto-IP Start Failed",
		[21] = "AUTO-IP Error",
		[22] = "AUTO-IP Failed",
		[23] = "Modem Busy",
		[24] = "Modem No Dial Tone",
		[25] = "Modem No Carrier",
		[26] = "Modem Dial Timeout",
		[27] = "Modem Dial Failed",
		[28] = "Modem Init Failed",
		[29] = "APN Failed",
		[30] = "Registration Not Searching",
		[31] = "Registration Denied",
		[32] = "Registration Timeout",
		[33] = "Registration Failed",
		[34] = "PIN Checking failed",
		[35] = "Firmware Error",
		[36] = "Removed",
		[37] = "Sleeping",
		[38] = "Connection Removed",
		[39] = "User Requested",
		[40] = "Carrier Status Changed",
		[41] = "Connection Assumed",
		[42] = "WIFI Manager avaliable",
		[43] = "Modem Not Found",
		[44] = "Bluetooth Failed",
		[45] = "No SIM Card",
		[46] = "PIN Required",
		[47] = "PUK Required",
		[48] = "SIM Error",
		[49] = "InfiniBand Mode",
		[50] = "Dependency Failed",
		[51] = "RFC2684 Error",
		[52] = "Modem Manager Unavaliable",
		[53] = "SSID Not Found",
		[54] = "Secondary Connection Failed",
		[55] = "DCB or FCoE Setup Failed",
		[56] = "Teamd Control Failed",
		[57] = "Modem Failed",
		[58] = "Modem Unavaliable",
		[59] = "PIN Wrong",
		[60] = "New Connection",
		[61] = "Parent Changed",
		[62] = "Parent Managed Changed"
	},
}

function _M.GetDeviceStateReasonName( id, lang )
	if not lang then
		lang = "en"
	elseif not StateReasons[lang] then
		lang = "en"
	end

	return StateReasons[lang][id] or "UNKNOWN"
end

local StateNames = {
	["cn"] = {
		[0] = "未知",
		[10] = "不受管理",
		[20] = "不可用",
		[30] = "断开",
		[40] = "正在准备",
		[50] = "已配置",
		[60] = "正在进行身份验证",
		[70] = "正在获取IP",
		[80] = "已获得IP地址",
		[90] = "等待第二个连接",
		[100] = "已连接",
		[110] = "正在断开",
		[120] = "失败"
	},

	["en"] = {
		[0] = "Unknown",
		[10] = "Unmanaged Device",
		[20] = "Unavaliable",
		[30] = "Disconnected",
		[40] = "Preparing",
		[50] = "Configured",
		[60] = "authorizing",
		[70] = "Requesting IP",
		[80] = "IP Determined",
		[90] = "Waiting for Secondary Connection",
		[100] = "Active",
		[110] = "Deactivating",
		[120] = "FAILED"
	},

	["jp"] = {
		[0] = "Unknown",
		[10] = "Unmanaged Device",
		[20] = "Unavaliable",
		[30] = "Disconnected",
		[40] = "Preparing",
		[50] = "Configured",
		[60] = "authorizing",
		[70] = "Requesting IP",
		[80] = "IP Determined",
		[90] = "Waiting for Secondary Connection",
		[100] = "Active",
		[110] = "Deactivating",
		[120] = "FAILED"
	}
}

function _M.GetDeviceStateName( id, lang )
	if not lang then
		lang = "en"
	elseif not StateReasons[lang] then
		lang = "en"
	end

	return StateNames[lang][id] or "UNKNOWN"
end


return _M


