--Controller of dashboard
local _M = require( "controller.generic" )()

------------------------------------------------

local function GetSystemStatus()
	local r, cpuLoad = _M.MasterCall( "SYS_STATUS.GetCpuLoad" )
	cpuLoad = r and cpuLoad or 0

	local r, memFree, memTotal = _M.MasterCall( "SYS_STATUS.GetMemUse" )
	local r, ptime,stime = _M.MasterCall( "SYS_STATUS.GetPersisTime" )
	local r, diskInfo = _M.MasterCall("SYS_STATUS.GetDiskSpaces" )

	return [[ {"Result":200, "Msg":"", "data": 
			{
				"cpu.load": ]] .. math.floor(cpuLoad) .. [[,
				"mem.free": ]] .. math.floor(memFree/1024) .. [[,
				"mem.total": ]] .. math.floor(memTotal/1024) .. [[,
				"systime.start":"]] .. string.format( "%04d/%02d/%02d %02d:%02d:%02d", stime.year,stime.month,stime.day,stime.hour,stime.min,stime.sec ) .. [[",
				"systime.persis":]] .. ptime .. [[,
				"sysdisk.free":]] .. diskInfo.sys.free .. [[,
				"sysdisk.use":]] .. diskInfo.sys.percent .. [[,
				"datadisk.free":]] .. diskInfo.data.free .. [[,
				"datadisk.use":]] .. diskInfo.data.percent .. [[
			} 
	} ]]
end

------------------------------------------------

local function GetSignalStatus()
	local r, info = _M.MasterCall( "BUS.GetSignalStatus" )
	if not r or not info then
		return [[ {"Result":400, "Msg":"Get signal status fail", "data": {} } ]]
	else
		return [[ {"Result":200, "Msg":"", "data": 
				{
					"video.locked": ]] .. info.video["video.locked"] .. [[,
					"video.format_name": "]] .. info.video["video.format_name"] .. [[",
					"audio.sampling": ]] .. info.audio["audio.sampling"] .. [[,
					"video.source": "]] .. info.video["video.source"] .. [[",
					"video.source_detected": "]] .. info.video["video.source_detected"] .. [[",
					"audio.source": "]] .. info.audio["audio.source"] .. [["
				} 
		} ]]
		
	end
end

------------------------------------------------
local function GetVideoSpecials()
	local params = {
		["avio.video.source_crop.enable"] = 0,
		["avio.video.source_crop.cropX"] = 0,
		["avio.video.source_crop.cropY"] = 0,
		["avio.video.source_crop.cropW"] = 0,
		["avio.video.source_crop.cropH"] = 0,
		["avio.video.spec_out.format"] = "default",
		["avio.video.spec_out.mode"] = "default",
		["avio.video.out_crop.enable"] = 0,
		["avio.video.out_crop.cropX"] = 0,
		["avio.video.out_crop.cropY"] = 0,
		["avio.video.out_crop.cropW"] = 0,
		["avio.video.out_crop.cropH"] = 0,
		["avio.video.RMF"] = "0"
	}

	local r, cc, rets = _M.MasterCall( "BUS.Get", params )
	if not r or not rets then
		return [[ {"Result":400, "Msg":"Get video specials fail", "data": {} } ]]
	else
		local src_crop = "default"
		if rets["avio.video.source_crop.enable"] and rets["avio.video.source_crop.enable"] ~= 0 then
			src_crop = "(" ..	(rets["avio.video.source_crop.cropX"] or 0) .. "," ..
										(rets["avio.video.source_crop.cropY"] or 0) .. "," ..
										(rets["avio.video.source_crop.cropW"] or 0) .. "," ..
										(rets["avio.video.source_crop.cropH"] or 0) .. ")"
		end
		
		local spec_out = "default"
		if rets["avio.video.spec_out.format"] ~= "default" or rets["avio.video.spec_out.mode"] ~= "default" then
			if rets["avio.video.spec_out.format"] ~= "default" then
				spec_out = string.match( rets["avio.video.spec_out.format"], "VIDOUT_(.+)" ) or ""
				spec_out = spec_out .. ", "
			end
			
			if rets["avio.video.spec_out.mode"] == "default"  then
				spec_out = spec_out .. "bydefault"
			elseif rets["avio.video.spec_out.mode"] == "OUTCTRL_ASPECT" then
				spec_out = spec_out .. "aspect"
			elseif rets["avio.video.spec_out.mode"] == "OUTCTRL_P2P" then
				spec_out = spec_out .. "p2p"
			elseif rets["avio.video.spec_out.mode"] == "OUTCTRL_PADDING" then
				spec_out = spec_out .. "padding"
			else
				spec_out = spec_out .. "scale"
			end
		end
	
		local out_crop = "default"
		if rets["avio.video.out_crop.enable"] and rets["avio.video.out_crop.enable"] ~= 0 then
			out_crop = "(" ..	(rets["avio.video.out_crop.cropX"] or 0) .. "," ..
										(rets["avio.video.out_crop.cropY"] or 0) .. "," ..
										(rets["avio.video.out_crop.cropW"] or 0) .. "," ..
										(rets["avio.video.out_crop.cropH"] or 0) .. ")"
		end
		
		local rmf = rets["avio.video.RMF"] or "0"
	
		return [[ {"Result":200, "Msg":"", "data": 
				{
					"video.special.source_crop": "]] .. src_crop .. [[",
					"video.special.spec_out": "]] .. spec_out .. [[",
					"video.special.out_crop": "]] .. out_crop .. [[",
					"video.special.rmf": "]] .. rmf .. [["
				} 
		} ]]
	end
end

------------------------------------------------

local function GetStreamStatus()
	return [[ {"Result":200, "Msg":"", "data": {} } ]]
end

------------------------------------------------

local function GetEncoderStatus( streamId )
	return [[ {"Result":200, "Msg":"", "data": {} } ]]
end

------------------------------------------------

local function GetPlatformStatus()
	return [[ {"Result":200, "Msg":"", "data": {} } ]]
end

------------------------------------------------

local function GetNetworkStatus()
	local r, stat, details = _M.MasterCall( "NET_STATUS.GetSummaryStatus" )
	if r and stat then
		local connectivity = ""
		if details and #details > 0 then
			local nm_names = require "kiloview.netdevNames"
			connectivity = nm_names.GetDeviceName( details[1].Name, _M.pageLang ) or ""
			connectivity = connectivity .. " (" .. _M.langs.TOTAL_CONN .. #details .. _M.langs.CONNECTION ..  ")"
		end
		return [[ {"Result":200, "Msg":"", "data": {
					"status":"]] .. _M.langs.NET_STATUS[stat] .. [[",
					"connectivity":"]] .. connectivity .. [["
			   } }]]
	else
		return [[ {"Result":400, "Msg":"", "data": {} } ]]
	end
end

------------------------------------------------

local function ForceDetect()
	local r, msg = _M.MasterCall( "NET_STATUS.Check", false, true )
	return [[ {"Result":200, "Msg":"]] .. (msg or "---") .. [[", "data": {} } ]]
end

------------------------------------------------
local function Ping(target)
	local r, ok, reach, loss = _M.MasterCall( "NET_STATUS.Ping", target )
	--local msg = (r and "true" or "false") .. " " .. tostring(ok) .. " " .. tostring(reach) .. " " .. tostring(loss)
	local msg = ""
	if not r then
		return [[ {"Result":200, "data": { "reachable":"system error", "loss_rate":100, "msg":"]] .. msg .. [[" } } ]]
	elseif not ok or not loss then
		return [[ {"Result":200, "data": { "reachable":"Ping error", "loss_rate":100, "msg":"]] .. msg .. [[" } } ]]
	elseif reach and loss < 100 then
		return [[ {"Result":200, "data": { "reachable":"]] .. _M.langs.REACH .. [[", "loss_rate":]] .. loss .. [[, "msg":"]] .. msg .. [[" } } ]]
	else
		return [[ {"Result":200, "data": { "reachable":"]] .. _M.langs.UNREACH .. [[", "loss_rate":100, "msg":"]] .. msg .. [[" } } ]]
	end
end

local function TracerouteStart(target)
	local r, ok, content = _M.MasterCall( "NET_STATUS.TracerouteStart", target )
	local msg = ""
	if not r then
		return [[ {"Result":200, "data": { "reachable":"system error", "msg":"]] .. msg .. [[" } } ]]
	elseif not ok then
		return [[ {"Result":200, "data": { "reachable":"Start error", "msg":"]] .. msg .. [[" } } ]]
	else
		return [[ {"Result":200, "data": { "reachable":"success", "msg":"]] .. msg .. [[" } } ]]
	end
end

local function TracerouteSet(target)
	local r, ok, content,status = _M.MasterCall( "NET_STATUS.TracerouteSet", target )
	local msg = ""
	if not r then
		return [[ {"Result":200, "data": { "reachable":"system error", "msg":"]] .. msg .. [[" } } ]]
	elseif not ok then
		return [[ {"Result":200, "data": { "reachable":"Set error", "msg":"]] .. msg .. [[" } } ]]
	else
		return [[ {"Result":200, "data": { "reachable":"success","status":"]] .. status .. [[","content":"]].. content .. [[", "msg":"]] .. msg .. [[" } } ]]
	end
end

------------------------------------------------

local function PlatformControl(opts)
	return [[ {"Result":200, "Msg":"", "data": {} } ]]
end

------------------------------------------------
local function SpeedMeasurement(target)
	local is_ok, skip, result, info = _M.MasterCall("SYS_STATUS.iperf3", target)
	if not is_ok or not skip then
		return [[ {"Result":400, "data": { "reachable":"system error" } } ]]
	end

	if result then
		return [[ {"Result":200, "data": { "reachable":"Test Success", "msg":"]] .. info .. [[" } } ]]
	end
	return [[ {"Result":400, "data": { "reachable":"Test error" } } ]]
end
------------------------------------------------

_M.AjaxExport ("GetSystemStatus", GetSystemStatus, "json") 
_M.AjaxExport ("GetSignalStatus", GetSignalStatus, "json" )
_M.AjaxExport ("GetStreamStatus", GetStreamStatus, "json" )
_M.AjaxExport ("GetEncoderStatus", GetEncoderStatus, "json" )
_M.AjaxExport ("GetPlatformStatus", GetPlatformStatus, "json" )
_M.AjaxExport ("GetNetworkStatus", GetNetworkStatus, "json" )
_M.AjaxExport ("PlatformControl", PlatformControl, "json" )
_M.AjaxExport ("GetVideoSpecials", GetVideoSpecials, "json" )
_M.AjaxExport ("ForceDetect", ForceDetect, "json" )
_M.AjaxExport ("Ping", Ping, "json" )
_M.AjaxExport ("TracerouteStart", TracerouteStart, "json" )
_M.AjaxExport ("TracerouteSet", TracerouteSet, "json" )

_M.AjaxExport ("SpeedMeasurement", SpeedMeasurement, "json")
------------------------------------------------

return _M
