--Controller of media_users
local _M = require( "controller.generic" )()
local cgilua = require "cgilua"

------------
local function AddUser( username, password )
	if not username or username == "" or not password or password == "" then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.ERRMSG_INVALID_USERNAME_OR_PASSWORD .. [["}]]
	end

	local args = {
		["media_users.add.username"] = username,
		["media_users.add.password"] = password
	}

	local r,x = _M.MasterCall( "BUS.Set", args )
	if r and x then 
		local status = {
			["media_users.lastError.operation"] = "",
			["media_users.lastError.status"] = ""
		}

		r,x,status = _M.MasterCall( "BUS.Get", status )
		local status_code = status["media_users.lastError.status"] or "error"
		if status_code == "ok" then
			return [[ {"Result":200, "Msg":"]] .. _M.langs.MSG_ADD_DONE .. [[", "Refresh":1 }]]
		else
			return [[ {"Result":400, "Msg":"]] .. (_M.langs.MSG_OPRFAILS[status_code] or _M.langs.MSG_UNKNOWN_ERROR).. [[" }]]
		end
	else
		return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_ADD_FAIL .. [["}]]
	end
end

local function ChangeUser( username, password )
	if not username or username == "" or not password or password == "" then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.ERRMSG_INVALID_USERNAME_OR_PASSWORD .. [["}]]
	end

	local args = {
		["media_users.change.username"] = username,
		["media_users.change.password"] = password
	}

	local r,x = _M.MasterCall( "BUS.Set", args )
	if r and x then 
		local status = {
			["media_users.lastError.operation"] = "",
			["media_users.lastError.status"] = ""
		}

		r,x,status = _M.MasterCall( "BUS.Get", status )
		local status_code = status["media_users.lastError.status"] or "error"
		if status_code == "ok" then
			return [[ {"Result":200, "Msg":"]] .. _M.langs.MSG_CHANGE_DONE .. [[", "Refresh":1 }]]
		else
			return [[ {"Result":400, "Msg":"]] .. (_M.langs.MSG_OPRFAILS[status_code] or _M.langs.MSG_UNKNOWN_ERROR).. [[" }]]
		end
	else
		return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_CHANGE_FAIL .. [["}]]
	end
end

local function RemoveUser(username)
	if not username or username == "" then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.ERRMSG_INVALID_USERNAME_OR_PASSWORD .. [["}]]
	end

	local args = {
		["media_users.delete.username"] = username,
	}

	local r,x = _M.MasterCall( "BUS.Set", args )
	if r and x then 
		local status = {
			["media_users.lastError.operation"] = "",
			["media_users.lastError.status"] = ""
		}

		r,x,status = _M.MasterCall( "BUS.Get", status )
		local status_code = status["media_users.lastError.status"] or "error"
		if status_code == "ok" then
			return [[ {"Result":200, "Msg":"]] .. _M.langs.MSG_REMOVE_DONE .. [[", "Refresh":1 }]]
		else
			return [[ {"Result":400, "Msg":"]] .. (_M.langs.MSG_OPRFAILS[status_code] or _M.langs.MSG_UNKNOWN_ERROR).. [[" }]]
		end
	else
		return [[ {"Result":400, "Msg":"]] .. _M.langs.MSG_REMOVE_FAIL .. [["}]]
	end
end
-------------- AJAX CALL ----------------
_M.AjaxExport ("AddUser", AddUser, "json") 
_M.AjaxExport ("RemoveUser", RemoveUser, "json" )
_M.AjaxExport ("ChangeUser", ChangeUser, "json" )
---------------------------------------------

function _M.GetUsers()
	local gets = {}
	local i
	for i=0, 15 do
		gets[ "media_users.users[" .. i .. "].username" ] = ""
	end
	local r,x,gets = _M.MasterCall( "BUS.Get", gets )
	if r and gets then
		local rets = {}
		local k,v
		for k,v in pairs(gets) do
			if v and v ~= "" then
				table.insert( rets, { username = v } )
			end
		end

		table.sort( rets, function(a,b)
			return a.username < b.username
		end)

		return rets
	else
		return nil
	end
end

return _M

