--Controller of netlist
local _M = require( "controller.generic" )()

-------------- AJAX CALL ----------------
local function StartHostAP(ifname, ssid, key, iprange, chan, hideSsid )
	if type(key) ~= "string" or #key < 8 then
		return [[ {"Result":400, "Msg":"]] .. _M.langs.START_AP_SHORT_KEY .. [["} ]]
	end
	local r,code = _M.MasterCall( "HOSTAP_CTRL.StartHostAP", ifname, "wifi", "", ssid, key, iprange, 24, chan and tonumber(chan) or 6, hideSsid or 0 )
	if r and code then
		return [[ {"Result":200, "Msg":"]] .. _M.langs.START_AP_DONE .. [["} ]]
	else
		return [[ {"Result":400, "Msg":"]] .. _M.langs.START_AP_FAIL .. [["} ]]
	end
end

local function ModifyOMNI(is_open)
	local is_ok, skip = _M.MasterCall("BUS.Set", {
		["hostapm.enableOMNI.enable"] = is_open
	})
	if is_ok and skip then
		return [[ {"Result":200, "Msg":"]] .. (_M.langs.MODIFY_OMNI_DONE or "nil") .. [["} ]]
	else
		return [[ {"Result":400, "Msg":"]] .. (_M.langs.MODIFY_OMNI_FAIL or "nil") .. [["} ]]
	end
end

local function StopHostAP(ifname)
	local r,code = _M.MasterCall( "HOSTAP_CTRL.StopHostAP" )
	if r and code then
		return [[ {"Result":200, "Msg":"]] .. _M.langs.STOP_AP_DONE .. [["} ]]
	else
		return [[ {"Result":400, "Msg":"]] .. _M.langs.STOP_AP_FAIL .. [["} ]]
	end
end

local function ResetModem(ifname)
	local r,code = _M.MasterCall( "NETWORK_MANAGER.ResetModem", ifname )
	if r and code then
		return [[ {"Result":200, "Msg":"]] .. _M.langs.RESET_MODEM_DONE .. [["} ]]
	else
		return [[ {"Result":400, "Msg":"]] .. _M.langs.RESET_MODEM_FAIL .. [["} ]]
	end
end

local function SaveModems(modems)
	local r,code = _M.MasterCall( "HOSTAP_CTRL.UpdateModems", modems )
	if r and code then
		return [[ {"Result":200, "Msg":"]] .. _M.langs.SAVE_MODEMS_DONE .. [["} ]]
	else
		return [[ {"Result":400, "Msg":"]] .. _M.langs.SAVE_MODEMS_FAIL .. [["} ]]
	end
end

------------------------------------------------
_M.AjaxExport ("StartHostAP", StartHostAP, "json") 
_M.AjaxExport ("ModifyOMNI", ModifyOMNI, "json") 
_M.AjaxExport ("StopHostAP", StopHostAP, "json" )
_M.AjaxExport ("ResetModem", ResetModem, "json" )
_M.AjaxExport ("SaveModems", SaveModems, "json" )
------------------------------------------------

function _M.convSignalIcon(s)
	return string.format( [[<img src="./images/%d.png" alt="%d%%" />]], math.floor(s/20 + 1), math.floor(s/20 + 1) )
end

function _M.GetModems()
	--TODO: Improve this.
	local file = io.open( "/data/configs/option-modems.conf", "r" )
	if file then
		local r = file:read("a")
		file:close()
		return r
	else
		return ""
	end
end

function _M.GetRtl()
	--TODO: Rtl this.
	local file_rtl = io.popen("/bin/ls /proc/net/ | /bin/grep rtl | /usr/bin/awk '{printf$1}'")
	local output = file_rtl:read("*a")
	return output
end


function _M.GetChannel()
	--TODO: channel this.
	local channel_list = {}
	local file_rtl = io.popen("/bin/cat  /proc/net/rtl88x2cu/wlan0/chan_plan | /usr/bin/awk '/^[[:space:]]*[0-9]+/ {print $1}'")
	if file_rtl then
		local line = file_rtl:read("l")
		while line do
			table.insert(channel_list, line)
			line = file_rtl:read("l")
		end
	    file_rtl:close()
		return channel_list
	end
end

------------------------------------------------

return _M
