local MSG = require "kvs_msg"
local CO = require "kvs_coservice"
local cjson = require "cjson"

local __flg, FEATURES = pcall( require, "features" )
if not __flg or type(FEATURES) ~= "table" then
	FEATURES = {
	}
end

local _M = {
	lang_path = "/data/web/kilolink/html/static/lang",
	support_langs = {
	},
	APIS = {}
}

_M.APIS = _M.APIS or {}

function _M.Init()
	_M.support_langs = {}

	--Check language paths.
	local langs_f = io.popen("/usr/bin/find " .. _M.lang_path .. "/ -mindepth 1 -maxdepth 1 -type d", "r")

	if not langs_f then
		return
	end

	--Parse languages.
	local all_langs={}
	local ln = langs_f:read("*l")
	while ln do
		local lang_name = ln:match(".+/(.*)$")
		if lang_name and lang_name ~= "failsafe" then
			table.insert( all_langs, lang_name )
		end
		ln = langs_f:read("*l")
	end
	langs_f:close()

	local failsafe_items={}

	--Check all failsafe items.
	langs_f = io.popen("/usr/bin/find " .. _M.lang_path .. "/failsafe/ -name '*.json'", "r" )
	if langs_f then
		ln = langs_f:read("*l")
		local item
		while ln do
			item = ln:match("^.+/failsafe/(.+)$")
			if item then
				failsafe_items[item] = item
			end
			ln = langs_f:read("*l")
		end
	end

	--Merge all language JSON files.
	for _i, lang_name in ipairs(all_langs) do
		local lang_opt = {
			value = lang_name,
			name = lang_name,
			valid = false,
			cached_langs = "{}"
		}

		local parse_items = {}
		local f = io.popen( "/usr/bin/find " .. _M.lang_path .. "/" .. lang_name .. "/ -name '*.json'", "r" )
		if f then
			local copied_failsafe_items={}
			for k,v in pairs(failsafe_items) do
				copied_failsafe_items[k] = v
			end

			ln = f:read("*l")
			local item
			while ln do
				table.insert(parse_items, ln)
				item = ln:match("^.+/" .. lang_name .. "/(.+)$")
				if item and copied_failsafe_items[item] then
					copied_failsafe_items[item] = nil
				end
				ln = f:read("*l")
			end
			f:close()

			for k,v in pairs(copied_failsafe_items) do
				table.insert(parse_items, _M.lang_path .. "/failsafe/" .. k)
			end
		end

		local lang_table = {}

		for _x, filename in ipairs(parse_items) do
			local json_f = io.open(filename, "r")
			if json_f then
				local json_text = json_f:read("*a")
				json_f:close()
				json_text = json_text:match("%b{}")
				--Valid JSON contents.
				if json_text then
					local valid_content = false
					local json_obj = cjson.decode(json_text)
					if type(json_obj) == "table" then
						--Check has any content in JSON.
						for k,v in pairs(json_obj) do
							valid_content = true
							break
						end
					end
					if valid_content then
						--Extract { ...content... }, get content only.
						local extract_json_text = json_text:match("^{(.+)}")
						table.insert( lang_table, extract_json_text)
					end

					if filename:match("lang%-basic%.json") and valid_content then
						lang_opt.valid = true
						lang_opt.name = json_obj.languageName or lang_name
						lang_opt.order = json_obj.order
					end
				end
			end
		end

		if lang_opt.valid then
			lang_opt.cached_langs = "{\n" .. table.concat(lang_table, ",\n") .. "\n}"
			lang_opt.order = lang_opt.order or 9999
			table.insert(_M.support_langs, lang_opt)
			_M.support_langs[lang_name] = lang_opt

			--Export API.
			_M[lang_name] = function()
				ngx.say(_M.support_langs[lang_name].cached_langs)
				return true
			end
		end
	end

	--Sort according to the order.
	table.sort(_M.support_langs, function(a,b)
		if a == b or not b.order then
			return false
		elseif not a.order then
			return true
		elseif a.order == b.order then
			return a.value < b.value
		else
			return a.order < b.order
		end
	end)
end

_M.Init()

return _M

