#!/usr/bin/env lua
local coaf = require "coaf.core"
local _dbg = require "coaf.debug"

-- Change current dir to app location
coaf.ChDir(coaf.GetAppDir(arg))

local socket = require "cosocket.core"
local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
local service = require "coaf.service"

local roll = require "coaf.log.rollfile"

local withLog = false
local _x
for _x=1, #arg do
	if arg[_x] == "--log" then
		withLog = true
		break
	end
end

if withLog then
	-- ** initialize the log file **
	local roll = require "coaf.log.rollfile"
	os.execute( "/bin/mkdir -p /tmp/" )
	local logF = roll.Create("/tmp/record_manager.log")
	if not logF or not logF:init() then
		_dbg.Error("RECORD_M", "Fail to open log file: /tmp/record_manager.log, so the log messages should be output to console.")
		withLog = false
	else
		_dbg.AddLogger("logfile", logF, nil, nil)
	end
	-- ** log **
end

coaf.Init( "record_manager" )

_G.print = function(...)
	_dbg.Info1( "RECORD_M", ... )
end

--The hostapd service monitor, for service a controlable WIFI sharing.
local dbus = require "kiloview.dbusServices"
--local dbusCli = require "kiloview.dbusClients"

local _DECL = require "kiloview.dbus.sunshine_record"
local configure = require "kiloview.dbus.sunshine_configure"
local modules = require "kiloview.dbus.moonlight_module"

require "config"
require "configuredb"

local diskRecorder = require "diskRecorder"

---------------------------------------------------
--NOTE: Must be global here.
_G.CONFIG_LOADING = true
_G.record_m = CONFIGS.InstanceOf( RECORD_MAN_Template, "record_m" )
_G.record_m:LoadFile( "/etc/record_m.conf" )
_G.CONFIG_LOADING = false
---------------------------------------------------

dbus.Init( "SYSTEM" )
service.InitSocket()

local cosvc_Record
local period_timer

----------------------------------------------------

local svcconn,msg = dbus.CreateServiceConnection( _DECL.RECORDM_BUSNAME )
if not svcconn then
	error(msg)
end

---------------------------------------------------

local function diskCheck()
	diskRecorder.StepCheck()
end

local function Main()

-- NOTE here:
-- The record manager is some specical compares to other service.
-- When 'udev' found removable disk(s), it should call 'recordUdevClients'
-- and 'recordUdevClients' will lauch this service by DBus. At that 
-- moment, the 'coserver' maybe not started yet! (udev is started 
-- at system loading time). So, the co-service creation should be
-- fail, need try it again and again. Now the service module support
-- the retrying.

	cosvc_Record = service.New( "record", "unix:/var/run/record.sock" )
	if cosvc_Record then
		_dbg.Info( "RECORD_M", "CoService of 'record' is created." )
		cosvc_Record:CreateCoServerProxy()
	else
		_dbg.Error( "RECORD_M", "Fail created CoService!" )
	end

	local r,msg,svc
	svc, msg = svcconn:CreateService( _DECL.RECORDM_OBJ )
	if not svc then
		error(msg)
	end
	
	r,msg = svc:AddInterfaceWithXmlMeta( modules.iModule, modules.iModule_decl )
	if not r then
		error(msg)
	end
	
	r,msg = svc:AddInterfaceWithXmlMeta( modules.iModuleEvents, modules.iModuleEvents_decl )
	if not r then
		error(msg)
	end
	
	r,msg = svc:AddInterfaceWithXmlMeta( configure.iConfigure, configure.iConfigure_decl )
	if not r then
		error(msg)
	end
	
	r,msg = svc:AddInterfaceWithXmlMeta( configure.iConfigureEvents, configure.iConfigureEvents_decl )
	if not r then
		error(msg)
	end
	
	r,msg = svc:AddInterfaceWithXmlMeta( _DECL.iDiskManager, _DECL.iDiskManager_decl )
	if not r then
		error(msg)
	end
	
	r,msg = svc:AddInterfaceWithXmlMeta( _DECL.iRecord, _DECL.iRecord_decl )
	if not r then
		error(msg)
	end
	
	r,msg = svc:AddInterfaceWithXmlMeta( _DECL.iRecordEvents, _DECL.iRecordEvents_decl )
	if not r then
		error(msg)
	end
	
	r,msg = svc:BindInterfaceObject( modules.iModule, diskRecorder )
	if not r then
		error(msg)
	end
	
	r,msg = svc:BindInterfaceObject( configure.iConfigure, diskRecorder )
	if not r then
		error(msg)
	end
	
	--Init, with current dbus service.
	diskRecorder.Init( svc )
	
	r,msg = svc:BindInterfaceObject( _DECL.iRecord, diskRecorder )
	if not r then
		error(msg)
	end
	
	r,msg = svc:BindInterfaceObject( _DECL.iDiskManager, diskRecorder )
	if not r then
		error(msg)
	end
	
	svc:Online()

	if cosvc_Record then
		cosvc_Record:ExposeObject( "/", diskRecorder )
	end

	print( "Disk/recording manager service running..." )

	period_timer = coaf.NewTimer( diskCheck, 0.01, 8 )--After 0.01 second, and per 5 seconds check.
	if period_timer then
		period_timer:start()
	end
end

coaf.Start(Main)

--------
_dbg.Info( "RECORD_M", "RecordManager service is terminated." )


