local _dbg = require "coaf.debug"

local _M = {
	CMD = {},
	LOC = {},
	SVC = {},
	OBJ = {}
}

function _M.MatchStringContent(str)
	local i
	if not str then
		return false
	end

	if #_M.CMD > 0 then
		local done
		for i=1, #_M.CMD do
			if str:match("@.-%s+" .. _M.CMD[i]) then
				done = true
				break
			end
		end
		if not done then
			return false
		end
	end

	if #_M.LOC > 0 then
		local done
		for i=1, #_M.LOC do
			if str:match("LOC:%s*" .. _M.LOC[i]) then
				done = true
				break
			end
		end
		if not done then
			return false
		end
	end

	if #_M.SVC > 0 then
		local done
		for i=1, #_M.SVC do
			if str:match("SVC:%s*" .. _M.SVC[i]) then
				done = true
				break
			end
		end
		if not done then
			return false
		end
	end

	if #_M.OBJ > 0 then
		local done
		for i=1, #_M.OBJ do
			if str:match("OBJ:%s*" .. _M.OBJ[i]) then
				done = true
				break
			end
		end
		if not done then
			return false
		end
	end

	return true
end

function _M.MatchContent(cont)
	local i

	if #_M.CMD > 0 then
		if not cont.CMD then
			return false
		end

		local done
		for i=1, #_M.CMD do
			if cont.CMD:match(_M.CMD[i]) then
				done = true
				break
			end
		end
		if not done then
			return false
		end
	end

	if #_M.LOC > 0 then
		local loc = cont.headers and cont.headers.LOC
		if not loc then
			return false
		end
		local done
		for i=1, #_M.LOC do
			if loc:match(_M.LOC[i]) then
				done = true
				break
			end
		end
		if not done then
			return false
		end
	end

	if #_M.SVC > 0 then
		local svc = cont.headers and cont.headers.SVC
		if not svc then
			return false
		end
		local done
		for i=1, #_M.SVC do
			if svc:match(_M.SVC[i]) then
				done = true
				break
			end
		end
		if not done then
			return false
		end
	end

	if #_M.OBJ > 0 then
		local obj = cont.headers and cont.headers.OBJ
		if not obj then
			return false
		end
		local done
		for i=1, #_M.OBJ do
			if obj:match(_M.OBJ[i]) then
				done = true
				break
			end
		end
		if not done then
			return false
		end
	end

	return true
end

function _M.ClearFilter()
	_M.CMD = {}
	_M.LOC = {}
	_M.SVC = {}
	_M.OBJ = {}
end

function _M.AddFilter( filterStr )
	if type(filterStr) ~= "string" then
		return false
	end
	filterStr = filterStr .. ";"
	local k,v
	for k,v in filterStr:gmatch("(.-)=(.-)[,;]+") do
		if k == "CMD" or k == "cmd" then
			if not _M.CMD[v] then
				_M.CMD[v] = true
				table.insert(_M.CMD, v)
			end
		elseif k == "LOC" or k == "loc" then
			if not _M.LOC[v] then
				_M.LOC[v] = true
				table.insert(_M.LOC, v)
			end
		elseif k == "SVC" or k == "svc" then
			if not _M.SVC[v] then
				_M.SVC[v] = true
				table.insert(_M.SVC, v)
			end
		elseif k == "OBJ" or k == "obj" then
			if not _M.OBJ[v] then
				_M.OBJ[v] = true
				table.insert(_M.OBJ, v)
			end
		else
			_dbg.Warn( "DEBUG", "Invalid protocol filter type of:", k )
		end
	end
	return true
end

return _M

