pcall( require, "sys_setting" )

------------------------
_G.KV = _G.KV or {}
local FAULT_LANG = "en"
local ALL_LANGUAGES = {
	"en", "cn"
}

if type(_G.KV.SERIAL_HISTORIES) == "table" and #_G.KV.SERIAL_HISTORIES > 0 then
	_G.KV.SERIAL_NO = _G.KV.SERIAL_HISTORIES[ #_G.KV.SERIAL_HISTORIES ].serial
end

if type(_G.KV.LANGUAGE_CONFIGS) == "table" and #_G.KV.LANGUAGE_CONFIGS > 0 then
	_G.KV.LANGUAGE_SUPPORT = _G.KV.LANGUAGE_CONFIGS[ #_G.KV.LANGUAGE_CONFIGS ]
end

------------------------
local _M = {
	configFileLoc = "/usr/share/lua/5.3"
}
--------
local function SaveSerialConfig()
	local cfgfile = io.open( _M.configFileLoc .. "/sys_setting.lua", "w" )
	if not cfgfile then
		print( "***ERROR: Could not create the system setting configure file!" )
		return false
	else
		cfgfile:write( [==[
--Kiloview's global configurations.
_G.KV = _G.KV or {}
]==] )
		
		cfgfile:write( [==[
_G.KV.SERIAL_HISTORIES = {
]==] )
		local i
		if type(_G.KV.SERIAL_HISTORIES) == "table" then
			for i = 1, #_G.KV.SERIAL_HISTORIES do
				local item = _G.KV.SERIAL_HISTORIES[i]
				cfgfile:write( [[    { date="]] .. ( item.date or "~NONE~") .. [[", serial="]] .. (item.serial or "BADBAD0000") .. '", comment=[==[' .. (item.comment or "") .. ']==] }, \n' )
			end
		end
	
		cfgfile:write( "}\n" )
		
		cfgfile:write( [==[
_G.KV.LANGUAGE_CONFIGS = {
]==] )
		local i
		if type(_G.KV.LANGUAGE_CONFIGS) == "table" then
			for i = 1, #_G.KV.LANGUAGE_CONFIGS do
				local item = _G.KV.LANGUAGE_CONFIGS[i]
				cfgfile:write( [[    { date="]] .. ( item.date or "~NONE~") .. [[", langs="]] .. (item.langs or FAULT_LANG) .. [[", def_lang="]] .. (item.def_lang or "") .. '", comment=[==[' .. (item.comment or "") .. ']==] }, \n' )
			end
		end
	
		cfgfile:write( "}\n" )
		cfgfile:close()
	end
	return true
end

--------

function _M.SetConfigLocation( loc )
	_M.configFileLoc = loc
end

--------

function _M.CheckDefaultLanguage(specLang)
	if not specLang or specLang == "" then
		if type(_G.KV.LANGUAGE_SUPPORT) == "table" then
			specLang = _G.KV.LANGUAGE_SUPPORT.def_lang or FAULT_LANG
		else
			specLang = FAULT_LANG
		end
	else
		specLang = specLang:lower()
	end

	local allLangs = _M.GetAllLanguages()
	local i
	for i=1,#allLangs do
		if allLangs[i] == specLang then
			return specLang
		end
	end
	return allLangs[1] or FAULT_LANG
end

--------

function _M.GetAllLanguages()
	if type(_G.KV) ~= "table" or type(_G.KV.LANGUAGE_SUPPORT) ~= "table" then
		return ALL_LANGUAGES 
	else
		local langs = _G.KV.LANGUAGE_SUPPORT.langs or FAULT_LANG
		if not _M.CachingLangs or _M.CachingLangs ~= langs then
			_M.CachingLangs = langs
			langs = langs .. ","
			local langs_table = {}
			local match
			for match in langs:gmatch( "(.-)[, \t]+" ) do
				if match == "all" then
					langs_table = ALL_LANGUAGES
					break
				else
					table.insert( langs_table, match:lower() )
				end
			end
			_M.CachingLangsTable = langs_table
			return langs_table
		else
			return _M.CachingLangsTable
		end
	end
end

--------

function _M.GetSerialNo()
	return _G.KV.SERIAL_NO or "0000000000"
end

--------

function _M.AddSerialNo( date, serial, comment )
	if type(_G.KV.SERIAL_HISTORIES) ~= "table" then
		_G.KV.SERIAL_HISTORIES = {}
	end
	
	table.insert( _G.KV.SERIAL_HISTORIES, {
		["date"] = date, ["serial"] = serial, ["comment"] = comment
	} )
	
	_G.KV.SERIAL_NO = serial or "BADBAD0000"
	return SaveSerialConfig()
end
--------

function _M.AddSerialNoLocal(date, serial, mac)
	if type(_G.KV.SERIAL_HISTORIES) ~= "table" then
		_G.KV.SERIAL_HISTORIES = {}
	end

	local comment = ""
	if #_G.KV.SERIAL_HISTORIES == 0 then
		date = "Factory Init"
		comment = "SYSTEM"
	end

	table.insert( _G.KV.SERIAL_HISTORIES, {
		["date"] = date, ["serial"] = serial, ["comment"] = comment
	} )

	if mac and mac ~= "" and mac:match("%w%w:%w%w:%w%w:%w%w:%w%w:%w%w") then
		local f = io.open("/etc/DEFAULT_MAC", "w")
		if f then
			f:write(mac)
			f:close()
			os.execute( "/sbin/ifconfig eth0 hw ether " .. mac )
		end
	end
	
	_G.KV.SERIAL_NO = serial or "BADBAD0000"
	return SaveSerialConfig()
end

--------
function _M.AddLanguageSupport( date, langs, def_lang, comment )
	if type(_G.KV.LANGUAGE_CONFIGS) ~= "table" then
		_G.KV.LANGUAGE_CONFIGS = {}
	end
	
	_G.KV.LANGUAGE_SUPPORT = {
		["date"] = date, ["langs"] = langs, ["def_lang"] = def_lang, ["comment"] = comment
	}

	table.insert( _G.KV.LANGUAGE_CONFIGS, _G.KV.LANGUAGE_SUPPORT )
	
	return SaveSerialConfig()
end

return _M
