local _M = {}

function _M.ipToU32(ip)
	local t1,t2,t3,t4 = ip:match("(%d+)%.(%d+)%.(%d+)%.(%d+)")
	if t1 and t2 and t3 and t4 then
		t1=tonumber(t1)
		t2=tonumber(t2)
		t3=tonumber(t3)
		t4=tonumber(t4)

		--local l = ((t4<<24)&0xff000000) | ((t3 << 16)&0x00ff0000) | ((t2<<8)&0x0000f00 | t1 & 0xff )
		--print(l,string.format("%08x",l))
		local l = string.format( "0x%02x%02x%02x%02x", t4,t3,t2,t1)
		return tonumber(l)
	else
		return 0
	end
end

function _M.u32ToIp(val)
	local t1,t2,t3,t4
	t1 = (val>>24)&0xff
	t2 = (val>>16)&0xff
	t3 = (val>>8)&0xff
	t4 = val&0xff
	return string.format("%d.%d.%d.%d", t4,t3,t2,t1)
end

function _M.ip4PrefixToMask(pfx)
	--The general using.
	if pfx == 0 then
		return "0.0.0.0"
	elseif pfx == 8 then
		return "255.0.0.0"
	elseif pfx == 16 then
		return "255.255.0.0"
	elseif pfx == 24 then
		return "255.255.255.0"
	elseif pfx == 32 then
		return "255.255.255.255"
	else
		local val = 0xffffffff
		val = (val << (32-pfx)) & 0xffffffff
		local rslt = tostring( (val >> 24) & 0xff )
		rslt = rslt .. "." .. tostring( (val >> 16) & 0xff )
		rslt = rslt .. "." .. tostring( (val >> 8 ) & 0xff )
		rslt = rslt .. "." .. tostring( val & 0xff )
		return rslt
	end
end

local function maskCheckByte(b)
	local cc
	local r = 0
	for cc = 1, 8, 1 do
		if (b & 0x80) > 0 then
			r = r + 1
		else
			return r
		end
		b = b << 1
	end
	return r
end

function _M.ip4MaskToPrefix(mask)
	if not mask or mask == "" then
		return 0
	else
		local t1,t2,t3,t4 = mask:match("(%d+)%.(%d+)%.(%d+)%.(%d+)")
		local r = 0
		t1 = tonumber(t1)
		t2 = tonumber(t2)
		t3 = tonumber(t3)
		t4 = tonumber(t4)

		if t1 == 255 then
			r = 8
			if t2 == 255 then
				r = r + 8
				if t3 == 255 then
					r = r + 8 + maskCheckByte(t4)
				else
					r = r + maskCheckByte(t3)
				end
			else
				r = r + maskCheckByte(t2)
			end
		else
			r = r + maskCheckByte(t1)
		end

		return r
	end
end

function _M.MACToArray(mac)
	if not mac or mac == "" then
		return {}
	else
		local r = { mac:match("(%x+):(%x+):(%x+):(%x+):(%x+):(%x+)") }
		local result = {}
		local i
		for i=1,#r,1 do
			--print(r[i])
			result[i] = tonumber("0x"..r[i])
		end
		return result
	end
end

return _M


