--[==[

The super-table module.
This super-table is a high-flex data table. Generally, it can be used for configuration, for memory caching, ...
This is a part of Co-Application Framework (CoAF).

(C) KILOVIEW Electronics Co., Ltd.

]==]

---------------------------------
local _M = {
}
---------------------------------
local cfg_meta_table = {
}
---------------------------------

-- Internal using, 
-- bind a table with declaration (Also optional specify the parent and name)

local function BindSuperTable(tb,decl,parent,name)
	if type(decl) ~= "table" or (decl.type ~= "OBJ" and decl.type ~= "ARR" and decl.type ~= "SET") or type(decl.default_decl) ~= "table" then
		decl = nil
	end
--[[
	if parent then
		print( "Bind:", (parent.__proto.name or "*") .. "." .. (name or "?") )
	else
		print( "Bind:", name or "*")
	end
]]
	local r = setmetatable({ __proto = {
		parent = parent,
		childs = {},
		name = name or "*",
		orig = type(tb) == "table" and tb or {},
		decl = decl,
		changes = {}
	}, __evals = {} }, cfg_meta_table)

	if type(parent) ~= "table" then
		r.__proto.root = r --point to self.
	else
		r.__proto.root = parent.__proto.root
	end

	r.TAB_BIND(r) --Bind(re-bind)
	return r
end

------------------------------------------------------
-- sup_table:TAB_DECLARE(decl)
--
-- Declare/Re-declare the table, the declaration rules
-- can be found in 'declare.lua'. 
------------------------------------------------------

function cfg_meta_table.TAB_DECLARE(cfg,decl)
	if type(decl) ~= "table" or (decl.type ~= "OBJ" and decl.type ~= "ARR" and decl.type ~= "SET") or type(decl.default_decl) ~= "table" then
		decl = nil
	end
	local cfg_proto = cfg.__proto
	cfg_proto.decl = decl
	cfg.TAB_BIND(cfg) --Bind(re-bind)
	return cfg
end

-------------------------------------------------------
-- sup_table:TAB_BIND(dataTable)
--
-- Bind/Re-bind a data table, to the current supper table.
--
-- NOTE: 	
-- If new dataTable specified, all the current data should 
-- be lost. dataTable can be 'nil', that's, if you specified 
-- new declaration, to re-bind the table according to the 
-- new declaration.
-------------------------------------------------------

function cfg_meta_table.TAB_BIND(cfg,dataTable)
	local cfg_proto = cfg.__proto
	if type(dataTable) == "table" then
		cfg_proto.orig = dataTable
	end

	local k,v
	for k,v in pairs(cfg_proto.childs) do
		cfg_proto.childs[k] = nil
	end

	--Reset changes.	
	cfg_proto.changes = {}

	if type(cfg_proto.orig) == "table" and cfg_proto.decl then
		local orig = cfg_proto.orig
		local decl = cfg_proto.decl

		if decl.type == "OBJ" then
			for k,v in pairs(decl.default_decl) do
				if v.type == "OBJ" or v.type == "ARR" or v.type == "SET" then
					if not orig[k] then
						orig[k] = {}
					end
					cfg_proto.childs[k] = BindSuperTable( orig[k], v, cfg, k )
				end
			end
		end	

		for k,v in pairs(orig) do
			local item_decl
			if decl.type == "OBJ" then
				item_decl = decl.default_decl[k]
			else
				item_decl = decl.default_decl
			end
			if item_decl and not cfg_proto.childs[k] and (item_decl.type == "OBJ" or item_decl.type == "ARR" or item_decl.type == "SET") then
				local subcfg = BindSuperTable(v, item_decl,cfg,k)
				cfg_proto.childs[k] = subcfg
			end
		end
	end

	return cfg
end


-------------------------------------------------------
-- sup_table:__TRIGGER( resetChanges )
--
-- To trigger all change events.
--
-- NOTE:
-- 	
-- You *MUST* call __TRIGGER() to trigger changes event, 
-- that's changed() callback declaration.
--
-- 'resetChanges' be true, it will reset all change flags;
-- otherwise, the change flags will not be cleared. 
-- So next time call __TRIGGER, the change events will be 
-- triggered again!
-------------------------------------------------------
function cfg_meta_table.__TRIGGER(cfg, resetChanges)
	local cfg_proto = cfg.__proto
	local done = false
	local k,v
	for k,v in pairs(cfg_proto.childs) do
		done = v.__TRIGGER(v, resetChanges) or done
	end

	if cfg_proto.decl then
		local decl = cfg_proto.decl
		for k, v in pairs(cfg_proto.changes) do
			done = true
			local item_decl
			if decl.type == "OBJ" then
				item_decl = decl.default_decl[k]
			else
				item_decl = decl.default_decl
			end
			if item_decl and type(item_decl.changed) == "function" then
				item_decl.changed(cfg, k, v)
			end
		end

		if done and type(decl.changed) == "function" then
			decl.changed(cfg, cfg_proto.name, "changed")
		end
	end

	if resetChanges then
		cfg_proto.changes = {}
	end
	return done
end

-------------------------------------------------------
-- sup_table:TAB_STORAGE( backend, options )
--
-- Set the storage options.
--   backend:  The storage backend, now only support 'file'
--   options:  The storage options (table), for 'file' 
--             backend, just set field 'file' to the
--             storage file name.
-------------------------------------------------------

function cfg_meta_table.TAB_STORAGE(cfg, backend, opts)
	local cfg_proto = cfg.__proto
	--TODO: support more storage backends (now only consider save to file)
	cfg_proto.store_backend = "file"
	cfg_proto.store_opts = opts
	return cfg
end

-------------------------------------------------------
-- sup_table:TAB_CHANGED()
--
-- Check the current changed status. return true or false.
-------------------------------------------------------

function cfg_meta_table.TAB_CHANGED(cfg)
	local cfg_proto = cfg.__proto
	local k,v
	if type(cfg_proto.changes) == "table" then
		for k,v in pairs(cfg_proto.changes) do
			return true
		end
	end

	for k,v in pairs(cfg_proto.childs) do
		if v.TAB_CHANGED(v) then
			return true
		end
	end
	return false
end

-------------------------------------------------------
-- sup_table:TAB_RESET_CHANGE()
--
-- Reset(clear) all change flags.
-------------------------------------------------------
function cfg_meta_table.TAB_RESET_CHANGE(cfg)
	local cfg_proto = cfg.__proto
	cfg_proto.changes = {}
	if cfg_proto.parent then
		cfg_proto.parent.__proto.changes[cfg_proto.name] = nil --Also remove parent record, if exist.
	end
	for k,v in pairs(cfg_proto.childs) do
		v.TAB_RESET_CHANGE(v)
	end
	return cfg
end

-------------------------------------------------------
-- sup_table:TAB_LOAD()
--
-- Load from storage.
-------------------------------------------------------

function cfg_meta_table.TAB_LOAD(cfg)
	local cfg_proto = cfg.__proto
	if cfg_proto.store_opts and cfg_proto.store_opts.file then
		local file = io.open( cfg_proto.store_opts.file, "r" )
		if not file then
			return cfg, false
		end
		local trunk = file:read("*a")
		file:close()

		local r
		local func
		func,r = load( "return " .. trunk )
		if func then
			local d
			r, d = pcall(func)
			if r and d then
				cfg_meta_table.TAB_BIND(cfg, d)
				return cfg, true
			else
				return cfg, false
			end
		else
			return cfg, false
		end
	end
	return cfg, false
end

-------------------------------------------------------
-- sup_table:TAB_GET_SAVING()
--
-- As know as the declaration should declare fields and
-- default values, even though they are not exist in the
-- data table actually; Also, declaration can set 'nosave'
-- flag for the fields. So, TAB_GET_SAVING() returns
-- a special table of current supper table, it just contains
-- the data for storage.
-------------------------------------------------------
function cfg_meta_table.TAB_GET_SAVING(cfg)
	local cfg_proto = cfg.__proto
	if not cfg_proto.decl then
		return cfg_proto.orig
	elseif cfg_proto.decl.nosave then
		return {}	--Just return an empty object.
	else
		local r = {}
		local k,v
		for k,v in pairs(cfg_proto.orig) do
			if cfg_proto.childs[k] then
				r[k] = cfg_proto.childs[k]:TAB_GET_SAVING()
			elseif cfg_proto.decl.type == "OBJ" then
				local item_decl = cfg_proto.decl.default_decl[k]
				if not item_decl or not item_decl.nosave then
					r[k] = v
				end
			else
				r[k] = v
			end
		end
		return r
	end
end

-------------------------------------------------------
-- sup_table:TAB_SAVE()
--
-- Save the supper table to storage backend.
-------------------------------------------------------
function cfg_meta_table.TAB_SAVE(cfg)
	local cfg_proto = cfg.__proto
	if cfg_proto.store_opts and cfg_proto.store_opts.file then
		local _x, pretty = pcall(require,"pl.pretty")
		if not _x then
			print("ERROR: Fail to require 'pl' modules, for saving configs.")
			return cfg, false
		end

		local f = io.open(cfg_proto.store_opts.file, "w")
		if f then
			f:write("-- CoAF configuration file of '" .. (cfg_proto.name or "...") .. "' \n")
			f:write("-- saved @ " .. os.date("%Y-%m-%d %H:%M:%S") .. "\n" )
			f:write( pretty.write(cfg:TAB_GET_SAVING(),'\t') )
			f:close()
			return cfg, true
		else
			return cfg, false
		end
	end
	return cfg, false
end

-------------------------------------------------------
-- sup_table:P_GET(...)
--
-- To get supper table's fields / sub fields by specified
-- field(s) name.
--
-- *NOTE*: This method is designed for remote call(RPC)
-- only! Of course, you can call this in local, but more
-- effectively, you can access fields directly, like:
--    var = obj.field.subfields
--
--  0,1 or more argument(s) can be passed to it. For 
--  example:
--     t:P_GET() => return the table itself;
--     t:P_GET("field_name") => return t.field_name;
--     t:P_GET("field1","field2") => return t.field1, t.field2
--     t:P_GET("field.subitem") => return t.field.subitem
--     t:P_GET("[1]") => return t[1]
--     t:P_GET("[1].subitem") => return t[1].subitem
-------------------------------------------------------
function cfg_meta_table.P_GET(cfg,...)
	local args = {...}
	if #args == 0 then
		return cfg
	end

	local eval = "local _args={...}; local _this = _args[1]; return "
	local i,v
	for i,v in ipairs(args) do
		if i > 1 then eval = eval .. "," end
		if v:match("^_this%.") then
			eval = eval .. v
		elseif v:match("^%[") then
			eval = eval .. "_this" .. v
		else
			eval = eval .. "_this." .. v
		end
	end
	local call = load(eval)
	if not call then
		return nil
	else
		local rets = { pcall(call, cfg) }
		if not rets[1] then 
			return nil
		else
			table.remove( rets, 1 )
			return table.unpack(rets)
		end
	end
end

-------------------------------------------------------
-- sup_table:P_SET(key, value) or
-- sup_table:P_SET({key1 = value1, key2=value2, ... })
--
-- To set supper table's fields / sub-fields values.
--
-- *NOTE*: This method is designed for remote call(RPC)
-- only! Of course, you can call this in local, but more
-- effectively, you can access fields directly, like:
--    obj.field.subfields = value
-------------------------------------------------------

function cfg_meta_table.P_SET(cfg,key_or_set,value)
	local eval = "local _args={...}; local _this = _args[1]; local _values = _args[2]; "
	if type(key_or_set) == "table" then
		local eval_vals = {}
		local cnt = 0
		local thisArg
		local k,v 
		for k,v in pairs(key_or_set) do
			cnt = cnt + 1
			eqThisVal = " = _values[" .. cnt .. "];"
			table.insert(eval_vals, v)
			if k:match("^_this%.") then
				eval = eval .. k .. eqThisVal
			elseif k:match("^%[") then
				eval = eval .. "_this" .. k .. eqThisVal
			else
				eval = eval .. "_this." .. k .. eqThisVal
			end
		end
		value = eval_vals
	elseif type(key_or_set) == "string" then
		if key_or_set:match("^_this%.") then
			eval = eval .. key_or_set .. " = _values;"
		elseif key_or_set:match("^%[") then
			eval = eval .. "_this" .. key_or_set .. " = _values;"
		else
			eval = eval .. "_this." .. key_or_set .. " = _values;"
		end
	end

	eval = eval .. " return true; "

	local call = load(eval)
	if not call then
		return false
	else
		local status, r = pcall(call, cfg, value)
		if not status then
			return false
		else
			local errmsg
			status, errmsg = pcall( cfg.__TRIGGER, cfg, true )
			if not status then
				print( "** ERROR: trigger event with execution error:", errmsg )
			end
			return r
		end
	end
end

-------------------------------------------------------
-- sup_table:EVAL("lua-codes", arg1, arg2, ...)
--     or
-- sup_table:EVAL( pre-loaded-trunk-token, arg1, arg2, ...)
--
-- For *RPC* call, remote can pass a Lua code trunk to
-- the supper table object, eval it and return values.
--
-- The lua code trunk is similar toclassic database 'Store Procedure',
-- but it's written by Lua. In the trunk, some special variants
-- is declared:
--    _this : it represent current supper table itself;
--    _args : A array, it stored all arguments passed to
--            this call, they are {arg1,arg2,arg3,...}
--
-- For example:
--
-- case 1) return sub fields: 
--      return _this.field1, _this.field2
--
-- case 2) assign values:
--		_this.field1 = _args[1]
--		_this.field2 = _args[2]
--
-- case 3) more complex:
--		local k,v
--		for k,v in pairs(_this) do
--			if k:match(_args[1]) then
--				-- do something
--			end
--		end
--
-- A lua trunk can be pre-loaded, so you needn't load the
-- code trunk each time. Refer to EVAL_LOAD() to pre-load
-- trunk. EVAL_LOAD() will return a token, like 
-- 'eval:0x81010000'. Pass this token to EVAL(), so it
-- can invoke the pre-loaded trunk.

-- *NOTE*: This method is designed for remote call(RPC)
-- only! Of course, you can call this in local, but more
-- effectively, you can access fields directly.
-------------------------------------------------------

function cfg_meta_table.EVAL(cfg, trunk, ...)
	if trunk:match("^eval:") then	--This is a pre-loaded trunk
		local func = cfg.__evals[trunk]
		if not func then
			return "#eval-invalid#", "Invalid pre-loaded trunk"
		end
		local rets = { pcall(func, cfg, ...) }
		if not rets[1] then
			return "#eval-error#", rets[2] or "Fail to eval pre-loaded trunk"
		else
			cfg:__TRIGGER(true)
			table.remove(rets, 1)
			return table.unpack(rets)
		end
	else
		local evalStr = "return function(_this,...) local _args = {...}; " .. trunk .. " end"
		local func, err = load(evalStr)
		if not func then
			return "#eval-error#", err or "Fail to load trunk"
		end
		func = func()
		local rets = { pcall(func, cfg, ...) }
		if not rets[1] then
			return "#eval-error#", rets[2] or "Fail to eval trunk"
		else
			cfg:__TRIGGER(true)
			table.remove(rets, 1)
			return table.unpack(rets)
		end
	end
end

-------------------------------------------------------
-- sup_table:EVAL_LOAD("lua-codes")
--
-- Refer to 'EVAL()' to know how EVAL_LOAD() does.
--
-- A lua trunk can be pre-loaded, so you needn't load the
-- code trunk each time. EVAL_LOAD() will return a token, 
-- like 'eval:0x81010000'. Pass this token to EVAL(), so it
-- can invoke the pre-loaded trunk.

-- *NOTE*: This method is designed for remote call(RPC)
-- only! 
-------------------------------------------------------
function cfg_meta_table.EVAL_LOAD(cfg, trunk)
	local evalStr = "return function(_this,...) local _args = {...}; " .. trunk .. " end"
	local func, err = load(evalStr)
	if not func then
		return false, err or "Fail to load trunk"
	end
	func = func()
	local tok = string.format("%s", func)
	tok = "eval:" .. tok:match("function:%s*(.+)")
	cfg.__evals[tok] = func
	return tok
end

-------------------------------------------------------
-- sup_table:EVAL_REMOVE(pre-loaded-trunk-token)
--
-- Refer to 'EVAL()'/'EVAL_LOAD()' to know what's the
-- eval trunk.
--
-- This method using for removing pre-loaded trunk. 
-------------------------------------------------------
function cfg_meta_table.EVAL_REMOVE(cfg, trunk)
	if trunk and trunk:match("^eval:") then
		cfg.__evals[trunk] = nil
		collectgarbage()
		return true
	else
		return false, "Invalid eval ID"
	end
end

-------------------------------------------------------
-- sup_table:EVAL_EXIST(pre-loaded-trunk-token)
--
-- Refer to 'EVAL()'/'EVAL_LOAD()' to know what's the
-- eval trunk.
--
-- This method checks a pre-loaded trunk is exist or
-- not. 
-------------------------------------------------------
function cfg_meta_table.EVAL_EXIST(cfg, trunk)
	if trunk and trunk:match("^eval:") then
		if cfg.__evals[trunk] then
			return true
		else
			return false
		end
	else
		return false
	end
end

-------------------------------------------------------
-- Internal using.
-- Get supper table's sub field value, by specified key.
-------------------------------------------------------
function cfg_meta_table.__GETFIELD(cfg,key)
	if not key then
		return cfg
	else
		local obj = cfg[key]
		if obj then
			return obj 
		else
			local eval = "local _args={...}; local _this=_args[1]; return "
			if key:match("^_this%.") then
				eval = eval .. key
			elseif key:match("^%[") then
				eval = eval .. "_this" .. key
			else
				eval = eval .. "_this." .. key
			end
			local call = load(eval)
			if not call then
				return nil
			end
			local status, r = pcall(call, cfg)
			if not status then
				return nil
			else
				return r
			end
		end
	end
end

-------------------------------------------------------
-- sup_table:A_SIZE([key])

-- Get array size of specified sub field. If not specified
-- key, it returns array size of supper table itself.
--
-- Just designed for *RPC* using, but you can call it 
-- locally, if you want. But more effectively, you can
-- get the table array size directly by '#obj'.
-------------------------------------------------------
function cfg_meta_table.A_SIZE(cfg,key)
	local obj = cfg:__GETFIELD(key)
	if type(obj) == "table" then
		return #obj
	else
		return nil
	end
end

-------------------------------------------------------
-- sup_table:A_INSERT([pos,],value,[key])

-- Insert item into array of specified sub field. If not 
-- specified key, it inserts into supper table itself.
--
-- Using this like lua 'table.insert', but please NOTE:
-- For Lua 5.1/5.2, 'table.insert' does *NOT* check the
-- __index/__newindex meta functions, so this may get
-- wrong result for supper table.
-- 
-- Just designed for *RPC* using, but you can call it 
-- locally, if you want.
-------------------------------------------------------

-- NOTE:
--	A_INSERT(cfg, <any>)								=> self append.
--	A_INSERT(cfg, <number>, <any>)						=> self insert.
--	A_INSERT(cfg, <no-number>, "string")				=> sub append.
--	A_INSERT(cfg, <any>, <any>, "string")				=> sub insert.
--	A_INSERT(cfg, <any>, <any>, <no-string>)			=> (wrong) Assume self insert.

function cfg_meta_table.A_INSERT(cfg, pos, value, key)
	if not key then
		if not value then
			value = pos
			pos = nil
		elseif type(pos) ~= "number" then
			if type(value) == "string" then
				key = value
				value = pos
				pos = nil
			else
				pos = nil
			end
		--else //nothing.
		end
	else
		if type(key) ~= "string" then
			key = nil
		end

		if type(pos) ~= "number" then
			pos = nil
		end
	end

	local targetObj = cfg:__GETFIELD(key)
	if type(targetObj) ~= "table" then
		return false
	else
		local sz = #targetObj
		if type(pos) ~= "number" then
			pos = sz + 1
			targetObj[pos] = value
			return pos
		elseif pos < 1 or pos > sz+1 then
			return false
		else
			local idx
			for idx = sz+1, pos+1, -1 do
				targetObj[idx] = targetObj[idx-1]
			end
			targetObj[pos] = value
			return pos
		end
	end
end

-------------------------------------------------------
-- sup_table:A_REMOVE([pos][,key])

-- Remove item from array of specified sub field. If not 
-- specified key, it removes from supper table itself.
--
-- Using this like lua 'table.remove', but please NOTE:
-- For Lua 5.1/5.2, 'table.remove' does *NOT* check the
-- __index/__newindex meta functions, so this may get
-- wrong result for supper table.
-- 
-- Just designed for *RPC* using, but you can call it 
-- locally, if you want.
-------------------------------------------------------
function cfg_meta_table.A_REMOVE(cfg,pos,key)
	local targetObj = cfg:__GETFIELD(key)
	if type(targetObj) ~= "table" then
		return false
	else
		local sz = #targetObj
		if not pos then
			targetObj[sz] = nil
			return true
		elseif pos < 1 or pos > sz then
			return false
		else
			local idx
			for idx=pos, sz-1 do
				targetObj[idx] = targetObj[idx+1]
			end
			targetObj[sz] = nil
			return #targetObj
		end
	end
end

-------------------------------------------------------
-- sup_table:S_EXIST(item[,key])
--
-- Check 'item' is exist or not in specified sub SET. 
-- If not specified key, it checks from supper table itself.
--
-- 'SET' is a new concept since it is not in Lua concepts.
-- It's a lua table, but each item has same declaration.
-- 
-- Just designed for *RPC* using, but you can call it 
-- locally, if you want.
-------------------------------------------------------
function cfg_meta_table.S_EXIST(cfg,item,key)
	if not item then
		return false
	end
	local obj = cfg:__GETFIELD(key)
	if type(obj) ~= "table" then
		return false
	else
		return obj[item] and true or false
	end
end

-------------------------------------------------------
-- sup_table:S_COUNT([key])
--
-- Get items count in specified sub SET. If not specified 
-- key, it checks from supper table itself.
--
-- 'SET' is a new concept since it is not in Lua concepts.
-- It's a lua table, but each item has same declaration.
-- 
-- Just designed for *RPC* using, but you can call it 
-- locally, if you want.
-------------------------------------------------------
function cfg_meta_table.S_COUNT(cfg,key)
	local obj = cfg:__GETFIELD(key)
	if type(obj) ~= "table" then
		return false
	else
		local k,v,cnt
		cnt = 0
		for k,v in pairs(obj) do
			cnt = cnt + 1
		end
		return cnt
	end
end

-------------------------------------------------------
-- sup_table:S_KEYS([key])
--
-- Get item keys in specified sub SET. If not specified 
-- key, it get all keys from supper table itself.
--
-- returns an array contains all keys.
--
-- 'SET' is a new concept since it is not in Lua concepts.
-- It's a lua table, but each item has same declaration.
-- 
-- Just designed for *RPC* using, but you can call it 
-- locally, if you want.
-------------------------------------------------------
function cfg_meta_table.S_KEYS(cfg,key)
	local obj = cfg:__GETFIELD(key)
	if type(obj) ~= "table" then
		return nil
	else
		local k,v
		local rslt = {}
		for k,v in pairs(obj) do
			table.insert( rslt, k )
		end
		return rslt
	end
end


-------------------------------------------------------
-- sup_table:S_ADD(item[,value][,key])
--
-- Add a new item into specified sub SET. If not specified 
-- key, it adds to supper table itself.
--
-- 'item' is the item name, and 'value' is the item value. 
--
-- 'SET' is a new concept since it is not in Lua concepts.
-- It's a lua table, but each item has same declaration.
-- 
-- Just designed for *RPC* using, but you can call it 
-- locally, if you want.
-------------------------------------------------------
function cfg_meta_table.S_ADD(cfg,item,value,key)
	if not item then
		return false, "invalid"
	end
	local obj = cfg:__GETFIELD(key)
	if type(obj) ~= "table" then
		return false, "invalid"
	end
	if not obj[item] then
		obj[item] = value
		return true
	else
		return false, "exist"
	end
end

-------------------------------------------------------
-- sup_table:S_REMOVE(item[,key])
--
-- Remove a item from specified sub SET. If not specified 
-- key, it removes from supper table itself.
--
-- 'item' is the item name. 
--
-- 'SET' is a new concept since it is not in Lua concepts.
-- It's a lua table, but each item has same declaration.
-- 
-- Just designed for *RPC* using, but you can call it 
-- locally, if you want.
-------------------------------------------------------
function cfg_meta_table.S_REMOVE(cfg,item,key)
	if not item then
		return false, "invalid"
	end
	local obj = cfg:__GETFIELD(key)
	if type(obj) ~= "table" then
		return false, "invalid"
	end
	obj[item] = nil
	return true
end

-------------------------------------------------------
-- Meta function: __index
-------------------------------------------------------
function cfg_meta_table.__index(cfg, key)
	local cfg_proto = cfg.__proto
	local r = cfg_meta_table[key]
	if r then -- If it's a meta method, return it directly.
		return r
	end
	local orig_table = cfg_proto.orig
	local thisIsSet = cfg_proto.decl and ( cfg_proto.decl.type == "ARR" or cfg_proto.decl.type == "SET" ) or false

	local decl = cfg_proto.decl and cfg_proto.decl.default_decl or nil

	local item = orig_table[key]
	local item_decl
	if decl then
		if thisIsSet then --Is that a dynamic set? (Array or Set)
			item_decl = decl
		else
			item_decl = decl[key]
		end
	end

	if type(item) == "nil" then
		if thisIsSet then --For sets
			return nil
		elseif item_decl and item_decl.type ~= "OBJ" and item_decl.type ~= "ARR" and item_decl.type ~= "SET" then
			item = item_decl.default_decl
		end
	end

	if item_decl then
		if type(item_decl.get) == "function" then
			return item_decl.get(cfg, key, item)	-- Aka. get callback: (current_config_obj, key, reference value)
		elseif item_decl.get == "disable" then
			return nil
		elseif item_decl.type == "OBJ" or item_decl.type == "ARR" or item_decl.type == "SET" then
			if not cfg_proto.childs[key] then
				return nil
			else
				return cfg_proto.childs[key]
			end
		end
	end

	return item
end

-------------------------------------------------------
-- Meta function: __newindex
-------------------------------------------------------
function cfg_meta_table.__newindex(cfg, key, val)
	local cfg_proto = cfg.__proto
	local r = cfg_meta_table[key]
	if r then return end	--Can't change the meta functions.

	local orig_table = cfg_proto.orig
	local thisIsSet = cfg_proto.decl and ( cfg_proto.decl.type == "ARR" or cfg_proto.decl.type == "SET" ) or false
	local decl = cfg_proto.decl and cfg_proto.decl.default_decl or nil

	local item_decl
	if decl then
		if thisIsSet then --Is that a dynamic set? (Array or Set)
			item_decl = decl
		else
			item_decl = decl[key]
		end
	end

	if item_decl then
		if type(item_decl.set) == "function" then
			local retVal
			retVal, val = item_decl.set(cfg, key, val, orig_table[key])	-- Aka. set callback: (current_config_obj, key, new_value, reference_old_value)
																		-- returns:  return-value flag[true/false], value. If flag = false, no return value.
			if not retVal then
				return 
			end
		elseif item_decl.set == "disable" then
			return
		end

		if type(item_decl.convert) == "function" then
			val = item_decl.convert(cfg,val)
		end
	end

	if val ~= orig_table[key] then
		if item_decl and (item_decl.type == "OBJ" or item_decl.type == "ARR" or item_decl.type == "SET") then
			if type(val) ~= "table" then
				orig_table[key] = nil
				cfg_proto.changes[key] = "loss"
				cfg_proto.childs[key] = nil
				return
			elseif getmetatable(val) == cfg_meta_table then
				orig_table[key] = val.__proto.orig
				cfg_proto.changes[key] = "move"
				cfg_proto.childs[key] = val  --This case, should be array moving or configuration object assigning directly.
				return
			end

			if cfg_proto.childs[key] then
				orig_table[key] = val
				cfg_proto.childs[key] = BindSuperTable(orig_table[key], item_decl, cfg, key)
				cfg_proto.changes[key] = "replace"
			else
				orig_table[key] = val
				cfg_proto.childs[key] = BindSuperTable(orig_table[key], item_decl, cfg, key)
				cfg_proto.changes[key] = "new"
			end
		else
			cfg_proto.changes[key] = "set"
			orig_table[key] = val
		end
	end
end

-------------------------------------------------------
-- Meta function: __len (to get array size)
-------------------------------------------------------
function cfg_meta_table.__len(cfg)
	local cfg_proto = cfg.__proto
	if type(cfg_proto.orig) ~= "table" then
		return 0
	else
		return #cfg_proto.orig
	end
end

-------------------------------------------------------
-- Meta function: __pairs (for key,value in pairs(...) do)
-------------------------------------------------------
function cfg_meta_table.__pairs(cfg)
	local cfg_proto = cfg.__proto
	if not cfg_proto.decl then
		local iterator = pairs(cfg_proto.orig)
		return function(tb,key)
			return iterator(cfg_proto.orig, key)
		end
	end

	local all_items = {}
	local k,v
	if cfg_proto.decl.type == "ARR" or cfg_proto.decl.type == "SET" then
		for k, v in pairs(cfg_proto.orig) do
			all_items[k] = cfg[k] --So can trigger the events, or get default values.
		end
	elseif cfg_proto.decl.type == "OBJ" then
		for k,v in pairs(cfg_proto.decl.default_decl) do
			all_items[k] = cfg[k]
		end
		for k,v in pairs(cfg_proto.orig) do 
			if not all_items[k] then
				all_items[k] = v
			end
		end
	end

	local iterator = pairs(all_items)
	return function(tb,key)
		return iterator(all_items,key)
	end
end

-------------------------------------------------------
-- Meta function: __ipairs (for key,value in ipairs(...) do)
-------------------------------------------------------
function cfg_meta_table.__ipairs(cfg)
	local cfg_proto = cfg.__proto
	if not cfg_proto.decl then
		local iterator = ipairs(cfg_proto.orig)
		return function(tb,idx)
			return iterator(cfg_proto.orig, idx or 0)
		end
	end

	local all_items = {}
	local k,v
	for k, v in ipairs(cfg_proto.orig) do
		all_items[k] = cfg[k] --So can trigger the events, or get default values.
	end

	local iterator = ipairs(all_items)
	return function(tb,idx)
		return iterator(all_items,idx or 0)
	end
end

-------------------------------------------------------
-- Main routine:
--      SUPER_TABLE.Create(dataTable, declaration, parent)
--
-- Create a supper table, by specify 'dataTable', and
-- 'declaration', and an optional 'parent' object.
--
-- Of course, the 'dataTable' and 'declaration' can also
-- be nil, so that it creates a empty supper table.
-- You can call TAB_BIND(dataTable) and TAB_DECLARE(decl)
-- laster to bind data table, and set declaration. 
-------------------------------------------------------
function _M.Create(dataTable, templateDecl, parent) --Parent should be nil, for general purpose.
	return BindSuperTable(dataTable, templateDecl, parent)
end

function _M.IsSuperTable(obj)
	return getmetatable(obj) == cfg_meta_table
end
-------------------------------------------------------

return _M

